// NewsMLConformance.java - demo to normalize a NewsML document.

import java.io.IOException;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLFactory;

import org.newsml.toolkit.conformance.ErrorVisitor;
import org.newsml.toolkit.conformance.NewsMLTestManager;

import org.newsml.toolkit.dom.DOMNewsMLFactory;


/**
 * Demo application to validate a NewsML document.
 *
 * TODO: report XML conformance errors consistently
 */
public class NewsMLConformance
{

    public static void showUsage ()
    {
	System.err.println("NewsMLConformance - validate a NewsML document");
	System.err.println("Usage:");
	System.err.println("  java NewsMLConformance [options] <URL...>");
	System.err.println("Options:");
	System.err.println("  -v, --verbose       Verbose output.");
	System.err.println("  -x, --external      Allow external URL access.");
	System.err.println("  -h, --help          Show this help.");
    }

    public static void main (String args[])
	throws Exception
    {
	boolean verbose = false;
	boolean useExternal = false;
	int startArg = 0;

				// Process command-line options

	while (startArg < args.length) {
	    if (args[startArg].equals("-v") ||
		args[startArg].equals("--verbose")) {
		verbose = true;
	    }
	    else if (args[startArg].equals("-h") ||
		     args[startArg].equals("--help")) {
		showUsage();
		System.exit(0);
	    }
	    else if (args[startArg].equals("-x") ||
		     args[startArg].equals("--external")) {
		useExternal = true;
	    }
	    else if (args[startArg].equals("--")) {
		startArg++;
		break;
	    }
	    else if (args[startArg].startsWith("-")) {
		System.err.println("Unrecognized option: " + args[startArg]);
		showUsage();
		System.exit(2);
	    } else {
		break;
	    }
	    startArg++;
	}

	if (startArg == args.length) {
	    System.err.println("No URLs specified!");
	    showUsage();
	    System.exit(2);
	}
	

	NewsMLTestManager manager = new NewsMLTestManager();
	manager.addDefaultTests();
	manager.setVerbose(verbose);
	manager.setErrorVisitor(new ErrorVisitor() {
		public void warning (BaseNode node, NewsMLException e)
		{
		    System.out.println("  NewsML Warning: " + e.getMessage()
				       + "\n    at " + node.getXPath());
		    warnings++;
		    totalWarnings++;
		}
		public void error (BaseNode node, NewsMLException e)
		{
		    System.out.println("  NewsML Error: " + e.getMessage()
				       + "\n    at " + node.getXPath());
		    errors++;
		    totalErrors++;
		}
	    });

	if (verbose) {
	    System.out.println("Verbosity enabled, external references " +
			       (useExternal ? "enabled" : "disabled"));
	}

	NewsMLFactory factory = new DOMNewsMLFactory();
	factory.setValidation(true);
	for (int i = startArg; i < args.length; i++) {
	    warnings = 0;
	    errors = 0;
	    System.out.println("\nPerforming XML validation for " + args[i]);
	    NewsML newsml = null;
	    try {
		newsml = factory.createNewsML(args[i]);
	    } catch (IOException e) {
		System.out.println("  XML Error: " + e.getMessage());
		System.out.println("Processing for " + args[i] + "aborted.");
		totalErrors++;
		continue;
	    }
	    System.out.println("Testing NewsML conformance for " + args[i]);
	    manager.runTests(newsml, useExternal);
	    System.out.println("Finished testing " + args[i] + ": "
			       + errors + " error(s), "
			       + warnings + " warning(s)");
	}
	System.out.println("\nFinished: " + totalErrors + " total error(s), "
			   + totalWarnings + " total warning(s)");
    }

    static int warnings = 0;
    static int totalWarnings = 0;
    static int errors = 0;
    static int totalErrors = 0;

}

// end of NewsMLConformance.java
