// AdministrativeMetadata - administrative information for a news component.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: AdministrativeMetadata.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Information about the provenance of a NewsComponent or NewsItem.
 *
 * <p>The administrative information includes the provider, creator,
 * and contributors to the component, among other information.</p>
 *
 * <p>The NewsML specification is not always clear about the
 * applicability of this information: it states that the information
 * applies either to the parent news component <em>or</em> to an
 * ancestor news item, with no clear rules for distinguishing the two
 * except what can be inferred from documentation for the individual
 * properties; users are advised to use great care with the
 * information in this interface, until the IPTC clarifies the
 * rules.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface AdministrativeMetadata
    extends BaseNode, CatalogNode, IdNode, PropertyNode
{

    /**
     * Get the suggested file name for the news item.
     *
     * <p>The NewsML documentation implies that this property applies
     * the ancestor news item rather than to the parent news
     * component.</p>
     *
     * @return The suggested file name as a text node.
     */
    public IdText getFileName ();


    /**
     * Get the system identifier of the news item.
     *
     * <p>The NewsML documentation implies that this property applies
     * to the ancestor news item rather than to the parent news
     * component.</p>
     *
     * <p>A system identifier is simply a relative or absolute URI
     * without a '#' part.  The NewsML specification does not state
     * the purpose of this identifier: it may be intended to provide a
     * URL where a normative version of the news item may be found for
     * comparison to the one transmitted in the NewsML package.</p>
     *
     * @return The system identifier as a text node.
     */
    public IdText getSystemIdentifier ();


    /**
     * Get a list of providers of a news object.
     *
     * <p>The NewsML documentation does not state whether this
     * property applies to the parent news component or to the
     * ancestor news item.</p>
     *
     * @return A list of parties representing the providers, or null
     * if the NewsML document did not list any.
     */
    public PartyList getProvider ();


    /**
     * Get a list of creators of a news object.
     *
     * <p>The NewsML documentation does not state whether this
     * property applies to the parent news component or to the
     * ancestor news item.</p>
     *
     * @return A list of parties representing the creators, or null
     * if the NewsML document did not provide any.
     */
    public PartyList getCreator ();


    /**
     * Get the number of source lists provided.
     *
     * @return The number of source lists.
     */
    public int getSourceCount ();


    /**
     * Get a list of parties providing source material for a news object.
     *
     * <p>The NewsML documentation does not state whether this
     * property applies to the parent news component or to the
     * ancestor news item.</p>
     *
     * @param index The index of the source list, zero-based, numbered
     * sequentially.
     * @return A list of parties, or if the NewsML document did not
     * include any at the specified index.
     * @see #getSourceCount
     */
    public SourceList getSource (int index);


    /**
     * Get an array of source party lists in this object.
     *
     * @return A (possibly-empty) array of source party lists.
     */
    public SourceList [] getSource ();


    /**
     * Get the number of contributor lists.
     *
     * @return The number of contributor lists.
     */
    public int getContributorCount ();


    /**
     * Get a list of contributors to a news object.
     *
     * <p>The NewsML documentation does not state whether this
     * property applies to the parent news component or to the
     * ancestor news item.</p>
     *
     * @param index The index of the contributors list, zero-based, in
     * document order.
     * @return A list of parties, or null if the NewsML document did
     * not include any at the specified index.
     * @see #getContributorCount
     */
    public PartyList getContributor (int index);


    /**
     * Get an array of contributor party lists in this object.
     *
     * @return A (possibly-empty) array of contributor party lists.
     */
    public PartyList [] getContributor ();

}

// end of AdministrativeMetadata.java
