// BaseNode.java - base interface for all NewsML objects.
// Copyright (c) 2001 by Reuters PLC
// $Id: BaseNode.java,v 1.17 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;
import java.io.Writer;


/**
 * Base interface for all NewsML objects.
 *
 * <p>The base interface provides basic services for navigation and
 * serialization, as well as access to the {@link NewsMLSession},
 * which allows the client application to execute queries against
 * the NewsML object tree.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface BaseNode
{

    /**
     * Get the XML element name associated with the node.
     *
     * @return The XML element name associated with the node, or
     * null if there is none.
     */
    public String getXMLName ();


    /**
     * Get the XPath from the document root to this node.
     *
     * @return an XPath to this node.
     */
    public String getXPath ();


    /**
     * Get the NewsML session associated with the package.
     *
     * @return The session (not null).
     */
    public NewsMLSession getSession ();


    /**
     * Get the parent of the node in the NewsML content tree.
     *
     * @return The parent node, or null if this is the root.
     * @see #getChild
     */ 
    public BaseNode getParent ();


    /**
     * Count the children for this node.
     *
     * <p>See {@link #getChild(int)} for information on what is and is
     * not counted as a child here.</p>
     *
     * @return The number of children.
     * @see #getChildCount(String)
     */
    public int getChildCount ();


    /**
     * Count the children with the specified name.
     *
     * <p>See {@link #getChild(int)} for information on what
     * is and is not counted as a child here.</p>
     *
     * @param xmlName The XML name of the children to count.
     * @return The number of children with the XML name specified.
     * @see #getXMLName
     * @see #getChildCount()
     */
    public int getChildCount (String xmlName);


    /**
     * Get a child of the node by position.
     *
     * <p>This method uses the XML notion of parent/child, not the
     * NewsML notion.  That difference can matter quite a bit; for
     * example, while the get* methods for information in subelements
     * and information in attributes are indistinguishable in the
     * NewsML Toolkit API, this method will deal only with the
     * element children.</p>
     *
     * <p>Some elements in the XML are not represented directly in the
     * NewsML toolkit; in that case, the element will be counted, but
     * invoking this method (or any other getChild method) will return
     * null.  Currently, the following XML elements do not map
     * directly to a NewsML Toolkit interface: Delete, InsertAfter,
     * InsertBefore, SizeInBytes (the toolkit treats it as an int),
     * and Update.</p>
     *
     * <p>Since the result of invoking this method can be
     * unpredictable in so many ways, it is recommend to use the named
     * {@link #getChild(String, int)} method whenever possible.</p>
     *
     * @param index The index of the child, zero-based, numbered
     * sequentially.
     * @return The node representing the specified child, or null
     * if none is available at the index provided.
     * @see #getParent
     * @see #getChild(String,int)
     */
    public BaseNode getChild (int index);


    /**
     * Get a child of the node by name and position.
     *
     * <p>See {@link #getChild(int)} for information on what
     * is and is not counted as a child here.</p>
     *
     * @param xmlName The XML name of the children to use.
     * @param index The index of the child, zero-based, numbered
     * sequentially, counting only children with the given XML name.
     * @return The node representing the specified child, or null
     * if none is available at the XML name and index provided.
     * @see #getChild(int)
     */
    public BaseNode getChild (String xmlName, int index);


    /**
     * Get an array of child nodes.
     *
     * <p>See {@link #getChild(int)} for information on what
     * is and is not counted as a child here.</p>
     *
     * @return A (possibly-empty) array of primary child nodes.
     */
    public BaseNode [] getChild ();


    /**
     * Get an array of child nodes with the specified XML name.
     *
     * <p>See {@link #getChild(int)} for information on what
     * is and is not counted as a child here.</p>
     *
     * @param xmlName The XML name of the child nodes to return.
     * @return A (possibly-empty) array of primary child nodes.
     */
    public BaseNode [] getChild (String xmlName);


    /**
     * Remove this node from the NewsML tree.
     *
     * <p>WARNING: removing a node may result in an invalid NewsML
     * tree.  Don't use this method unless you know what you're
     * doing.</p>
     *
     * <p>This method will work for nodes represented as XML elements,
     * but not for nodes represented as XML attributes.  It will also
     * fail for the root node of the NewsML tree (often, but not
     * always "NewsML") and will throw a NewsMLException for all
     * failures.</p>
     *
     * <p>If the implementation does not support modification, it may
     * also throw an exception.</p>
     *
     * @exception NewsMLException if the implementation cannot remove
     * this node for any reason.
     */
    public void removeSelf ()
	throws NewsMLException;


    /**
     * Get the path from the root node to this node.
     *
     * @return An array of nodes beginning with the root BaseNode
     * and ending with the current BaseNode.
     */
    public BaseNode[] getPath ();


    /**
     * Serialize this node as XML.
     *
     * <p>This method will create XML markup representing the current
     * node and all of its descendants.  It will not reproduct the
     * original XML markup character-for-character, but will create a
     * structurally-equivalent version.  The method works on the raw
     * XML level, and has no specialized knowledge of Namespaces
     * (yet).</p>
     *
     * @param output The character stream to which the XML should
     * be written (this method will <em>not</em> close the stream).
     * @param isDocument True if this node is to represent a complete
     * XML document (with DOCTYPE declaration and prolog), false
     * otherwise.
     * @exception IOException if there is an error writing the
     * document.
     * @see #toXML(boolean)
     */
    public void writeXML (Writer output, boolean isDocument)
	throws IOException;


    /**
     * Serialize a node as an XML document.
     *
     * <p>This method serializes a NewsML node like {@link
     * #writeXML(Writer,boolean)}, except that the node is
     * <em>always</em> made into a full XML docuument, and that the
     * provided character encoding and internal DTD subset will be
     * used in place of the defaults.</p>
     *
     * @param output The character stream to which the XML should
     * be written (this method will <em>not</em> close the stream).
     * @param encoding The encoding string to be used in the XML
     * declaration, or null to use the default.  Note that this will
     * not affect the encoding actually used by the Writer -- it is
     * the application's responsibility to ensure that the two
     * correspond.
     * @param internalSubset The internal DTD subset to use, the empty
     * string for an empty subset, or null to use the default.
     * @see #toXML(String,String)
     */
    public void writeXML (Writer output, String encoding,
			  String internalSubset)
	throws IOException;


    /**
     * Get an XML rendition of this node.
     *
     * <p>See {@link #writeXML(Writer,boolean)} for information on the
     * serialization.</p>
     *
     * @param isDocument True if this node is to represent a complete
     * XML document (with DOCTYPE declaration and prolog), false
     * otherwise.
     * @return A string containing an XML rendition of the node.
     * @see #writeXML(Writer,boolean)
     */
    public String toXML (boolean isDocument);



    /**
     * Get an XML document rendition of this node.
     *
     * <p>See {@link #writeXML(Writer,String,String)} for details.</p>
     *
     * @param encoding The encoding to include in the XML declaration,
     * or null to use the default.
     * @param internalSubset The internal DTD subset to use, the empty
     * string for an empty subset, or null to use the default.
     * @return A string containing an XML document.
     */
    public String toXML (String encoding, String internalSubset);
    
}

// end of BaseNode.java
