// Catalog.java - a resource catalog.
// Copyright (c) 2001 by Reuters PLC.
// $Id: Catalog.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;


/**
 * Resource catalog.
 *
 * <p>The resource catalog declares vocabularies and other information
 * that the rest of the document (or document subtree) can use.  The
 * NewsML Toolkit automatically resolves default schemes and
 * vocabularies, so many client applications can safely ignore this
 * interface.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface Catalog
    extends BaseNode, HrefNode, IdNode
{

    /**
     * Count the resources in the catalog.
     *
     * @return The number of resources in the catalog.
     */
    public int getResourceCount ();


    /**
     * Get a single resource in a catalog.
     *
     * @param index The index of the resource, zero-based, numbered
     * sequentially in document order.
     * @return The resource, or null if none was present with the
     * specified index.
     */
    public Resource getResource (int index);


    /**
     * Get an array of resources in this object.
     *
     * @return A (possibly-empty) array of resources
     */
    public Resource [] getResource ();


    /**
     * Count the topic use elements in the catalog.
     *
     * @return The number of topic use elements.
     */
    public int getTopicUseCount ();


    /**
     * Get topic use information from a catalog.
     *
     * @param index The index of the topic use element, zero-based,
     * numbered sequentially in document order.
     * @return The topic use information, or null if none was provided
     * at the specified index.
     */
    public TopicUse getTopicUse (int index);


    /**
     * Get all TopicUse children in an array.
     *
     * @return A (possibly-empty) array of TopicUse children.
     */
    public TopicUse [] getTopicUse ();


    /**
     * Retrieve the final catalog referenced.
     *
     * <p>Follow any Href attributes recursively to the final catalog
     * referenced.  If there is not an Href attribute, return a copy
     * of this catalog.  The useExternal parameter controls whether
     * this method will follow links outside the current NewsML
     * document.</p>
     *
     * <p>If the reference has no fragment identifier, the root
     * element of the document must be <var>Catalog</var>; otherwise,
     * the fragment identifier must be the Duid of a Catalog
     * element.</p>
     *
     * @param useExternal If false, do not attempt to retrieve any
     * referenced catalog from outside the current NewsML package.
     * @exception IOException if the external catalog cannot be processed.
     * @exception NewsMLException if the external document is loaded
     * successfully but the reference does not point to a Catalog.
     * @return The catalog referenced by the href attribute, if
     * available, or null otherwise.
     */
    public Catalog getReferencedCatalog (boolean useExternal)
	throws IOException;

}

// end of Catalog.java
