// ContentItem.java - leaf content in a NewsML package.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: ContentItem.java,v 1.10 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import org.w3c.dom.NodeList;


/**
 * News content for presentation to humans.
 *
 * <p>A content item represents a leaf node in the NewsML content
 * tree: it either contains or points to actual content, such as a
 * textual news story, a photo, or a video.  If the content item
 * contains the content, the content will be accessible through the
 * {@link #getDataContent} method; otherwise, the content's URL will
 * be available through the {@link #getHref} method.</p>
 *
 * <p>The {@link #getMediaType}, {@link #getFormat}, and {@link
 * #getNotation} methods all provide redundant information already
 * available through {@link #getMimeType}, and provide that
 * information in ways that will be less familiar to users; these
 * redundant methods have been included only for the sake of
 * completeness.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface ContentItem
    extends BaseNode, CatalogNode, CommentNode, EquivalentNode,
	    HrefNode, IdNode
{

    /**
     * Get the content's media type.
     *
     * <p>NewsML media type information is redundant, filling the same
     * function as the first part of the MIME type (see {@link
     * #getMimeType}).  Typical media types include Text, Audio,
     * Video, and Image.</p>
     *
     * @return A formal name representing the media type, or
     * null if no media type was specified.
     * @see #getMimeType
     */
    public FormalName getMediaType ();


    /**
     * Get the content's format.
     *
     * <p>NewsML format information is redundant, filling the same
     * function as the second part of the MIME type (see {@link
     * #getMimeType}).  Typical media types include ANPA 1.1beta2 and
     * NITF.</p>
     *
     * @return A formal name describing the format, or null if no
     * format was specified.
     * @see #getMimeType
     */
    public FormalName getFormat ();


    /**
     * Return the content's MIME type.
     *
     * <p>While this value is a formal name, and may be defined by any
     * vocabulary according to the NewsML document type, it would be a
     * very poor choice for a provider not to use the proper MIME-type
     * definition in IETF <a
     * href="http://www.ietf.org/rfc/rfc2045.txt">RFC 2045</a>.</p>
     *
     * @return A formal name describing the MIME type, or null
     * if no MIME type was specified.
     * @see #getCharacteristics
     */
    public FormalName getMimeType ();


    /**
     * Get the content's data type notation.
     *
     * <p>NewsML notation information is redundant, filling the same
     * role as the MIME type (see {@link #getMimeType}) but using a
     * less familiar representation.</p>
     *
     * @return A formal name describing the data type notation, or
     * null if no notation was specified.
     * @see #getMimeType
     */
    public FormalName getNotation ();


    /**
     * Get the physical characteristics of the news content.
     *
     * <p>Physical characteristics include the physical content's size
     * in bytes.  Note that information on media and format is
     * provided by the MIME type (see {@link #getMimeType}).</p>
     *
     * @return An object describing the physical characteristics,
     * or null if none was specified.
     * @see #getMimeType
     */
    public Characteristics getCharacteristics ();


    /**
     * Get an encoding step for the raw inline content.
     *
     * <p>The encoding step will eventually lead to {@link
     * DataContent}, possibly through one or more further encoding
     * steps.</p>
     *
     * <p>This method, {@link #getHref}, and {@link #getDataContent}
     * are mutually exclusive; only one may return a non-null
     * value.</p>
     *
     * @return The encoding step, or null if no intermediate encoding
     * steps were performed on the raw, inline data (or if the
     * data is referenced out of line).
     */
    public Encoding getEncoding ();


    /**
     * Get raw inline content.
     *
     * <p>This method, {@link #getHref}, and {@link #getEncoding} are
     * mutually exclusive; only one may return a non-null value.</p>
     *
     * @return The inline content, or null if an intermediate encoding
     * step was performed on the data (or if the data is referenced
     * out of line).
     */
    public DataContent getDataContent ();

}

// end of ContentItem.java
