// Delta.java - a delta for an incremental update.
// Copyright (c) 2001 by Reuters PLC
// $Id: Delta.java,v 1.3 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * A delta for an incremental update.
 *
 * <p>This interface represents a single change request that forms
 * part or all of an incremental update to an existing news item.  The
 * change request consists of a single target node (represented by a
 * Duid reference) and zero or more replacement nodes (accessible
 * through {@link BaseNode#getChild}), depending on the action.
 * Deltas are represented by the InsertBefore, InsertAfter, Replace,
 * and Delete elements in the NewsML XML markup.</p>
 *
 * <p>NOTE: The XML content model for all but Delete uses ANY, which
 * would also allow character data replacements; this was not likely
 * the intention of the IPTC, so this interface should silently
 * discards any top-level character data inside an InsertBefore,
 * InsertAfter, Replace, or Delete element.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface Delta
    extends BaseNode, IdNode
{


    /**
     * Constant: insert replacement node(s) before target node.
     */
    public final static int INSERT_BEFORE = 1;


    /**
     * Constant: insert replacement node(s) after target node.
     */
    public final static int INSERT_AFTER = 2;


    /**
     * Constant: replace target node with replacement node(s).
     */
    public final static int REPLACE = 3;


    /**
     * Constant: delete target node.
     */
    public final static int DELETE = 4;


    /**
     * Get the action requested by this delta.
     *
     * @return A constant representing the action requested.
     * @see #INSERT_BEFORE
     * @see #INSERT_AFTER
     * @see #REPLACE
     * @see #DELETE
     */
    public int getAction ();


    /**
     * Get the Duid of the target node.
     *
     * @return The Duid of the target node (not null).
     */
    public Text getTargetRef ();


    /**
     * Return the actual target node.
     *
     * <p>This is a convenience method that looks up the node 
     * corresponding to the Duid.</p>
     *
     * @return The target base node.
     * @see #getTargetRef
     */
    public BaseNode getTargetNode ();

}

// end of Delta.java
