// DescriptiveMetadata.java - information about a news component's content.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: DescriptiveMetadata.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Information about a news component's content.
 *
 * <p>Descriptive metadata provides information about the language,
 * genre, subject matter, and intended audience for news
 * information.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface DescriptiveMetadata
    extends AssignmentNode, BaseNode, CatalogNode, IdNode, PropertyNode
{

    /**
     * Count the languages present.
     *
     * @return The number of languages.
     */
    public int getLanguageCount ();


    /**
     * Get one of the languages of the news component.
     *
     * <p>It is unclear from NewsML documentation how this value is
     * meant to interact with the language property (the
     * <code>xml:lang</code> attribute) for the news component itself
     * (see {@link NewsComponent}), so programmers should use this
     * property with caution.</p>
     *
     * @param index The index of the language, zero-based, numbered
     * sequentially.
     * @return A formal name, with assignment information,
     * representing the language, or null if no language was provided
     * at the specified index.
     */
    public AssignedFormalName getLanguage (int index);


    /**
     * Get all Language children in an array.
     *
     * @return A (possibly-empty) array of Language children.
     */
    public AssignedFormalName [] getLanguage ();


    /**
     * Get the genre of the news component.
     *
     * @return A formal name representing the genre.
     */
    public AssignedFormalName getGenre ();


    /**
     * Count the subject codes present.
     *
     * @return The number of subject codes.
     */
    public int getSubjectCodeCount ();


    /**
     * Get a subject code for the news component.
     *
     * @param index The index of the subject code, zero-based, numbered
     * sequentially.
     * @return The subject code, or null if none was provided at the
     * specified index.
     */
    public SubjectCode getSubjectCode (int index);


    /**
     * Get all SubjectCode children in an array.
     *
     * @return A (possibly-empty) array of SubjectCode children.
     */
    public SubjectCode [] getSubjectCode ();


    /**
     * Count the OfInterestTo members.
     *
     * @return The number of OfInterestTo members.
     */
    public int getOfInterestToCount ();


    /**
     * Get an indication of the intended audience for a news component.
     *
     * @param index The index of the information, zero-based, numbered
     * sequentially.
     * @return Information about the intended audience..
     */
    public OfInterestTo getOfInterestTo (int index);


    /**
     * Get all OfInterestTo children in an array.
     *
     * @return A (possibly-empty) array of OfInterestTo children.
     */
    public OfInterestTo [] getOfInterestTo ();


    /**
     * Count the topic occurrences present.
     *
     * @return The number of topic occurrences.
     */
    public int getTopicOccurrenceCount ();


    /**
     * Get a topic associated with a news component.
     *
     * @param index The index of the information, zero-based, numbered
     * sequentially.
     * @return Text with assignment information attached.
     */
    public TopicOccurrence getTopicOccurrence (int index);


    /**
     * Get all TopicOccurrence children in an array.
     *
     * @return A (possibly-empty) array of TopicOccurrence children.
     */
    public TopicOccurrence [] getTopicOccurrence ();

}

// end of DescriptiveMetadata.java
