// FormalNameNode.java - a three-part NewsML formal name.
// Copyright (c) 2001 by Reuters PLC.
// $Id: FormalNameNode.java,v 1.11 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;


/**
 * Base interface for three-part NewsML names.
 *
 * <p>Formal names are the key to NewsML's extensibility, and bear
 * very strong similarities to XML Namespace-qualified names.  Each
 * formal name consists of three parts:</p>
 *
 * <ol>
 * <li>A pointer to a vocabulary including the name.  This property
 * is roughly equivalent to an XML Namespace, but it may also be used
 * to retrieve an actual document (or fragment of the current document)
 * describing the vocabulary.</li>
 * <li>The name of a scheme within the vocabulary (essentially, a
 * mechanism for dividing large vocabularies into sections when
 * needed.  This property has no analogue in XML Namespaces.</li>
 * <li>The name itself.  This property is roughly equivalent to
 * an XML local name in a Namespace-qualified name.</li>
 * </ol>
 *
 * <p>For full interoperability, applications must consider all three
 * parts of the formal name.</p>
 *
 * <p>Note that unlike many of the interfaces in this library, the
 * FormalNameNode interface does not correspond to a single XML
 * element type; instead, it is used to deliver formal-name
 * information present in many different types of XML elements.</p>
 *
 * <p>The vocabulary and scheme information for a formal name may
 * appear in the XML element itself, or it may be defaulted from
 * another element in the XML document or even from a separate
 * document.  To help avoid nasty surprises (such delays or failures
 * retrieving large external catalogs over HTTP), this interface has
 * two pairs of methods for retrieving the vocabulary and scheme:
 * {@link #getVocabulary} and (in the derived {@link FormalName} and
 * {@link NewsItemId} interfaces) {@link FormalName#getScheme} and
 * {@link NewsItemId#getScheme} retrieve only the vocabulary and
 * scheme explicitly provided in the XML element containing the formal
 * name, and thus work in constant time; {@link #getDefaultVocabulary}
 * and {@link #getDefaultScheme} look for {@link DefaultVocabularyFor}
 * declarations in higher-level {@link Catalog} objects within the
 * current document, and, if the useExternal parameter is set, in
 * external catalogs as well.  Please note that using this option
 * could be extremely slow, depending on the quality of your network
 * connection and the availability of the server hosting the
 * catalog.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface FormalNameNode
{

    /**
     * Get the local name.
     *
     * <p>The local name is the basic name, such as "length" or
     * "person"; it may be qualified by a vocabulary and, optionally,
     * a scheme.  In the XML markup, the local name is usually
     * specified by a FormalName attribute, but it occasionally
     * appears under a different attribute name (such as "HowPresent")
     * or even as character-data content; this interface hides the
     * inconsistency from the user.</p>
     *
     * @return Text containing the local part of the name.
     * @see #getVocabulary
     */
    public Text getName ();


    /**
     * Get an explicitly-specified vocabulary reference.
     *
     * <p>This method returns only a vocabulary that has been
     * explicitly provided in the XML.</p>
     *
     * @return Text containing a reference to the vocabulary, either a
     * URI or a local Duid reference beginning with '#'.
     * @see #getDefaultVocabulary
     */
    public Text getVocabulary ();


    /**
     * Get an explicit or defaulted vocabulary reference.
     *
     * <p>Get a pointer to the vocabulary for the formal name.  The
     * pointer may be a reference to an internal topic set (beginning
     * with '#'), or it may be a URL or URN.  If the vocabulary comes
     * from a resource with a DefaultVocabularyFor declaration, then
     * the following order of precedence applies:</p>
     *
     * <ol>
     * <li>If there is an internal Url for the resource (i.e. one
     * beginning with '#'), return it; if there is more than one, 
     * return the first one found.</li>
     * <li>Otherwise, if there is a Urn for the resource, return
     * it.</li>
     * <li>Otherwise, if there are any Urls at all for the resource
     * return the first one.</li>
     * <li>Otherwise, return null.</li>
     * </ol>
     *
     * <p>An explicit Vocabulary attribute always takes precedence
     * over a DefaultVocabularyFor declaration, however.</p>
     *
     * <p><strong>Note 1:</strong> the XPath for the
     * DefaultVocabularyFor declaration will be applied using the
     * parent of the containing Catalog as the context node (the
     * NewsML 1.0 functional specification fails to specify any
     * context node, so this is a best guess).  According to the XPath
     * specification, an XPath expression like "TopicType" is an
     * abbreviation of "child::TopicType", so it will fail unless
     * TopicType is a sibling of the Catalog.  Normally, it will be
     * best to write expressions like "descendant::TopicType" to avoid
     * ambiguity.</p>
     *
     * <p><strong>Note 2:</strong> XPath expressions may match the
     * FormalName attribute or the containing element.  If an
     * attribute other than FormalName holds the formal name (as is
     * the case with the %assignment; attributes Importance,
     * Confidence, and HowPresent) the XPath expression <em>must</em>
     * match the attribute itself to avoid ambiguity.</p>
     *
     * @param useExternal If true, follow links outside of the current
     * NewsML document during catalog resolution.
     * @return Text containing a pointer to the vocabulary for the
     * formal name, or null if none was provided.
     * @exception IOException If there is an error retrieving an
     * external resource.
     * @exception NewsMLException If there is an error retrieving an
     * ancestor Catalog (see {@link Catalog#getReferencedCatalog}).
     * @see #getName
     * @see #getVocabulary
     */
    public String getDefaultVocabulary (boolean useExternal)
	throws IOException;


    /**
     * Get an explicit or defaulted scheme.
     *
     * <p>See {@link #getDefaultVocabulary} for resolution
     * details.</p>
     *
     * @param useExternal If true, follow links outside of the current
     * document for Catalog resolution.
     * @return The scheme, or null if none was found.
     * @exception IOException If there is an error retrieving an
     * external resource.
     * @exception NewsMLException If there is an error retrieving an
     * ancestor Catalog (see {@link Catalog#getReferencedCatalog}).
     * @see FormalName#getScheme
     * @see NewsItemId#getScheme
     */
    public String getDefaultScheme (boolean useExternal)
	throws IOException;


    /**
     * Get the topic set for the formal name.
     *
     * <p>This method will attempt to look up a topic set
     * corresponding to the vocabulary.</p>
     *
     * @param useExternal If true, follow links outside of the current
     * document for topic set resolution.
     * @return The topic set, or null if none could be found.
     * @exception IOException If there is an error retrieving an
     * external resource.
     */
    public TopicSet getTopicSet (boolean useExternal)
	throws IOException;


    /**
     * Find the topic associated with this formal name.
     *
     * <p>Try to find a topic for this formal name, using the
     * vocabulary to locate a TopicSet, and then the name
     * and scheme to locate a Topic.  This method will follow
     * all TopicSetRefs within the limits of the useExternal
     * parameter, but will fail with NewsML URNs.</p>
     *
     * @param useExternal If true, follow links outside of the current
     * NewsML document during topic-set reference resolution.
     * @return The Topic associated with this FormalNameNode,
     * or null if none could be located.
     * @exception IOException If there is an error retrieving
     * an external resource.
     */
    public Topic findTopic (boolean useExternal)
	throws IOException;


    /**
     * Generate a naive string representation of this FormalNameNode.
     *
     * <p>The generated string will consist of the vocabulary URL in
     * braces, followed by the scheme in braces, followed by the local
     * name, as in</p>
     *
     * <pre>
     * {Vocabulary}{Scheme}LocalName
     * </pre>
     *
     * <p>If the vocabulary or scheme is absent, it will be
     * represented by an empty pair of braces.</p>
     *
     * <p><strong>NOTE:</strong> to avoid problems during debugging
     * situations, the string will be generated only from the
     * information supplied explicitly in attributes; defaulted
     * vocabularies and schemes from {@link DefaultVocabularyFor}
     * declarations in {@link Resource} objects will not be used.  To
     * get the defaulted information in the string, use the {@link
     * #toString(boolean)} method.</p>
     *
     * @return A string representation of the formal name.
     * @see #getVocabulary
     * @see FormalName#getScheme
     * @see NewsItemId#getScheme
     * @see #getName
     */
    public String toString ();


    /**
     * Generate a proper string representation of this formal name.
     *
     * <p>The format is the same as that used by the zero-argument
     * {@link #toString()} method, but this method will look for
     * defaulted vocabulary and scheme information from catalogs in
     * the current document, and, if the useExternal flag is true,
     * from external documents as well.</p>
     *
     * @param useExternal If true, follow links outside of the current
     * NewsML document during catalog resolution.
     * @return A string representation of the formal name.
     * @exception IOException If there is an error retrieving
     * an external resource.
     * @see #getDefaultVocabulary
     * @see #getDefaultScheme
     * @see #getName
     */
    public String toString (boolean useExternal)
	throws IOException;

}

// end of FormalNameNode.java
