// NewsComponent.java - a (sub)component of a news item or news component.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsComponent.java,v 1.11 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Container for specifying role information and metadata for news objects.
 *
 * <p>A news component is a branch in the NewsML content tree.  A news
 * component may contain zero or more members, and the members may
 * function as complements or equivalents (see the {@link
 * #getEquivalentsList} method).</p>
 *
 * <p>A member of a NewsComponent may be a {@link
 * org.newsml.toolkit.NewsItem}, a reference to a NewsItem, another
 * {@link NewsComponent}, or a {@link ContentItem}.  If a news
 * component has more than one member and they are flagged as
 * equivalents, then the {@link #getBasisForChoice} method specifies
 * the criteria for choosing among the alternatives.  It is also
 * possible to have a NewsComponent with no members (say, to transmit
 * {@link NewsLines} alone).</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsComponent
    extends BaseNode, CatalogNode, CommentNode, EquivalentNode,
	    IdNode, LanguageNode
{

    /**
     * Test whether the provider has flagged this component as essential.
     *
     * <p>An essential flag indicates that the provider considers this
     * component necessary to the understanding of the story;
     * otherwise, the component may safely be omitted.</p>
     *
     * <p>To test this property as a boolean, use the {@link
     * Text#toBoolean} method in the {@link Text} object returned.</p>
     *
     * @return Text containing the word 'yes' if this node is
     * essential, or 'no' otherwise (or null if unspecified, which
     * means default to 'no').
     */
    public Text getEssential ();


    /**
     * Test whether the members of this component are equivalents.
     *
     * If the members are equivalent, then the user should choose
     * among them using the criteria specified by {@link
     * #getBasisForChoice}.
     *
     * @return Text containing the word 'yes' if the members of this
     * component are equivalent versions of the same information, or
     * 'no' otherwise (or null if unspecified, which means default to
     * 'no').
     * @see #getMember
     * @see #getBasisForChoice
     */
    public Text getEquivalentsList ();


    /**
     * Get the number of topic sets.
     *
     * @return The number of topic sets present.
     */
    public int getTopicSetCount ();

    
    /**
     * Get a topic set attached to this component.
     *
     * @param index The index of the topic set, zero-based, numbered
     * sequentially in document order.
     * @return The topic set, or null if none was provided at the
     * specified index.
     */
    public TopicSet getTopicSet (int index);


    /**
     * Get all TopicSet children in an array.
     *
     * @return A (possibly-empty) array of TopicSet children.
     */
    public TopicSet [] getTopicSet ();


    /**
     * Get the role of this component in a parent component.
     *
     * When a news component contains one or more subcomponents that
     * are not equivalent, the role property specifies how they
     * should be used together.
     *
     * @return A formal name representing the component's role, or
     * null if none was specified.
     * @see #getEquivalentsList
     */
    public FormalName getRole ();


    /**
     * Get the number of bases for choice.
     *
     * @return The number of bases for choice.
     */
    public int getBasisForChoiceCount ();


    /**
     * Get the basis for choosing among equivalent members.
     *
     * If the news component contains more than one equivalent
     * member, this method specifies what information an
     * application should examine for choosing among the
     * members.  The return value is an XPath expression, which
     * may be tricky for applications to use; it may be advisable
     * to obtain a fixed list of XPath expressions from your
     * news provider and treat them as enumerated values.
     *
     * @param index The index of the basis for choice, zero-based,
     * in sequential order.
     * @return An XPath expression representing the basis for choice,
     * or null if none was specified with the current index.
     * @see #getEquivalentsList
     * @see #getMember
     */
    public BasisForChoice getBasisForChoice (int index);


    /**
     * Get all BasisForChoice children in an array.
     *
     * @return A (possibly-empty) array of BasisForChoice children.
     */
    public BasisForChoice [] getBasisForChoice ();


    /**
     * Get all BasisForChoice children, sorted by rank.
     *
     * <p>The entries will be arranged first by the value of the
     * BasisForChoice 'rank' attribute (treating no value as the
     * lowest possible rank), and then, in document order.</p>
     *
     * @return An array of all BasisForChoice children, sorted by
     * rank from least to most important.
     */
    public BasisForChoice [] getRankedBasisForChoice ();


    /**
     * Get a news component's headline, byline, etc.
     *
     * The news lines represent human-readable information about
     * the content, including the headline and subheadline, dateline,
     * byline, rights line, and other information.
     *
     * @return An object containing the news lines for the component,
     * or null if none was specified.
     */
    public NewsLines getNewsLines ();


    /**
     * Get administrative information for the news component.
     *
     * Administrative information includes the providers, creators,
     * sources, and contributors, among other information.
     *
     * @return An object containing the administrative metadata
     * for the component, or null if none was provided.
     * @see #getRightsMetadata
     * @see #getDescriptiveMetadata
     * @see #getMetadata
     */
    public AdministrativeMetadata getAdministrativeMetadata ();


    /**
     * Get rights information for the news component.
     *
     * Rights information includes copyright and usage rights.
     *
     * @return An object containing the rights metadata for the
     * component, or null if none was provided.
     * @see #getAdministrativeMetadata
     * @see #getDescriptiveMetadata
     * @see #getMetadata
     */
    public RightsMetadata getRightsMetadata ();


    /**
     * Get descriptive information about the news component.
     *
     * Descriptive information includes the language, genre, 
     * and subject codes, among other information.
     *
     * @return An object containing the descriptive metadata for
     * the component, or null if none was provided.
     * @see #getAdministrativeMetadata
     * @see #getRightsMetadata
     * @see #getMetadata
     */
    public DescriptiveMetadata getDescriptiveMetadata ();


    /**
     * Get the number of generic metadata objects present.
     *
     * @return The number of generic metadata objects present.
     */
    public int getMetadataCount ();


    /**
     * Get generic metadata about the news component.
     *
     * This information includes metadata not classified as
     * administrative, rights-related, or descriptive.  There may
     * be multiple generic metadata sections, each containing a
     * related group of information.
     *
     * @param index The index of the generic metadata, zero-based,
     * numbered sequentially.
     * @return An object containing the generic metadata, or null
     * if none was provided at the specified index.
     */
    public Metadata getMetadata (int index);


    /**
     * Get all Metadata children in an array.
     *
     * @return A (possibly-empty) array of Metadata children.
     */
    public Metadata [] getMetadata ();


    /**
     * Get the number of members in this news component.
     *
     * @return The number of members.
     */
    public int getMemberCount ();


    /**
     * Return one of the members of this news component.
     *
     * The members of the component will be all news components,
     * all content items, or a mixture of news items and references
     * to news items.
     *
     * @param index The index of the member, zero-based, numbered
     * sequentially.
     * @return The member or null if no member was provided at
     * the specified index.
     */
    public EquivalentNode getMember (int index);


    /**
     * Get all Member children in an array.
     *
     * @return A (possibly-empty) array of Member children.
     */
    public EquivalentNode [] getMember ();

}

// end of NewsComponent.java
