// NewsEnvelope.java - routing information for the NewsML package.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsEnvelope.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Information about the transmission of the NewsML package.
 *
 * <p>A news envelope contains information about the transmission itself
 * rather than about the transmission's content; it is the equivalent
 * of an address label on the outside of a package.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsEnvelope
    extends BaseNode, IdNode
{

    /**
     * Get the identifier for this transmission.
     *
     * @return The transmission identifier, or null if none was
     * specified.
     */
    public TransmissionId getTransmissionId ();


    /**
     * Get a list of senders of this package.
     *
     * @return A list of parties representing the senders, or null if
     * none were provided.
     */
    public PartyList getSentFrom ();


    /**
     * Get a list of recipients of this package.
     *
     * @return A list of recipients, or null of none were provided.
     */
    public PartyList getSentTo ();


    /**
     * Get the date on time when this package was transmitted.
     *
     * @return Text representing the date and time of transmission,
     * in ISO 8601 Basic format.
     */
    public IdText getDateAndTime ();


    /**
     * Count the news services present.
     *
     * @return The number of news services.
     */
    public int getNewsServiceCount ();


    /**
     * Get one of the news services associated with this package.
     *
     * @param index The index of the service, zero-based, numbered
     * sequentially.
     * @return A formal name representing the service, or null
     * if none was provided at the specified index.
     */
    public FormalName getNewsService (int index);


    /**
     * Get all NewsService children in an array.
     *
     * @return A (possibly-empty) array of NewsService children.
     */
    public FormalName [] getNewsService ();


    /**
     * Count the news products.
     *
     * @return The number of news products present.
     */
    public int getNewsProductCount ();


    /**
     * Get one of the news products associated with this package.
     *
     * @param index The index of the product, zero-based, numbered
     * sequentially.
     * @return A formal name representing the product, or null
     * if none was provided at the specified index.
     */
    public FormalName getNewsProduct (int index);


    /**
     * Get all NewsProduct children in an array.
     *
     * @return A (possibly-empty) array of NewsProduct children.
     */
    public FormalName [] getNewsProduct ();


    /**
     * Get the priority of this package.
     *
     * @return A formal name representing the priority, or null
     * if none was provided.
     */
    public FormalName getPriority ();

}

// end of NewsEnvelope.java

