// NewsIdentifier.java - a globally-unique identifier for a news item.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsIdentifier.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * A globally-unique identifier for a news item.
 *
 * <p>The combination of the provider ID, date ID, news item ID, and
 * revision ID make the news item unique; different revisions of the
 * same item will differ only in the revision ID.</p>
 *
 * <p>Note that the NewsIdentifier does <em>not</em> have a Duid or
 * Euid.</p>
 *
 * <p>TODO:</p>
 *
 * <ul>
 * <li>FormalName returned should not have a getDuid or getEuid method.</li>
 * </ul>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsIdentifier
    extends BaseNode
{


    /**
     * Get the provider portion of the news identifier.
     *
     * <p>Recommended practice is to use a domain name for the
     * provider ID; this will be available through the
     * {@link FormalName#getName} method.  Note that the formal
     * name returned will never have a scheme attached.</p>
     *
     * @return A node representing the provider (not null).
     * @see #getDateId
     * @see #getNewsItemId
     * @see #getRevisionId
     */
    public ProviderId getProviderId ();


    /**
     * Get the date  portion of the news identifier.
     *
     * <p>The date ID will always appear in ISO 8601 Basic Format,
     * containing only the year, month, and day-of-the-month portions
     * (CCYYMMDD), as in "20001122" for November 22, 2000.  Once a
     * news item is created, its date ID will not change, even if
     * new revision come out on subsequent days.</p>
     *
     * @return Text containing the date identifier.
     * @see #getProviderId
     * @see #getNewsItemId
     * @see #getRevisionId
     */
    public Text getDateId ();


    /**
     * Get the news item ID for the story.
     *
     * <p>The news item ID may appear in any format desired by the
     * sender.  The name part of the ID is be unique for each
     * date; in other words, the combination of date ID and news item
     * ID is unique for each provider ID.</p>
     *
     * @return A node representing the news item ID (not null).
     * @see #getProviderId
     * @see #getDateId
     * @see #getRevisionId
     */
    public NewsItemId getNewsItemId ();


    /**
     * Get the revision ID for the news item.
     *
     * @return The revision ID (not null).
     */
    public RevisionId getRevisionId ();


    /**
     * Get a serialized version of the ID in a single string.
     *
     * <p>The string will appear in the format
     * urn:newsml:{ProviderId}:{DateId}:{NewsItemId}:{RevisionId}{RevisionId@Update}
     * (note that "newsml" is not a registered URN namespace as of
     * 2000-11-22, so this usage is non-standard).  This information,
     * though required, is redundant with the rest of the
     * NewsIdentifier interface, and applications should take
     * precautions if there is a conflict between this string and the
     * four analysed parts of the news identifier.</p>
     *
     * @return Text containing the serialized news identifier (not
     * null)
     */
    public Text getPublicIdentifier ();

}

// end of NewsIdentifier.java
