// NewsItem.java - a top-level news item.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsItem.java,v 1.9 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * A managed set of news information.
 *
 * <p>A news item is a branch in the NewsML content tree that attaches
 * identification and management information to news objects; much of
 * the interesting information (such as headlines and subheadlines,
 * metadata, and the actual content) is available in or through the
 * root news component (see {@link #getRootNewsComponent}).
 * Alternatively, the news item may contain a topic set (i.e. a
 * vocabulary being distributed) or deltas for an incremental update:
 * see {@link #getRootTopicSet} and {@link #getUpdateCount}.</p>
 * 
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsItem
    extends BaseNode,CatalogNode, CommentNode, EquivalentNode, IdNode,
	    LanguageNode
{

    /**
     * Get the identification labels.
     *
     * @return An object containing the identification labels
     * for this news item (not null).
     */
    public Identification getIdentification ();


    /**
     * Get the management information for the news item.
     *
     * The management information includes the type of the news item,
     * its status (including whether it's embargoed), when it was
     * created, and similar information.
     *
     * @return An object containing the management information for the
     * news item (not null).
     */
    public NewsManagement getNewsManagement ();


    /**
     * Get the root component of the news item.
     *
     * <p>Return the root component of the news item.  The component
     * contains newslines, metadata, and other similar information,
     * and is the beginning of the path down the NewsML tree to the
     * actual content.  If this method returns a non-null value,
     * {@link #getRootTopicSet} must return null and {@link
     * #getUpdateCount} must return 0.</p>
     *
     * @return The root news component, or null if none was specified.
     */
    public NewsComponent getRootNewsComponent ();


    /**
     * Get the root topic set of the news item.
     *
     * <p>If the topic set is present, it is actually the payload of
     * this newsitem, rather than simply metadata describing the
     * payload.  If this method returns a non-null value, {@link
     * #getRootNewsComponent} must return null and {@link
     * #getUpdateCount} must return 0.</p>
     *
     * @return The root topic set, or null if none was specified.
     */
    public TopicSet getRootTopicSet ();


    /**
     * Count the updates present in this news item.
     *
     * <p>If the news item contains deltas rather than a news
     * component or topic set, the deltas will be accessible through a
     * series of Update objects.  If this method returns a non-zero
     * value, {@link #getRootNewsComponent} and {@link
     * #getRootTopicSet} must both return null.</p>
     *
     * @return The number of updates present.
     * @see #getUpdate(int)
     */
    public int getUpdateCount ();


    /**
     * Get one of the updates in this news item.
     *
     * @param index The index of the update to get.
     * @return The update, or null if the index is out of range.
     * @see #getUpdateCount()
     * @see #getUpdate()
     */
    public Update getUpdate (int index);


    /**
     * Get all of the updates in this news item.
     *
     * @return All of the update nodes in an array (the array may be
     * empty).
     * @see #getUpdate(int)
     */
    public Update [] getUpdate ();

}

// end of NewsItem.java
