// NewsItemRef.java - reference to a news item.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsItemRef.java,v 1.10 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;


/**
 * A reference to a news item located elsewhere.
 *
 * <p>This class represents a cross-link in the NewsML content tree.
 * Instead of including a NewsItem physically, the content tree may
 * include a pointer to it; the pointer may point to another
 * news item within the same package, or it may point to an
 * external news item using a URL reference.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsItemRef
    extends BaseNode, CommentNode, EquivalentNode, IdNode
{

    /**
     * Get the reference string.
     *
     * If this is an internal reference, the reference string will
     * consist of the character '#' followed by the Duid of the
     * news item in the current document; otherwise, the value is
     * either a NewsML URN or a URL referring to an external
     * resource.
     *
     * @return Text containing the reference string, or null if none
     * was provided.
     */
    public Text getRef ();



    /**
     * Get the referenced NewsItem, if possible.
     *
     * <p>This method will fail for a NewsML URN, but might succeed
     * for an internal reference (beginning with '#') or a URL,
     * depending on whether the target is available.</p>
     *
     * <p>The URL <em>must</em> point either to a specific NewsItem or
     * to a NewsML package.  If the URL points to a complete NewsML
     * package, the first NewsItem in the document will be returned;
     * if that is not what you want, use a fragment identifier to
     * point to the Duid of a different NewsItem.</p>
     *
     * <p>For any other situations, you can still get a non-URN
     * referenced resource easily using {@link
     * NewsMLSession#findReference} in the {@link NewsMLSession} (see
     * {@link #getSession}).</p>
     *
     * @param useExternal Follow links outside of the current
     * document.
     * @return A NewsItem, or null if no reference was provided.
     * @exception IOException If there is an error retrieving the
     * news item.
     * @exception NewsMLException If the external document is
     * retrieved successfully but does not have a root element type
     * of NewsML or NewsItem.
     */
    public NewsItem getReferencedNewsItem (boolean useExternal)
	throws IOException;

}

// end of NewsItemRef.java
