// NewsLines.java - container for a news component's news lines.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsLines.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Container for a news component's news lines.
 *
 * <p>A {@link NewsComponent} may have a series of standard (and
 * non-standard) news lines attached to it, such as the headline,
 * byline, and dateline.  All of those news lines are accessible
 * through this interface.  There are also special convenience
 * methods to return headlines and subheadlines as grouped pairs
 * in a {@link HeadLineGroup}.</p>
 *
 * <p>News lines contain natural-language text intended for human
 * readers, and do not necessarily follow rigid formats.  The
 * information in the news lines may be duplicated in some types of
 * content (such as textual news stories) but not in others (such as
 * photos).</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsLines
    extends BaseNode, IdNode
{

    /**
     * Count the headlines.
     *
     * @return The number of headlines present.
     */
    public int getHeadLineCount ();


    /**
     * Get a headline.
     *
     * <p>The headline contains a primary title for the {@link
     * NewsComponent}.</p>
     *
     * @param index The index of the headline, zero-based, numbered
     * sequentially.
     * @return Text representing the headline, or null if none
     * was provided at the specified index.
     */
    public OriginText getHeadLine (int index);


    /**
     * Get all headline children in an array.
     *
     * @return A (possibly-empty) array of headline children.
     */
    public OriginText [] getHeadLine ();


    /**
     * Count the subheadlines.
     *
     * @return The number of subheadlines present.
     */
    public int getSubHeadLineCount ();


    /**
     * Get a subheadline.
     *
     * <p>The subheadline contains an optional subtitle for the {@link
     * NewsComponent}, linked with the preceding HeadLine in document
     * order.</p>
     *
     * <p>Note: HeadLine and SubHeadLine indices will not necessarily
     * correspond, since some headlines may not be accompanied by
     * subheadlines; use {@link #getHeadLineGroup(int)} to get
     * headlines and subheadlines grouped properly.</p>
     *
     * @param index The index of the subheadline, zero-based, numbered
     * sequentially.
     * @return Text representing the subheadline, or null if none
     * was provided at the specified index.
     */
    public OriginText getSubHeadLine (int index);


    /**
     * Get all subheadline children in an array.
     *
     * @return A (possibly-empty) array of subheadline children.
     */
    public OriginText [] getSubHeadLine ();


    /**
     * Count the ByLines.
     *
     * @return The number of ByLines present.
     */
    public int getByLineCount ();


    /**
     * Get a ByLine.
     *
     * <p>The ByLine presents information about the authors or
     * creators of the {@link NewsComponent}.</p>
     *
     * @param index The index of the ByLine, zero-based, numbered
     * sequentially.
     * @return Text representing the ByLine, or null if none
     * was provided at the specified index.
     */
    public OriginText getByLine (int index);


    /**
     * Get all byline children in an array.
     *
     * @return A (possibly-empty) array of byline children.
     */
    public OriginText [] getByLine ();


    /**
     * Count the datelines.
     *
     * @return The number of datelines present.
     */
    public int getDateLineCount ();


    /**
     * Get a dateline.
     *
     * <p>The byline presents information about the date when and/or
     * location where the {@link NewsComponent} was created.</p>
     *
     * @param index The index of the dateline, zero-based, numbered
     * sequentially.
     * @return Text representing the dateline, or null if none
     * was provided at the specified index.
     */
    public OriginText getDateLine (int index);


    /**
     * Get all DateLine children in an array.
     *
     * @return A (possibly-empty) array of DateLine children.
     */
    public OriginText [] getDateLine ();


    /**
     * Count the credit lines.
     *
     * @return The number of credit lines present.
     */
    public int getCreditLineCount ();


    /**
     * Get a credit line.
     *
     * <p>The credit line provides credits (attribution,
     * contributions, and so on) for the {@link NewsComponent}.</p>
     *
     * @param index The index of the credit line, zero-based, numbered
     * sequentially.
     * @return Text representing the credit line, or null if none
     * was provided at the specified index.
     */
    public OriginText getCreditLine (int index);


    /**
     * Get all CreditLine children in an array.
     *
     * @return A (possibly-empty) array of CreditLine children.
     */
    public OriginText [] getCreditLine ();


    /**
     * Count the copyright lines.
     *
     * @return The number of copyright lines.
     */
    public int getCopyrightLineCount ();


    /**
     * Get a copyright line.
     *
     * <p>A natural-language statement of copyright information for
     * the {@link NewsComponent}.</p>
     *
     * @param index The index of the copyright line, zero-based,
     * numbered sequentially.
     * @return Text representing the copyright line, or null if
     * none was provided at the specified index.
     */
    public OriginText getCopyrightLine (int index);


    /**
     * Get all CopyrightLine children in an array.
     *
     * @return A (possibly-empty) array of CopyrightLine children.
     */
    public OriginText [] getCopyrightLine ();


    /**
     * Count the rights lines.
     *
     * @return The number of rights lines present.
     */
    public int getRightsLineCount ();


    /**
     * Get a rights line.
     *
     * <p>A description of rights assigned for the {@link
     * NewsComponent} (such as distribution and archiving).</p>
     *
     * @param index The index of the rights line, zero-based, numbered
     * sequentially.
     * @return Text representing the rights line, or null if none
     * was provided at the specified index.
     */
    public OriginText getRightsLine (int index);


    /**
     * Get all RightsLine children in an array.
     *
     * @return A (possibly-empty) array of RightsLine children.
     */
    public OriginText [] getRightsLine ();


    /**
     * Count the series lines.
     *
     * @return The number of series lines present.
     */
    public int getSeriesLineCount ();


    /**
     * Get a series line.
     *
     * <p>Information describing the role of the {@link NewsComponent}
     * in a larger series.</p>
     *
     * @param index The index of the series line, zero-based, numbered
     * sequentially.
     * @return Text representing the series line, or null if none
     * was provided at the specified index.
     */
    public OriginText getSeriesLine (int index);


    /**
     * Get all SeriesLine children in an array.
     *
     * @return A (possibly-empty) array of SeriesLine children.
     */
    public OriginText [] getSeriesLine ();


    /**
     * Count the slug lines.
     *
     * @return The number of slug lines present.
     */
    public int getSlugLineCount ();


    /**
     * Get a slug line.
     *
     * <p>An abbreviated string for referring to a news story.  There
     * is no fixed format for slug lines, and they will vary among
     * providers.</p>
     *
     * @param index The index of the slug line, zero-based, numbered
     * sequentially.
     * @return Text representing the slug line, or null if none
     * was provided at the specified index.
     */
    public OriginText getSlugLine (int index);


    /**
     * Get all SlugLine children in an array.
     *
     * @return A (possibly-empty) array of SlugLine children.
     */
    public OriginText [] getSlugLine ();


    /**
     * Count the keyword lines.
     *
     * @return The number of keyword lines present.
     */
    public int getKeywordLineCount ();


    /**
     * Get a keyword line.
     *
     * <p>Displayable keywords relevant to a {@link
     * NewsComponent}.</p>
     *
     * @param index The index of the keyword line, zero-based,
     * numbered sequentially.
     * @return Text representing the keyword line, or null if none
     * was provided at the specified index.
     */
    public OriginText getKeywordLine (int index);


    /**
     * Get all KeywordLine children in an array.
     *
     * @return A (possibly-empty) array of KeywordLine children.
     */
    public OriginText [] getKeywordLine ();


    /**
     * Count the generic news lines.
     *
     * @return The number of generic news lines present.
     */
    public int getNewsLineCount ();


    /**
     * Get a user-defined news line.
     *
     * <p>This method returns news lines of types not covered by the
     * other methods.</p>
     *
     * @param index The index of the news line, zero-based, numbered
     * sequentially.
     * @return The news line, or null if none was provided at the
     * specified index.
     */
    public NewsLine getNewsLine (int index);


    /**
     * Get all NewsLine children in an array.
     *
     * @return A (possibly-empty) array of NewsLine children.
     */
    public NewsLine [] getNewsLine ();


    /**
     * Convenience method: count the headline groups present.
     *
     * @return The number of headline groups.
     * @see #getHeadLineCount
     */
    public int getHeadLineGroupCount ();


    /**
     * Convenience method: get a headline/subheadline pair.
     *
     * @param index The index of the headline group, zero-based,
     * numbered sequentially.
     * @return A group containing a headline and optional subheadline,
     * or null if no group was provided at the index specified..
     * @see #getHeadLine(int)
     * @see #getSubHeadLine(int)
     */
    public HeadLineGroup getHeadLineGroup (int index);


    /**
     * Convenience method: get all HeadLineGroup children in an array.
     *
     * @return A (possibly-empty) array of HeadLineGroup children.
     * @see #getHeadLine()
     * @see #getSubHeadLine()
     */
    public HeadLineGroup [] getHeadLineGroup ();

}

// end of NewsLines.java
