// NewsML.java - top-level NewsML package.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsML.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * The top-level NewsML package.
 *
 * <p>This interface represents the root of the NewsML content tree.
 * The NewsML specification defines packaging and metadata, but not
 * content: in other words, a NewsML package shows how news objects go
 * together, and provides information (<em>metadata</em>) about the
 * news objects, but the objects themselves will appear in other
 * formats (either XML-based or non XML-based).</p>
 *
 * <p>The package contains two major parts: the news envelope
 * (see {@link #getNewsEnvelope}), which is roughly equivalent to the
 * address on the outside of a package, and a list of news items
 * (see {@link #getNewsItem}), which assemble collections of news objects
 * by theme or content.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsML
    extends BaseNode, CatalogNode, IdNode
{

    /**
     * Get the number of topic sets present.
     *
     * @return The number of topic sets present.
     */
    public int getTopicSetCount ();


    /**
     * Get a topic set attached to this component.
     *
     * @param index The index of the topic set, zero-based, numbered
     * sequentially in document order.
     * @return The topic set, or null if none was provided at the
     * specified index.
     */
    public TopicSet getTopicSet (int index);


    /**
     * Get all TopicSet children in an array.
     *
     * @return A (possibly-empty) array of TopicSet children.
     */
    public TopicSet [] getTopicSet ();


    /**
     * Get the envelope for the NewsML package.
     *
     * The envelope includes an identifier for the transmission, the
     * sender(s), the intended recipient(s), the date and time of
     * sending, the priority, and other related information.
     *
     * @return An object containing the envelope information (not
     * null).
     */
    public NewsEnvelope getNewsEnvelope ();


    /**
     * Count the top-level news items.
     *
     * @return The number of top-level news items.
     */
    public int getNewsItemCount ();


    /**
     * Get one of the top-level news items.
     *
     * Each of the items represents a self-contained, managed set
     * of news information.
     *
     * @param index The index of the news item, zero-based, numbered
     * sequentially.
     * @return The news item at the specified index, or null if none
     * was provided.
     */
    public NewsItem getNewsItem (int index);


    /**
     * Get all NewsItem children in an array.
     *
     * @return A (possibly-empty) array of NewsItem children.
     */
    public NewsItem [] getNewsItem ();

}

// end of NewsML.java
