// NewsMLException.java - a NewsML-specific exception.
// Copyright (c) 2001 by Reuters PLC.
// $Id: NewsMLException.java,v 1.6 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;


/**
 * A NewsML-related exception.
 *
 * <p>This class extends {@link java.io.IOException} for two reasons:</p>
 *
 * <ol>
 *
 * <li>because, conceptually, reading NewsML (or XML in general) is an
 * I/O operation; and</li>
 *
 * <li>so that applications can simply catch IOException without
 * having a hard-coded dependency on NewsML.</li>
 *
 * </ol>
 *
 * <p>The class may embed another exception of some kind, such as a
 * {@link org.xml.sax.SAXException}, depending on the implementation
 * being used.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public class NewsMLException extends IOException
{


    /**
     * Construct an exception with no message.
     */
    public NewsMLException ()
    {
	super();
    }


    /**
     * Construct an exception with an explicit message.
     *
     * @param message The error message for the exception.
     */
    public NewsMLException (String message)
    {
	super(message);
    }


    /**
     * Construct an exception wrapping another exception.
     *
     * @param exception The exception being wrapped.
     */
    public NewsMLException (Exception exception)
    {
	super();
	this.exception = exception;
    }


    /**
     * Construct an exception with a message and embedded exception.
     *
     * @param message The error message.
     * @param exception The exception being wrapped.
     */
    public NewsMLException (String message, Exception exception)
    {
	super(message);
	this.exception = exception;
    }


    /**
     * Get the message for the exception, if any.
     *
     * <p>If there is no explicit message, look for a message
     * in the embedded exception (if present).</p>
     *
     * @return An exception message, or null if none is available.
     */
    public String getMessage ()
    {
	String message = super.getMessage();
	
	if (message == null && exception != null) {
	    return exception.getMessage();
	} else {
	    return message;
	}
    }
    
    
    /**
     * Get the embedded exception, if any.
     *
     * @return The embedded exception, or null if there is none.
     */
    public Exception getException ()
    {
	return exception;
    }


    private Exception exception;
}

// end of NewsMLException.java
