// NewsMLFactory.java - interface for a factory to create NewsML documents.
// Copyright (c) 2001 by Reuters PLC.
// $Id: NewsMLFactory.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package org.newsml.toolkit;

import java.io.IOException;
import java.io.Reader;


/**
 * Factory interface for NewsML documents.
 *
 * <p>This is an abstract factory interface for creating NewsML
 * documents and nodes from external resources: it contains methods
 * that can create a top-level {@link NewsML} object or any arbitrary
 * NewsML node (implementing the {@link BaseNode} interface) given a
 * URL or a character stream.  Normally, this will be the only public
 * point of access to subpackages that contain contain concrete
 * implementations, such as {@link
 * org.newsml.toolkit.dom.DOMNewsMLFactory}.</p>
 *
 * <p>HINT: a Java {@link java.io.Reader} can read from any source,
 * not just a file.  For example, if you need to read NewsML document
 * from a string, try using a StringReader; if you need to parse data
 * coming from somewhere else in your program, try a PipedReader.</p>
 *
 * <p>This class includes {@link #setValidation} and {@link
 * #getValidation} methods for enabling or disabling DTD validation.
 * If the <var>validation</var> property is <code>true</code>, the
 * factory will process the XML source with DTD-validation or throw an
 * exception if the underlying implementation does not support
 * validation; if it is false, the factory will process the XML source
 * without DTD-validation or throw an exception if the underlying
 * implementation does not support non-validating parsing.  The
 * default value is system-dependent, but will usually be
 * <code>false</code> for reasons explained below.</p>
 *
 * <p>If <var>validation</var> is false (recommended), the
 * implementation will not load any external XML entities, including
 * the external DTD subset, and will not report validation errors; if
 * <var>validation</var> is true (dangerous: see below), the
 * implementation will attempt to load all externally-referenced XML
 * entities, and will fail with an exception if there are any
 * validation errors.</p>
 *
 * <p>While DTD-validation is frequently promoted in XML books and
 * workshops, it is almost always a bad idea for production
 * applications: it has the potential to cause serious problems and
 * should be used only with extreme caution (or better yet, not at
 * all).  DTD validation opens your applications and your organization
 * to the following serious risks:</p>
 *
 * <ol>
 *
 * <li>Performance degradation: a DTD hosted at a remote site could
 * take seconds or minutes to load because of network congestion or a
 * heavy load on the host: for a system processing thousands of NewsML
 * packages daily, the delays may be fatal.</li>
 *
 * <li>Denial of service: if system hosting the DTD is down, or if the
 * system administrator changes the location of the DTD file, your
 * system will be unable to process any NewsML packages.  Malicious
 * parties may crack the system hosting the DTD and make subtle (and
 * difficult to find changes) that cause all of your NewsML packages
 * to be rejected, or the DTD may be legitimately upgraded to a newer
 * version with the same effect.  In essence, the security of your
 * whole system is no better than the security of the external
 * computer hosting the DTD.</li>
 *
 * <li>Sabotage: in addition to modifying the DTD to force your NewsML
 * documents to be rejected, a malicious user could modify the DTD to
 * change default values for attributes, causing your system to
 * process the NewsML packages correctly but produce incorrect output
 * and mis-categorization.</li>
 *
 * <li>Unintentional disclosure: since a DTD-validating system hits
 * the external host every time it processes a NewsML package, the
 * external host can do traffic analysis to learn about how your
 * internal system works.</li>
 *
 * </ol>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLException
 */
public interface NewsMLFactory
{

    /**
     * Set the validation flag.
     *
     * @param validation true if the factory is required to perform
     * DTD validation, false if it is required not to.
     * @exception IOException if the implementation does not
     * support the requested state.
     * @see #getValidation()
     */
    public void setValidation (boolean validation)
	throws IOException;


    /**
     * Get the validation flag.
     *
     * @return true if the factory is required to perform DTD validation,
     * false if it is required not to.
     * @see #setValidation(boolean)
     */
    public boolean getValidation ();
    

    /**
     * Create a top-level NewsML object from a URL.
     *
     * @param url A string in URL format.
     * @return A top-level NewsML node.
     * @exception IOException if there is some kind of error reading
     * the document, including a validation error if validation is
     * requested.
     * @exception NewsMLException if the root element of the document
     * is not NewsML, or if the underlying implementation does not
     * support the requested validation type.
     */
    public NewsML createNewsML (String url)
	throws IOException;


    /**
     * Create a top-level NewsML object from a character stream.
     *
     * @param input The character stream.
     * @param baseURL The base URL for resolving relative links;
     * if null, will default to a file: URL based on the current
     * directory.
     * @return The top-level NewsML node.
     * @exception IOException if there is some kind of error reading
     * the document.
     * @exception NewsMLException if the root element of the document
     * is not NewsML.
     */
    public NewsML createNewsML (Reader input, String baseURL)
	throws IOException;


    /**
     * Create a NewsML node from a URL.
     *
     * @param url A string in URL format.
     * @return A new base node, or null if the root element type is not
     * recognized or not supported.
     * @exception IOException if there is some kind of error reading
     * the document.
     */
    public BaseNode createNode (String url)
	throws IOException;


    /**
     * Create a NewsML node from a URL.
     *
     * @param input The character stream.
     * @param baseURL The base URL for resolving relative links;
     * if null, will default to a file: URL based on the current
     * directory.
     * @return A new base node, or null if the root element type is not
     * recognized or not supported.
     * @exception IOException if there is some kind of error reading
     * the document.
     */
    public BaseNode createNode (Reader input, String baseURL)
	throws IOException;

}

// end of NewsMLFactory.java
