// NewsMLSession.java - meta-information for a NewsML session.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsMLSession.java,v 1.9 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * Meta-information for a NewsML session.
 *
 * <p>This interface provides a node-independent access point for
 * getting and setting information about a NewsML session.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsMLSession
{

    /**
     * Get the factory for creating new NewsML documents.
     *
     * @return The current NewsML factory (not null).
     */
    public NewsMLFactory getFactory ();


    /**
     * Get the root node of the current NewsML document.
     *
     * @return An object representing the root node; the caller
     * will have to cast it to the appropriate type.
     */
    public BaseNode getRootNode ();


    /**
     * Look up a reference to a NewsML node.
     *
     * <p>If the reference contains '#', the part following
     * corresponds with a Duid in the target document (otherwise uses
     * the root element); if the part preceding '#' is empty, the
     * target document is the current document.</p>
     *
     * <p>The {@link #getNodeByDuid(String)} method is a little 
     * simpler and more efficient if you know for you're simply
     * searching for a node that matches a specific Duid.</p>
     *
     * @param reference The node reference, as a string in URI
     * reference format (may include a '#').
     * @param useExternal true to follow references in external URLs,
     * false otherwise.
     * @return A BaseNode (which must be cast to the appropriate type)
     * or null if no match was found.
     * @exception IOException if there is an error finding the node.
     * @see #getNodeByDuid(String)
     */
    public BaseNode findReference (String reference, boolean useExternal)
	throws IOException;


    /**
     * Look up a node in the current document by Duid.
     *
     * <p>This is a more restrictive (and slightly more efficient)
     * version of {@link #findReference}; unlike findReference, this
     * method does <em>not</em> require a leading '#' on the Duid
     * (though it's optionally allowed), and does not throw an
     * IOException.</p>
     *
     * <p>[Please note that in the current DOM implementation, only
     * a few node types are castable; everything else must remain
     * as BaseNode].</p>
     *
     * @param duid the document-unique identifier of the node to get.
     * @return A BaseNode (which must be cast to the appropriate type)
     * or null if no match was found.
     * @see #findReference
     */
    public BaseNode getNodeByDuid (String duid);


    /**
     * Look up nodes in the current document by XPath.
     *
     * <p>This method looks up nodes by XPath, starting with the
     * context node provided.  Note that the XPath must match
     * elements or no results will be returned (attribute matches
     * are not currently supported, though attributes may be used
     * in node tests).</p>
     *
     * @param contextNode The starting point, or context node, for
     * the XPath expression.
     * @param xpath The XPath expression as a string.
     * @return An possibly-empty array of matching nodes.
     * @exception NewsMLException if the XPath expression provided
     * is invalid.
     */
    public BaseNode [] getNodesByXPath (BaseNode contextNode, String xpath)
	throws NewsMLException;


    /**
     * Get the first node matching an XPath expression.
     *
     * <p>This method finds the first node matching the xpath
     * expression provided, starting at the context node.  Note that
     * the XPath must match elements or no matching node will be found
     * (attribute matches are not currently supported, though
     * attributes may be used in node tests).</p>
     * 
     * @param contextNode The starting point, or context node, for
     * the XPath expression.
     * @param xpath The XPath expression as a string.
     * @return The first node matching the expression, or null if
     * there are no matches.
     * @exception NewsMLException if the XPath expression provided
     * is invalid.
     */
    public BaseNode getNodeByXPath (BaseNode contextNode, String xpath)
	throws NewsMLException;


    /**
     * Get the base URL for the document.
     *
     * @return The base URL.
     * @see #resolveHref(String)
     */
    public String getBaseURL ();


    /**
     * Resolve a reference.
     *
     * <p>This method is useful for retrieving an external resource --
     * text or binary -- referenced from a NewsML package.  See the
     * standard Java {@link URL} class for information on how your
     * application can use the URL returned to load remote resources.
     * The method automatically resolves any relative hrefs against
     * the NewsML document's base URL.</p>
     *
     * @param href A (possibly relative) URL to resolve.
     * @return The href resolved against the current document's base URL.
     * @exception MalformedURLException if the href cannot be resolved
     * into a URL (this can be caught as an {@link IOException}).
     * @see #getBaseURL()
     */
    public URL resolveHref (String href)
	throws MalformedURLException;
    
}

// end of NewsMLSession.java
