// NewsManagement.java - management information for a news item.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsManagement.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Management information for a news item.
 *
 * <p>The NewsManagement class provides information about the context
 * of a {@link NewsItem}, including the time when the item was first
 * created, the time when the current revision was created, the
 * current status, a scheduled future status change, the urgency, and
 * its relationship to other news items.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface NewsManagement
    extends BaseNode, IdNode, PropertyNode
{

    /**
     * Get the type of the news item.
     *
     * @return A formal name representing the type (not null).
     */
    public FormalName getNewsItemType ();


    /**
     * Get the date when the news item was first created.
     *
     * @return The date of first creation as text in ISO 8601 basic
     * format (not null).
     * @see #getThisRevisionCreated
     */
    public IdText getFirstCreated ();


    /**
     * Get the creation date of the current revision of the news item.
     *
     * @return The creation date as text in ISO 8601 basic format
     * (not null).
     * @see #getFirstCreated
     */
    public IdText getThisRevisionCreated ();


    /**
     * Get the current status of the news item.
     *
     * @return A formal name representing the status (not null).
     */
    public FormalName getStatus ();


    /**
     * Get a schedule future status change.
     *
     * @return An object representing a future status change, or null
     * if none was specified.
     * @see #getStatusWillChange
     */
    public StatusWillChange getStatusWillChange ();


    /**
     * Get the news item's urgency.
     *
     * @return A formal name representing the urgency, or null if none
     * was specified.
     * @see #getStatus
     */
    public FormalName getUrgency ();


    /**
     * Get a pointer to the news item's revision history.
     *
     * @return A string containing the URL of a revision-history document
     * for the news item, or null if none was specified.
     */
    public RevisionHistory getRevisionHistory ();


    /**
     * Count the derived-from objects.
     *
     * @return The number of derived-from objects present.
     */
    public int getDerivedFromCount ();


    /**
     * Get a reference to a news item from which this item is derived.
     *
     * @param index The index of the reference, zero-based, in sequential
     * order.
     * @return An object containing the reference, or null if none was
     * provided with the specified index.
     */
    public NewsItemRef getDerivedFrom (int index);


    /**
     * Get all DerivedFrom children in an array.
     *
     * @return A (possibly-empty) array of DerivedFrom children.
     */
    public NewsItemRef [] getDerivedFrom ();


    /**
     * Count the associated-with objects.
     *
     * @return The number of associated-with objects present.
     */
    public int getAssociatedWithCount ();


    /**
     * Get a reference to a news item associated with this one.
     *
     * @param index The index of the reference, zero-based, in sequential
     * order.
     * @return An object containing the reference, or null if none was
     * provided with the specified index.
     */
    public NewsItemRef getAssociatedWith (int index);


    /**
     * Get all AssociatedWith children in an array.
     *
     * @return A (possibly-empty) array of AssociatedWith children.
     */
    public NewsItemRef [] getAssociatedWith ();


    /**
     * Count the instructions.
     *
     * @return The number of instructions present.
     */
    public int getInstructionCount ();


    /**
     * Get a user-defined news-management instruction.
     *
     * @param index The index of the instruction, zero-based, in sequential
     * order.
     * @return An object containing the instruction, or null if none was
     * provided with the specified index.
     */
    public Instruction getInstruction (int index);


    /**
     * Get all Instruction children in an array.
     *
     * @return A (possibly-empty) array of Instruction children.
     */
    public Instruction [] getInstruction ();

}

// end of NewsManagement.java
