// OriginNode.java - mixed content text and Origin elements.
// Copyright (c) 2001 by Reuters PLC.
// $Id: OriginNode.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Mixed content text and Origin elements.
 *
 * <p>This interface provides a base for specialized text (possibly)
 * containing mixed content: text mixed in with Origin subelements
 * that can be made into hyperlinks.  It is still possible to treat
 * this as simple text, but this interface exposes the Origin
 * subelements for applications that need them.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface OriginNode
    extends IdText
{

    /**
     * Constant: a chunk contains text.
     */
    public final static int TEXT = 1;


    /**
     * Constant: a chunk is an origin element.
     */
    public final static int ORIGIN = 2;


    /**
     * Get the number of separate content chunks.
     *
     * <p>Note that text may be broken down into an arbitrary
     * number of chunks by the DOM implementation: it is not
     * safe to assume that all contiguous text will appear in the
     * same chunk.</p>
     *
     * @return The number of content chunks (0 for an empty element).
     */
    public int getChunkCount ();


    /**
     * Get the type of a chunk.
     *
     * @param index The index of the chunk.
     * @return One of {@link #TEXT} or {@link #ORIGIN}, representing
     * the chunk's type, or -1 if the index is out of range.
     */
    public int getChunkType (int index);


    /**
     * Get a chunk as plain text.
     *
     * <p>For any index, only one of this method and {@link
     * #getOrigin} may return non-null</p>
     *
     * @param index The index of the chunk.
     * @return The chunk as a string, or null if the chunk is an
     * Origin element or the index is out of range.
     * @see #getOrigin
     */
    public String getData (int index);

    
    /**
     * Get a chunk as an Origin subelement.
     *
     * @param index The index of the chunk.
     * @return The chunk as an Origin object, or null if the
     * chunk is text or if the index is out of range.
     */
    public Origin getOrigin (int index);

}

// end of OriginNode.java
