// SubjectCode.java - an IPTC subject code.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: SubjectCode.java,v 1.7 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * An IPTC subject code from the IPTC Information Interchange Model (IIM).
 *
 * <p>The subject code is part of the {@link DescriptiveMetadata} for
 * a {@link NewsComponent}.</p>
 *
 * <p>Although the NewsML documentation limits this property to IPTC
 * subject codes, the fact that the codes are formal names implies
 * that other vocabularies could be used; programmers should take care
 * using this interface until the IPTC clarifies the situation, since
 * providers may choose to take a liberal interpretation.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface SubjectCode
    extends AssignmentNode, BaseNode, IdNode
{

    /**
     * Count the subjects in the code.
     *
     * @return The number of subjects.
     * @see #getSubject(int)
     * @see #getSubject()
     */
    public int getSubjectCount ();


    /**
     * Get the IIM code for the subject of a news item.
     *
     * @param index The index of the subject, zero-based, in document
     * order.
     * @return The subject, or null if none exists at the specified
     * index.
     * @see #getSubjectCount()
     * @see #getSubject()
     */
    public AssignedFormalName getSubject (int index);


    /**
     * Get an array of all subjects in the code.
     *
     * @return A possibly-empty array of subjects (not null).
     * @see #getSubjectCount()
     * @see #getSubject(int)
     */
    public AssignedFormalName [] getSubject ();


    /**
     * Count the subject matter in the code.
     *
     * @return The number of subject matter objects.
     * @see #getSubjectMatter(int)
     * @see #getSubjectMatter()
     */
    public int getSubjectMatterCount ();


    /**
     * Get the IIM code for the subject matter of a news item.
     *
     * @param index The index of the subject matter, zero-based, in
     * document order.
     * @return The subject matter, or null if none exists at the
     * specified index.
     * @see #getSubjectMatterCount()
     * @see #getSubjectMatter()
     */
    public AssignedFormalName getSubjectMatter (int index);


    /**
     * Get an array of all subject matter in the code.
     *
     * @return A possibly-empty array of subject matters (not null).
     * @see #getSubjectMatterCount()
     * @see #getSubjectMatter(int)
     */
    public AssignedFormalName [] getSubjectMatter ();


    /**
     * Count the subject details in the code.
     *
     * @return The number of subject details.
     * @see #getSubjectDetail(int)
     * @see #getSubjectDetail()
     */
    public int getSubjectDetailCount ();


    /**
     * Get the IIM code for a subject detail of a news item.
     *
     * @param index The index of the subject detail, zero-based, in
     * document order.
     * @return The subject detail, or null if none exists at the
     * specified index.
     * @see #getSubjectDetailCount()
     * @see #getSubjectDetail()
     */
    public AssignedFormalName getSubjectDetail (int index);


    /**
     * Get an array of all subject details in the code.
     *
     * @return A possibly-empty array of subject details (not null).
     * @see #getSubjectDetailCount()
     * @see #getSubjectDetail(int)
     */
    public AssignedFormalName [] getSubjectDetail ();


    /**
     * Count the subject qualifiers in the code.
     *
     * @return The number of subject qualifiers.
     * @see #getSubjectQualifier(int)
     * @see #getSubjectQualifier()
     */
    public int getSubjectQualifierCount ();


    /**
     * Get the IIM code for a subject qualifier.
     *
     * <p>Note: since every subject, subject matter, or subject detail
     * may have zero or more qualifiers, the index of the qualifier will
     * not likely have any correspondence to the index of the thing
     * it qualifiers; to obtain qualifiers grouped with the things they
     * qualify, use the {@link #getItem} method.</p>
     *
     * @param index The index of the subject qualifier, zero-based, in
     * document order.
     * @return The subject qualifier, or null if none exists at the
     * specified index.
     * @see #getSubjectQualifierCount()
     * @see #getSubjectQualifier()
     */
    public AssignedFormalName getSubjectQualifier (int index);


    /**
     * Get an array of all subject qualifiers in the code.
     *
     * @return A possibly-empty array of subject qualifiers (not
     * null).
     * @see #getSubjectQualifierCount()
     * @see #getSubjectQualifier(int)
     */
    public AssignedFormalName [] getSubjectQualifier ();


    /**
     * Convenience method: count the items available in this subject code.
     *
     * @return The number of items available.
     * @see #getItem
     */
    public int getItemCount ();


    /**
     * Convenience method: get an item from the subject code.
     *
     * <p>The item includes a single piece of Subject, SubjectMatter,
     * or Subject information together with zero or more
     * SubjectQualifiers.</p>
     *
     * @param index The index of the item, zero-based, numbered
     * sequentially.
     * @return The item, or null if none was provided at the specified
     * index.
     */
    public SubjectCodeItem getItem (int index);


    /**
     * Convenience method: get all Item children in an array.
     *
     * @return A (possibly-empty) array of Item children.
     */
    public SubjectCodeItem [] getItem ();

}

// end of SubjectCode.java
