// SubjectCodeItem.java - a single qualified item in a subject code.
// Copyright (c) 2001 by Reuters PLC
// $Id: SubjectCodeItem.java,v 1.6 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * A single qualified item in a subject code.
 *
 * <p>This interface represents the grouping of Subject,
 * SubjectDetail, or SubjectMatter elements with SubjectQualifier
 * elements.  This grouping is implied by ordering, and is not
 * represented by an explicit XML element in a NewsML document; as a
 * result, this interface does not extend {@link BaseNode}.</p>
 *
 * <p>Only one of {@link #getSubject}, {@link #getSubjectMatter}, and
 * {@link #getSubjectDetail} will return non-null.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see SubjectCode
 */
public interface SubjectCodeItem
{

    /**
     * Get the IIM code for the subject of a news item.
     *
     * @return A formal name representing the IIM code, or null if
     * this item contains SubjectMatter or a SubjectDetail instead.
     * @see #getSubjectMatter
     * @see #getSubjectDetail
     */
    public AssignedFormalName getSubject ();


    /**
     * Get the IIM code for the subject matter of a news item.
     *
     * @return A formal name representing the IIM code, or null if
     * this item contains a Subject or SubjectDetail instead.
     * @see #getSubject
     * @see #getSubjectDetail
     */
    public AssignedFormalName getSubjectMatter ();


    /**
     * Get the IIM code for a subject detail of a news item.
     *
     * @return A formal name representing the IIM code, or null if
     * this item contains a Subject or SubjectMatter instead.
     * @see #getSubject
     * @see #getSubjectMatter
     */
    public AssignedFormalName getSubjectDetail ();


    /**
     * Count the subject qualifiers.
     *
     * @return The number of subject qualifiers present.
     */
    public int getSubjectQualifierCount ();


    /**
     * Get the IIM code for a subject qualifier.
     *
     * <p>Note that the index applies only to qualifiers immediately
     * following the current Subject, SubjectMatter, or SubjectDetail
     * element in the XML document.</p>
     *
     * @param The index of the subject qualifier, zero-based, numbered
     * sequentially.
     * @return A formal name representing the IIM code, or null if
     * none was provided at the specified index.
     */
    public AssignedFormalName getSubjectQualifier (int index);


    /**
     * Get an array of all subject qualifiers present.
     *
     * @return A (possibly empty) array of subject qualifiers.
     */
    public AssignedFormalName [] getSubjectQualifier ();

}

// end of SubjectCodeItem.java
