// Topic.java - basis for choosing among equivalents.
// Copyright (c) 2001 by Reuters PLC.
// $Id: Topic.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * A single item of metadata.
 *
 * <p>This interface currently omits the following information:</p>
 *
 * <ul>
 * <li>xml:lang, and Variant for the Description subelements.</li>
 * </ul>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface Topic
    extends BaseNode, CatalogNode, CommentNode, IdNode, PropertyNode
{

    /**
     * Get a URI pointing to additional details about the topic.
     *
     * @return Text containing a URI, or null if none was provided.
     */
    public Text getDetails ();


    /**
     * Count the types.
     *
     * @return The number of topic types present.
     */
    public int getTopicTypeCount ();


    /**
     * Get the type of the topic.
     *
     * <p>The type is roughly equivalent to a class.</p>
     *
     * @param index The index of the topic type, zero-based, numbered
     * sequentially in document order.
     * @return A formal name representing the topic's type, or null
     * if none was present at the specified index (the first type
     * must not be null).
     */
    public FormalName getTopicType (int index);

    
    /**
     * Get all TopicType children in an array.
     *
     * @return A (possibly-empty) array of TopicType children.
     */
    public FormalName [] getTopicType ();


    /**
     * Count the formal names.
     *
     * @return The number of formal names present.
     */
    public int getFormalNameDeclCount ();


    /**
     * Get one of the topic's formal names.
     *
     * @param index The index of the formal name, zero-based, numbered
     * sequentially in document order.
     * @return A formal name declaration, or null if none was provided
     * at the specified index.
     */
    public FormalNameDecl getFormalNameDecl (int index);


    /**
     * Get all FormalName children in an array.
     *
     * @return A (possibly-empty) array of FormalName children.
     */
    public FormalNameDecl [] getFormalNameDecl ();


    /**
     * Count the descriptions.
     *
     * @return The number of descriptions present.
     */
    public int getDescriptionCount ();


    /**
     * Get one of the topic's descriptions.
     *
     * <p>If there are multiple descriptions, the variant property may
     * help to distinguish them; see {@link
     * Description#getVariant}.</p>
     *
     * @param index The index of the description, zero-based, numbered
     * sequentially in document order.
     * @return A description object, or null if none was provided at
     * the specified index.
     */
    public Description getDescription (int index);


    /**
     * Get all Description children in an array.
     *
     * @return A (possibly-empty) array of Description children.
     */
    public Description [] getDescription ();

}

// end of Topic.java
