// DateTimeTest.java - test the value of DateTime attributes.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DateTimeTest.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsMLException;


/**
 * Test date/time format.
 *
 * <p>The NewsML DTD sets out the required format, which it mistakenly
 * refers to as ISO 8601 Basic format; in fact, it's a very restricted
 * subset.  This test may be used on any text node, element or
 * attribute.  In addition to checking the format, it runs some 
 * sanity checks on the date itself, and will report a warning
 * for years outside the range 1900:2100.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLTestManager
 */
public class DateTimeTest
    extends TestBase
{


    /**
     * Run tests for a text node containing ISO 8601 Date/Time data.
     *
     * @param node A Text node.
     * @param useExternal Not used.
     * @exception NewsMLException If the test fails.
     * @see TestBase#run
     */
    public void run (BaseNode node, boolean useExternal)
	throws NewsMLException
    {
	String date = node.toString();
	if (!Util.isISODate(date))
	    error("Date does not follow NewsML ISO 8601 subset: " + date);

	int year = Integer.parseInt(date.substring(0, 4));
	int month = Integer.parseInt(date.substring(4, 6));
	int day = Integer.parseInt(date.substring(6, 8));

	if (year < 1900 || year > 2100)
	    warning("Year does not fall in range 1900:2100: " + year);

	if (month < 1 || month > 12)
	    error("Month does not fall in range 1.1beta2: " + month);

	if (day < 1 || day > monthLength(year, month))
	    error("Day does not fall in range 1:"
		  + monthLength(year, month) + ": " + day);

	if (date.length() > 8) {
	    int hour = Integer.parseInt(date.substring(9, 11));
	    int min = Integer.parseInt(date.substring(11, 13));
	    int sec = Integer.parseInt(date.substring(13, 15));
	    int hour_offset = Integer.parseInt(date.substring(16, 18));
	    int min_offset = Integer.parseInt(date.substring(18, 20));

	    if (hour < 0 || hour > 23)
		error("Hour does not fall in range 0:23: " + hour);

	    if (min < 0 || min > 59)
		error("Minute does not fall in range 0:59: " + min);

	    if (sec < 0 || sec > 59)
		error("Second does not fall in range 0:59: " + sec);

	    if (hour_offset < 0 || hour_offset > 12)
		error("Hour GMT offset does not fall in range 0:12: "
		      + hour_offset);

	    if (min_offset < 0 || min_offset> 59)
		error("Minute GMT offset does not fall in range 0:59: "
		      + min_offset);
	}
    }


    /**
     * Calculate the number of days in a month.
     *
     * @param year The year as an integer.
     * @param month The month as an integer from 1.1beta2.
     * @param days The number of days as an integer.
     */
    private int monthLength (int year, int month)
    {
	if (month == 2) {
	    if (((year % 4) == 0) &&
		(((year % 100) != 0) || ((year % 1000) == 0)))
		return 29;
	    else
		return 28;
	} else {
	    return monthLengths[month-1]; // 0-based array
	}
    }


    /**
     * Lengths of the months of the year.
     */
    private final static int monthLengths [] = 
    {
	31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
    };

}

// end of DateTimeTest.java
