// ErrorVisitor.java - custom handler for errors and warnings.
// Copyright (c) 2001 by Reuters PLC.
// $Id: ErrorVisitor.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsMLException;


/**
 * Custom handler for errors and warnings.
 *
 * <p>Applications can implement this handler to control the reporting
 * of errors and warnings through the {@link NewsMLTestManager}.  By
 * default, warnings are printed to stderr and errors cause an exception,
 * but the user-supplied visitor can perform different actions.  If the
 * handlers throw exceptions, they will stop the conformance; otherwise,
 * the manager will continue until the end of the tests.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public interface ErrorVisitor
{

    /**
     * Report a warning.
     *
     * <p>The application can implement this method to receive
     * notification of warnings and perform its own, customized
     * actions in response to them.  If desired, the application
     * can throw an exception to stop the processing.</p>
     *
     * <p>The default implementation prints the warning message
     * to standard error and continues.</p>
     *
     * @param e A NewsMLException containing the warning information.
     */
    public abstract void warning (BaseNode node, NewsMLException e)
	throws NewsMLException;

    /**
     * Report an error.
     *
     * <p>The application can implement this method to receive
     * notification of errors and perform its own, customized
     * actions in response to them.  If desired, the application
     * can throw an exception to stop the processing.</p>
     *
     * <p>The default implementation rethrows the exception and
     * stops processing.</p>
     *
     * @param e A NewsMLException containing the error information.
     */
    public abstract void error (BaseNode node, NewsMLException e)
	throws NewsMLException;

}

// end of ErrorVisitor.java
