// NewsComponentTest.java - test a NewsComponent.
// Copyright (c) 2001 by Reuters PLC.
// $Id: NewsComponentTest.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.EquivalentNode;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsMLException;


/**
 * NewsComponent-specific conformance tests.
 *
 * <p>This class reports an error if a NewsComponent has no Role and
 * does not appear immediately inside a NewsItem, or if any of the
 * news component's members does not have a match for one of the bases
 * for choice.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLTestManager
 */
public class NewsComponentTest
    extends TestBase
{

    /**
     * Run tests for a NewsComponent node.
     *
     * @param node A NewsComponent node.
     * @param useExternal Not used.
     * @exception NewsMLException If the test fails.
     * @exception java.lang.ClassCastException If the node parameter is not
     *   an instance of {@link NewsComponent}. 
     * @see TestBase#run
     */
    public void run (BaseNode node, boolean useExternal)
	throws NewsMLException
    {
	testRole((NewsComponent)node);
	testBasisForChoice((NewsComponent)node);
    }


    /**
     * Test that a role appears where required.
     *
     * @param component The NewsComponent to test.
     * @exception NewsMLException If the test fails.
     * @exception java.lang.ClassCastException If the node parameter is not
     *   an instance of {@link NewsComponent}. 
     */
    private void testRole (NewsComponent component)
	throws NewsMLException
    {
	if (!"NewsItem".equals(component.getParent().getXMLName()) &&
	    component.getRole() == null)
	    warning("Nested NewsComponent has no Role");
    }


    /**
     * Test that each member has a match for each BasisForChoice.
     *
     * @param component The NewsComponent to test.
     * @exception NewsMLException If the test fails.
     * @exception java.lang.ClassCastException If the node parameter is not
     *   an instance of {@link NewsComponent}. 
     */
    private void testBasisForChoice (NewsComponent component)
	throws NewsMLException
    {
	int nBasisForChoice = component.getBasisForChoiceCount();
	int nMembers = component.getMemberCount();
	for (int i = 0; i < nMembers; i++) {
	    EquivalentNode member = component.getMember(i);
	    BaseNode bases[] = member.getBasisForChoiceNodes();
	    for (int j = 0; j < nBasisForChoice; j++) {
		if (bases[j] == null)
		    error("Member " + j + " has no match for BasisForChoice: "
			  + component.getBasisForChoice(j).toString());
	    }
	}
    }

}

// end of NewsComponentTest.java

