// NewsMLTestManager.java - manager for NewsML conformance tests.
// Copyright (c) 2001 by Reuters PLC.
// $Id: NewsMLTestManager.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import java.util.ArrayList;
import java.util.Iterator;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsMLException;


/**
 * Top-level manager for NewsML conformance tests.
 *
 * <p>This manager handles registration, running, and error reporting
 * for NewsML conformance tests.  It can be used with a predefined set
 * of core tests, optionally augmented by user-supplied tests for
 * specific business rules, or exclusively with user-supplied tests.
 * The tests managed by this class are intended to augment the DTD
 * conformance that can be (optionally) performed when the NewsML
 * is loaded from XML.</p>
 *
 * <p>The user can customize error handling by providing an
 * {@link ErrorVisitor} through the {@link #setErrorVisitor}
 * method.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public class NewsMLTestManager
{

    /**
     * Constructor.
     *
     * <p>Build a new manager with no tests registered.</p>
     */
    public NewsMLTestManager ()
    {
	entries = new ArrayList();
	verbose = false;
	errorVisitor = new DefaultErrorVisitor();
    }


    /**
     * Enable or disable the display of messages during processing.
     *
     * @param verbose true if the manager should display progress messages
     * to stdout, false (the default) otherwise.
     */
    public void setVerbose (boolean verbose)
    {
	this.verbose = verbose;
    }


    /**
     * Set the error vistor.
     *
     * <p>The error visitor allows the application to customize error
     * handling during a conformance.</p>
     *
     * @param errorVisitor The new error handler.
     */
    public void setErrorVisitor (ErrorVisitor errorVisitor)
    {
	this.errorVisitor = errorVisitor;
    }


    /**
     * Register a new test for the manager.
     *
     * <p>The caller supplies both a test and an XPath -- the XPath
     * expression will be used to find notes against which the
     * test should be run.</p>
     *
     * @param xpath The XPath for nodes to which the test will
     * be applied.
     * @param test The test itself.
     */
    public void addTest (String xpath, TestBase test)
    {
	entries.add(new Entry(xpath, test));
    }


    /**
     * Add a set of default conformance tests.
     *
     * <p>The default tests check general problems that can occur but
     * are not caught by DTD conformance.</p>
     */
    public void addDefaultTests ()
    {
	//
	// Reusable tests.
	//
	PatternTest positive_int =
	    new PatternTest("^\\d+$", "positive integer");
	PatternTest positive_int_not_zero =
	    new PatternTest("^[1-9]\\d*$",
			    "positive integer greater than zero");
	PatternTest not_empty = new PatternTest(".+", "not empty");
	PatternTest domain_name =
	    new PatternTest("^[a-zA-Z][a-zA-Z0-9-]*(\\.[a-zA-Z][a-zA-Z0-9-]*)+$",
			    "domain name");


	addTest("//node()/@Euid", new EuidTest());
	addTest("//node()/@xml:lang", new XMLLangTest());
	addTest("//node()/@AssignedBy",
			new RefTest(RefTest.DUID, "Topic"));
	addTest("//node()/@DateAndTime", new DateTimeTest());
	addTest("//node()[@FormalName]", new FormalNameTest());
	addTest("//node()/@Importance", new FormalNameTest());
	addTest("//node()/@Confidence", new FormalNameTest());
	addTest("//node()/@HowPresent", new FormalNameTest());
	addTest("//AssociatedWith/@NewsItem",
			new RefTest((RefTest.HTTP_URL | RefTest.NEWSML_URN),
				    "NewsItem"));
	addTest("//DerivedFrom/@NewsItem",
			new RefTest((RefTest.HTTP_URL | RefTest.NEWSML_URN),
				    "NewsItem"));
	addTest("//NewsItemRef/@NewsItem",
			new RefTest((RefTest.HTTP_URL
				     | RefTest.NEWSML_URN
				     | RefTest.DUID),
				    "NewsItem"));
	addTest("//Source/@NewsItem",
			new RefTest(RefTest.NEWSML_URN, "NewsItem"));
	addTest("//BasisForChoice/@Rank", positive_int);
	addTest("//Catalog", new CatalogTest());
	addTest("//Catalog/@Href", new RefTest((RefTest.DUID |
							RefTest.NEWSML_URN |
							RefTest.HTTP_URL),
						       "Catalog"));
	addTest("//ContentItem", new ContentItemTest());
	addTest("//ContentItem/@Href", new RefTest(RefTest.URL));
	addTest("//DateAndTime", new DateTimeTest());
	addTest("//DateId", new PatternTest("^\\d{8}$", "YYYYMMDD"));
	addTest("//DateLabel", not_empty);
	addTest("//DefaultVocabularyFor",
			new DefaultVocabularyForTest());
	addTest("//NameLabel", not_empty);
	addTest("//NewsComponent", new NewsComponentTest());
	addTest("//NewsIdentifier", new NewsIdentifierTest());
	addTest("//NewsItem", new NewsItemTest());
	addTest("//NewsItemId", new FormalNameTest(false));
	addTest("//NewsLine", new NewsLineTest());
	addTest("//Origin/@Href", new RefTest(RefTest.DUID|
						      RefTest.URL));
	addTest("//Party/@Topic", new RefTest((RefTest.DUID|
						RefTest.HTTP_URL|
						RefTest.NEWSML_URN),
					       "Topic"));
	addTest("//Property", new PropertyTest());
	addTest("//Property/@ValueRef",
			new RefTest((RefTest.DUID
				     |RefTest.URL
				     |RefTest.NEWSML_URN)));
	addTest("//Property/@AllowedValues",
			new RefTest((RefTest.DUID
				     |RefTest.HTTP_URL
				     |RefTest.NEWSML_URN),
				    "TopicSet"));
	addTest("//ProviderId", new FormalNameTest(false));
	addTest("//ProviderId", domain_name);
				// FIXME: once URNs can be retrieved,
				// flag this one not to be retrieved
	addTest("//PublicIdentifer", new RefTest(RefTest.NEWSML_URN));
	addTest("//Resource", new ResourceTest());
	addTest("//RevisionHistory", new RefTest(RefTest.URL));
	addTest("//RevisionId", new RevisionIdTest());
	addTest("//RevisionId", positive_int_not_zero);
	addTest("//RevisionId/@PreviousRevision", positive_int);
	addTest("//RevisionStatus", new RevisionStatusTest());
	addTest("//RevisionStatus/@Revision", positive_int_not_zero);
	addTest("//SizeInBytes", positive_int);
	addTest("//Topic", new TopicTest());
	addTest("//Topic/@Details",
			new RefTest(RefTest.URL|RefTest.URN));
	addTest("//TopicOccurrence/@Topic",
			new RefTest(RefTest.DUID, "Topic"));
	addTest("//TopicSet", new TopicSetTest());
	addTest("//TopicSetRef/@TopicSet",
			new RefTest((RefTest.DUID|RefTest.NEWSML_URN
				     |RefTest.HTTP_URL), "TopicSet"));
	addTest("//TopicUse", new TopicUseTest());
	addTest("//TopicUse/@Topic",
			new RefTest(RefTest.DUID, "Topic"));
	addTest("//TransmissionId", not_empty);
	addTest("//TransmissionId/@Repeat", not_empty);
				// FIXME: assuming Duid ref OK
	addTest("//Url", new RefTest(RefTest.URL|RefTest.DUID));
	addTest("//Urn", new RefTest(RefTest.NEWSML_URN));
    }


    /**
     * Run all registered tests against a context node.
     *
     * @param contextNode The context node for matching tests
     * (usually the root NewsML node).
     * @param useExternal true if the tests may attempt to retrieve
     * external resources, false otherwise.
     */
    public void runTests (BaseNode contextNode, boolean useExternal)
	throws NewsMLException
    {
	int nTests = 0;
	if (verbose)
	    System.out.println("Starting tests...");
	Iterator it = entries.iterator();
	while (it.hasNext()) {
	    Entry entry = (Entry)it.next();
	    if (verbose)
		System.out.print("  Trying xpath " + entry.xpath + " ... ");
	    BaseNode nodes[] = contextNode.getSession()
		.getNodesByXPath(contextNode, entry.xpath);
	    if (verbose) {
		System.out.println(nodes.length + " match(es)");
		nTests += nodes.length;
	    }
	    for (int i = 0; i < nodes.length; i++) {
		if (nodes[i] != null) {
		    try {
			entry.test.run(nodes[i], useExternal);
		    } catch (ConformanceWarning w) {
			errorVisitor.warning(nodes[i], w);
		    } catch (NewsMLException e) {
			errorVisitor.error(nodes[i], e);
		    }
		}
	    }
	}
	if (verbose)
	    System.out.println("Done: " + nTests + " tests performed");
    }

				// Inner state.
    private ArrayList entries;
    private boolean verbose;
    private ErrorVisitor errorVisitor;



    ////////////////////////////////////////////////////////////////////
    // Inner class for an XPath/test pair.
    ////////////////////////////////////////////////////////////////////

    /**
     * An Xpath/test pair.
     */
    class Entry
    {
	Entry (String xpath, TestBase test)
	{
	    this.xpath = xpath;
	    this.test = test;
	}
	String xpath;
	TestBase test;
    }



    ////////////////////////////////////////////////////////////////////
    // Inner class for the default error handler.
    ////////////////////////////////////////////////////////////////////


    /**
     * The default visitor.
     */
    class DefaultErrorVisitor
	implements ErrorVisitor
    {
	public void warning (BaseNode node, NewsMLException e)
	{
	    System.err.println("Warning: " + e.getMessage()
			       + "\n  at " + node.getXPath());
	}
	public void error (BaseNode node, NewsMLException e)
	    throws NewsMLException
	{
	    System.err.println("Error: " + e.getMessage()
			       + "\n  at " + node.getXPath());
	}
    }

}

// end of NewsMLTestManager.java
