// RefTest.java - test a Duid, URN, or URL reference.
// Copyright (c) 2001 by Reuters PLC.
// $Id: RefTest.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.Text;


/**
 * Test a reference.
 *
 * <p>This class provides a configurable base for testing Duid, URL,
 * and URN references of various types.  It tests the syntax and
 * attempts to resolve the references where possible and allowed.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLTestManager
 */
public class RefTest
    extends TestBase
{


    /**
     * Constant: Duid references are allowed.
     */
    public final static int DUID = 1;


    /**
     * Constant: NewsML URNs are allowed.
     */
    public final static int NEWSML_URN = 2;


    /**
     * Constant: HTTP URLs are allowed.
     */
    public final static int HTTP_URL = 4;


    /**
     * Constant: General URLs are allowed.
     */
    public final static int URL = 8;


    /**
     * Constant: General URNs are allowed.
     */
    public final static int URN = 16;


    /**
     * Construct a new reference test with the allowed types specified.
     *
     * @param flags The flags for the allowed types, or'ed together.
     */
    public RefTest (int flags)
    {
	this(flags, null);
    }


    /**
     * Construct a new reference test with a specific NewsML target type.
     *
     * @param flags The flags for the allowed types, or'ed together.
     * @param targetName The XML name of the expected NewsML target
     * (should be null when the target is not NewsML markup).
     */
    public RefTest (int flags, String targetName)
    {
	this.flags = flags;
	this.targetName = targetName;
    }


    /**
     * Test a reference.
     *
     * @param node A Text node.
     * @param useExternal If true, try external references.
     * @exception NewsMLException If the test fails.
     * @exception java.lang.ClassCastException If the node parameter is not
     *   an instance of {@link Text}. 
     * @see TestBase#run
     */
    public void run (BaseNode node, boolean useExternal)
	throws NewsMLException
    {
	testSyntax((Text)node);
	testAccess((Text)node, useExternal);
    }



    /**
     * Test the reference syntax.
     *
     * @param node The node to test.
     * @exception NewsMLException if the test fails.
     */
    private void testSyntax (Text node)
	throws NewsMLException
    {
	String ref = node.toString();
	if ((hasFlag(DUID) && Util.isDuidRef(ref)) ||
	    (hasFlag(NEWSML_URN) && Util.isNewsMLUrn(ref)) ||
	    (hasFlag(HTTP_URL) && Util.isHttpUrl(ref)) ||
	    (hasFlag(URL) && Util.isUrl(ref)) ||
	    (hasFlag(URN) && Util.isUrn(ref))) {
	    return;
	} else {
	    StringBuffer message = new StringBuffer();
	    message.append("Reference not one of allowed types");
	    if (hasFlag(DUID))
		message.append(", Duid reference ('#' + XML IDREF)");
	    if (hasFlag(NEWSML_URN))
		message.append(", NewsML URN");
	    if (hasFlag(HTTP_URL))
		message.append(", HTTP URL");
	    if (hasFlag(URL))
		message.append(", general URL (RFC 1738)");
	    if (hasFlag(URN))
		message.append(", URN (RFC 2141)");
	    message.append(": ");
	    message.append(ref);
	    error(message.toString());
	}
    }


    /**
     * Test whether a resource is accessible.
     *
     * <p>This test will honor the useExternal flag.</p>
     *
     * @param node The node to test.
     * @param useExternal true if external access is allowed.
     * @exception NewsMLException if the test fails.
     */
    private void testAccess (Text node, boolean useExternal)
	throws NewsMLException
    {
	String ref = node.toString();

	if (hasFlag(DUID) && Util.isDuidRef(ref)) {
	    BaseNode target = node.getSession().getNodeByDuid(ref);
	    if (target == null)
		error("No node with Duid: " + ref);
	    if (targetName != null) {
		if (!targetName.equals(target.getXMLName()))
		    error("Expected target node " + targetName +
			  " but found " + target.getXMLName());
	    }
	}

	else if (hasFlag(NEWSML_URN) && Util.isNewsMLUrn(ref)) {
	    // no test yet
	}

	else if (hasFlag(HTTP_URL) && Util.isHttpUrl(ref)) {
	    // no test yet
	}

	else if (hasFlag(URL) && Util.isUrl(ref)) {
	    // no test yet
	}

	else if (hasFlag(URN) && Util.isUrn(ref)) {
	    // no test yet
	}

    }


    /**
     * Test whether a flag is set for this test.
     *
     * @param flag The flag to test.
     * @return true if the flag is set, false otherwise.
     */
    private boolean hasFlag (int flag)
    {
	return ((flags&flag) == flag);
    }


				// Internal state.
    private int flags;
    private String targetName;


}

// end of RefTest.java
