// TopicTest.java - test a Topic.
// Copyright (c) 2001 by Reuters PLC.
// $Id: TopicTest.java,v 1.2 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.conformance;

import java.io.IOException;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.FormalNameNode;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.Text;
import org.newsml.toolkit.Topic;


/**
 * Topic-specific conformance tests.
 *
 * <p>Report a warning if the topic is not either (a) referenced by
 * Duid or (b) used by a formal name.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLTestManager
 */
public class TopicTest
    extends TestBase
{

    /**
     * Run tests for a Topic node.
     *
     * @param node A Topic node.
     * @param useExternal Not used.
     * @exception NewsMLException If the test fails.
     * @exception java.lang.ClassCastException If the node parameter is not
     *   an instance of {@link Topic}. 
     * @see TestBase#run
     */
    public void run (BaseNode node, boolean useExternal)
	throws NewsMLException
    {
	String name = "[no name provided]";
	if (((Topic)node).getFormalNameDeclCount() > 0)
	    name = ((Topic)node).getFormalNameDecl(0).toString();
	if (hasDuidRef((Topic)node) || hasFormalName((Topic)node))
	    return;
	else
	    warning("Unused topic: " + name);
    }


    /**
     * Test whether this topic is the target of a Duid ref.
     *
     * @param topic The topic to test.
     * @return true if the topic is the target of a Duid ref from
     * elsewhere in the document, false otherwise.
     * @exception NewsMLException if there is an error looking for
     * the Duid references.
     */
    private boolean hasDuidRef (Topic topic)
	throws NewsMLException
    {
	Text duid = topic.getDuid();
	if (duid == null)
	    return false;

	BaseNode matches[] = topic.getSession()
	    .getNodesByXPath(topic,
			     "//node()[@Topic=\"#" + duid.toString() + "\"]");
	if (matches.length > 0)
	    return true;

	matches = topic.getSession()
	    .getNodesByXPath(topic,
			     "//node()[@AssignedBy=\"#" + duid.toString()
			     + "\"]");
	if (matches.length > 0)
	    return true;

	return false;
    }


    /**
     * Test whether this topic is referenced by a formal name.
     *
     * @param topic The topic to test
     * @return true if the topic is used by a formal name, false otherwise.
     * @exception NewsMLException if there is an error searching
     * for references.
     */
    private boolean hasFormalName (Topic topic)
	throws NewsMLException
    {
	int nDecl = topic.getFormalNameDeclCount();

	for (int i = 0; i < nDecl; i++) {
	    String name = topic.getFormalNameDecl(i).toString();

	    if (findFormalName(topic,
			       "//node()[@FormalName=\"" + name + "\"]") ||
		findFormalName(topic,
			       "//node()/@Importance[text()=\"" + name+ "\"]") ||
		findFormalName(topic,
			       "//node()/@Confidence[text()=\"" + name + "\"]") ||
		findFormalName(topic,
			       "//node()/@HowPresent[text()=\"" + name + "\"]") ||
		findFormalName(topic,
			       "//NewsItemId[text()=\"" + name + "\"]") ||
		findFormalName(topic,
			       "//ProviderId[text()=\"" + name + "\"]"))

		return true;
	}


	return false;
    }


    /**
     * Find a formal name reference in a collection of hits.
     *
     * @param topic The topic to search for.
     * @param xpath The xpath for formal names that might apply.
     * @return true if a match is found, false otherwise.
     * @exception NewsMLException if there is an error searching
     * the names.
     */
    static boolean findFormalName (Topic topic, String xpath)
	throws NewsMLException
    {
	BaseNode matches[];
	try {
	    matches = topic.getSession()
		.getNodesByXPath(topic, xpath.toString());
	} catch (IOException e) {
	    throw new NewsMLException(e);
	}

	for (int i = 0; i < matches.length; i++) {
	    FormalNameNode formalName = (FormalNameNode)matches[i];
	    try {
		if (topic.equals(formalName.findTopic(false)))
		    return true;
	    } catch (IOException e) {
		throw new NewsMLException(e);
	    }
	}

	return false;
    }

}

// end of TopicTest.java
