// DOMBaseNode.java - base functionality for all NewsML objects.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMBaseNode.java,v 1.17 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.newsml.toolkit.AssignedFormalName;
import org.newsml.toolkit.AssignedOriginText;
import org.newsml.toolkit.AssignmentNode;
import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.BasisForChoice;
import org.newsml.toolkit.Catalog;
import org.newsml.toolkit.CatalogNode;
import org.newsml.toolkit.Comment;
import org.newsml.toolkit.CommentNode;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.HrefNode;
import org.newsml.toolkit.IdNode;
import org.newsml.toolkit.LanguageNode;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLSession;
import org.newsml.toolkit.OriginText;
import org.newsml.toolkit.Party;
import org.newsml.toolkit.PartyList;
import org.newsml.toolkit.Property;
import org.newsml.toolkit.PropertyNode;
import org.newsml.toolkit.RevisionHistory;
import org.newsml.toolkit.Text;
import org.newsml.toolkit.Topic;
import org.newsml.toolkit.TopicNode;
import org.newsml.toolkit.TopicOccurrence;


/**
 * Base class for all NewsML objects.
 *
 * <p>This base class provides access to the underlying DOM node,
 * including a series of convenience methods based on the {@link
 * DOMUtils} class.  The base class also contains an {@link #equals}
 * method that compares the underlying DOM nodes rather than the
 * wrapper classes.</p>
 *
 * <p>Additionally, since Java does not allow multiple inheritance,
 * this class provides default implementations for all of the methods
 * declared in the common *Node interfaces (except for OriginNode).
 * The interfaces will hide unused methods from the user.</p>
 *
 * <p>Finally, this class provides convenience methods for accessing
 * the underlying DOM tree.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
class DOMBaseNode
    implements AssignmentNode, BaseNode, CatalogNode, CommentNode,
	       HrefNode, IdNode, LanguageNode, PropertyNode,
	       RevisionHistory, TopicNode, TopicOccurrence
{

    /**
     * Constructor.
     *
     * @param session The NewsML session for the package.
     */
    protected DOMBaseNode (Node node, DOMNewsMLSession session)
    {
	if (node == null)
	    throw new NullPointerException("null DOM Element node");
	this.node = node;
	this.session = session;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the AssignmentNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see AssignmentNode#getAssignedBy()
     */
    public Text getAssignedBy ()
    {
	return getAttribute("AssignedBy");
    }


    /**
     * @see AssignmentNode#getImportance()
     */
    public FormalName getImportance ()
    {
	return new DOMFormalName(node, session, "Importance");
    }


    /**
     * @see AssignmentNode#getConfidence()
     */
    public FormalName getConfidence ()
    {
	return new DOMFormalName(node, session, "Confidence");
    }


    /**
     * @see AssignmentNode#getHowPresent()
     */
    public FormalName getHowPresent ()
    {
	return new DOMFormalName(node, session, "HowPresent");
    }


    /**
     * @see AssignmentNode#getAssignmentDateAndTime()
     */
    public Text getAssignmentDateAndTime ()
    {
	return getAttribute("DateAndTime");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the BaseNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see BaseNode#getXMLName
     */
    public String getXMLName ()
    {
	switch (node.getNodeType()) {
	case Node.ELEMENT_NODE:
	    return node.getNodeName();
	case Node.ATTRIBUTE_NODE:
	    return "@" + node.getNodeName();
	default:
	    throw new RuntimeException("Illegal DOM node type");
	}
    }


    /**
     * @see BaseNode#getXPath
     */
    public String getXPath ()
    {
	String name = getXMLName();
	int index = -1;
	StringBuffer buf = new StringBuffer();
	BaseNode parent = getParent();
	if (parent != null) {
	    buf.append(parent.getXPath());
	    if ( !name.startsWith("@")) {
		int nChildren = parent.getChildCount(name);
		for (int i = 0; i < nChildren; i++) {
		    if (equals(parent.getChild(name, i))) {
			index = i;
			break;
		    }
		}
	    }
	}
	buf.append('/');
	buf.append(getXMLName());
	if (index != -1) {
	    buf.append('[');
	    buf.append(index);
	    buf.append(']');
	}
	return buf.toString();
    }


    /**
     * @see BaseNode#getSession
     */
    public NewsMLSession getSession ()
    {
	return session;
    }


    /**
     * @see BaseNode#getParent()
     */ 
    public BaseNode getParent ()
    {
	Node parent;
	switch (node.getNodeType()) {
	case Node.ELEMENT_NODE:
	    return session.makeNewsMLNode(node.getParentNode());
	case Node.ATTRIBUTE_NODE:
	    return session.makeNewsMLNode(((Attr)node).getOwnerElement());
	default:
	    throw new RuntimeException("Illegal DOM node type");
	}
    }


    /**
     * @see BaseNode#getChildCount()
     */
    public int getChildCount ()
    {
				// Default to counting all element children.
	NodeList domChildren = node.getChildNodes();
	int matches = 0;
	int nDOMChildren = domChildren.getLength();
	for (int i = 0; i < nDOMChildren; i++) {
	    if (domChildren.item(i).getNodeType() == Node.ELEMENT_NODE) {
		matches++;
	    }
	}
	return matches;
    }


    /**
     * @see BaseNode#getChildCount(String)
     */
    public int getChildCount (String xmlName)
    {
				// Defined this way so that it will work
				// with derived classes that override
				// only getChildCount() and
				// getChild(int)
	int nChildren = getChildCount();
	int matches = 0;
	for (int i = 0; i < nChildren; i++) {
	    BaseNode child = getChild(i);
	    if (child != null && child.getXMLName().equals(xmlName))
		matches++;
	}
	return matches;
    }


    /**
     * @see BaseNode#getChild(int)
     */
    public BaseNode getChild (int index)
    {
				// Default to using all element children.
	NodeList domChildren = node.getChildNodes();
	int matches = 0;
	int nDOMChildren = domChildren.getLength();
	for (int i = 0; i < nDOMChildren; i++) {
	    Node node = domChildren.item(i);
	    if (node.getNodeType() == Node.ELEMENT_NODE) {
		if (matches == index)
		    return session.makeNewsMLNode(node);
		else
		    matches++;
	    }
	}
	return null;
    }


    /**
     * @see BaseNode#getChild(String, int)
     */
    public BaseNode getChild (String xmlName, int index)
    {
				// Defined this way so that it will work
				// with derived classes that override
				// only getChildCount() and
				// getChild(int)
	int nChildren = getChildCount();
	int matches = 0;
	for (int i = 0; i < nChildren; i++) {
	    BaseNode child = getChild(i);
	    if (child != null && child.getXMLName().equals(xmlName)) {
		if (index == matches)
		    return child;
		else
		    matches++;
	    }
	}
	return null;
    }


    /**
     * @see BaseNode#getChild()
     */
    public BaseNode [] getChild ()
    {
				// Defined this way so that it will work
				// with derived classes that override
				// only getChildCount() and
				// getChild(int)
	int count = getChildCount();
	BaseNode ret[] = new BaseNode[count];
	for (int i = 0; i < count; i++)
	    ret[i] = getChild(i);
	return ret;
    }


    /**
     * @see BaseNode#getChild()
     */
    public BaseNode [] getChild (String xmlName)
    {
				// Defined this way so that it will work
				// with derived classes that override
				// only getChildCount() and
				// getChild(int)
	int count = getChildCount(xmlName);
	BaseNode ret[] = new BaseNode[count];
	for (int i = 0; i < count; i++)
	    ret[i] = getChild(xmlName, i);
	return ret;
    }


    /**
     * @see BaseNode#removeSelf
     */
    public void removeSelf ()
	throws NewsMLException
    {
	if (getParent() == null)
	    throw new NewsMLException("Cannot remove root node " +
				      getXMLName());

	switch (node.getNodeType()) {
	case Node.ELEMENT_NODE:
	    try {
		DOMUtils.removeNode(node);
	    } catch (DOMException e) {
		throw new NewsMLException(e);
	    }
	    return;
	case Node.ATTRIBUTE_NODE:
	    throw new NewsMLException("Removing attributes not supported");
	default:
	    throw new RuntimeException("Bad DOM node type.");
	}
    }


    /**
     * @see BaseNode#getNodePath
     */
    public BaseNode [] getPath ()
    {
	int count = 0;
	BaseNode node = this;
	while (node != null) {
	    count++;
	    node = node.getParent();
	}
	BaseNode ret[] = new BaseNode[count];
	node = this;
	for (int i = count - 1; i >= 0; i--) {
	    ret[i] = node;
	    node = node.getParent();
	}
	return ret;
    }


    /**
     * @see BaseNode#writeXML(Writer,boolean)
     */
    public void writeXML (Writer output, boolean isDocument)
	throws IOException
    {
	DOMUtils.writeNode(node, output, isDocument, null, null);
    }


    /**
     * @see BaseNode#writeXML(Writer,String,String)
     */
    public void writeXML (Writer output, String encoding,
			  String internalSubset)
	throws IOException
    {
	DOMUtils.writeNode(node, output, true, encoding, internalSubset);
    }


    /**
     * @see BaseNode#toXML(boolean)
     */
    public String toXML (boolean isDocument)
    {
	StringWriter writer = new StringWriter();
	try {
	    writeXML(writer, isDocument);
	} catch (IOException e) {
	    // shouldn't happen
	}
	return writer.toString();
    }


    /**
     * @see BaseNode#toXML(String,String)
     */
    public String toXML (String encoding, String internalSubset)
    {
	StringWriter writer = new StringWriter();
	try {
	    writeXML(writer, encoding, internalSubset);
	} catch (IOException e) {
	    // shouldn't happen
	}
	return writer.toString();
    }


    /**
     * Not yet in base node -- get an attribute value.
     *
     * @param name The attribute name (without leading '@').
     * @return A text node containing the attribute, if present.
     */
    public Text getAttribute (String name)
    {
	return (Text)(session.makeNewsMLNode(getDOMAttribute(name)));
    }
	



    ////////////////////////////////////////////////////////////////////
    // Implementation of the CatalogNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see CatalogNode#getCatalog()
     */
    public Catalog getCatalog ()
    {
	return (Catalog)getChild("Catalog", 0);
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the CommentNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see CommentNode#getCommentCount
     */
    public int getCommentCount ()
    {
	return getChildCount("Comment");
    }


    /**
     * @see CommentNode#getComment(int)
     */
    public Comment getComment (int index)
    {
	return (Comment)getChild("Comment", index);
    }


    /**
     * @see CommentNode#getComment()
     */
    public Comment [] getComment ()
    {
	int len = getCommentCount();
	Comment ret[] = new Comment[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getComment(i);
	return ret;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the EquivalentNode interface.
    ////////////////////////////////////////////////////////////////////

    /**
     * @see EquivalentNode#getBasisForChoiceNodes.
     */
    public BaseNode [] getBasisForChoiceNodes ()
	throws NewsMLException
    {
	BasisForChoice bases[] =
	    ((NewsComponent)getParent()).getRankedBasisForChoice();
	BaseNode result[] = new BaseNode[bases.length];
	for (int i = 0; i < bases.length; i++) {
	    BaseNode matches[] =
		session.getNodesByXPath(this, bases[i].toString());
	    if (matches.length > 0)
		result[i] = matches[0];
	}
	return result;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the HrefNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see HrefNode#getHref()
     */
    public Text getHref ()
    {
	return getAttribute("Href");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the IdNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see IdNode#getDuid()
     */
    public Text getDuid ()
    {
	return getAttribute("Duid");
    }


    /**
     * @see IdNode#getEuid()
     */
    public Text getEuid ()
    {
	return getAttribute("Euid");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the LanguageNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see LanguageNode#getLang()
     */
    public Text getLang ()
    {
	return getAttribute("xml:lang");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the PropertyNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see PropertyNode#getPropertyCount()
     */
    public int getPropertyCount ()
    {
	return getChildCount("Property");
    }


    /**
     * @see PropertyNode#getProperty(int)
     */
    public Property getProperty (int index)
    {
	return (Property)getChild("Property", index);
    }


    /**
     * @see PropertyNode#getProperty()
     */
    public Property [] getProperty ()
    {
	int len = getPropertyCount();
	Property ret[] = new Property[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getProperty(i);
	return ret;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the TopicNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see TopicNode#getTopicRef
     */
    public Text getTopicRef ()
    {
	return getAttribute("Topic");
    }


    /**
     * @see TopicNode#getReferencedTopic
     */
    public Topic getReferencedTopic (boolean useExternal)
	throws IOException
    {
	BaseNode node = null;
	Text ref = getTopicRef();
	if (ref != null)
	    node = session.findReference(ref.toString(), useExternal);

	if (node != null) {
	    if (!"Topic".equals(node.getXMLName()))
		throw new IOException("Topic reference points to " +
				      node.getXMLName());
	    else
		return (Topic)node;
	}

	return null;
    }



    ////////////////////////////////////////////////////////////////////
    // Low-level DOM methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Get the underlying DOM node for this NewsML object.
     *
     * The underlying node allows users to query information not
     * available directly through this library.
     *
     * @return The underlying DOM node (not null).
     */
    public Node getDOMNode ()
    {
	return node;
    }


    /**
     * Get the text content of the current element.
     *
     * Return all of the text in the current element and its descendants.
     *
     * @return The text as a string.
     * @see DOMUtils#getText(org.w3c.dom.Node)
     */
    public String getDOMContent ()
    {
	switch (node.getNodeType()) {
	case Node.ELEMENT_NODE:
	    return DOMUtils.getText(node);
	case Node.ATTRIBUTE_NODE:
	    return node.getNodeValue();
	default:
	    throw new RuntimeException("Bad DOM node type.");
	}
    }


    /**
     * Get the DOM node for an attribute.
     *
     * @return The DOM node or null.
     */
    public Node getDOMAttribute (String name)
    {
	NamedNodeMap atts = node.getAttributes();
	return (atts == null ? null : atts.getNamedItem(name));
    }



    ////////////////////////////////////////////////////////////////////
    // Override methods from java.lang.Object.
    ////////////////////////////////////////////////////////////////////


    /**
     * Test whether two NewsML objects are equal.
     *
     * Two NewsML objects are equal if they share the same subclass
     * and point to the same underlying DOM node.
     *
     * @param o The object to compare.
     * @return true if the objects are equal, false otherwise.
     */
    public boolean equals (Object o)
    {
	boolean retval;

	if (o == null) {
	    retval = false;
	} else if (o == this) {
	    retval = true;
	} else if (o instanceof DOMBaseNode){
	    retval = (getClass().equals(o.getClass()) &&
		      getDOMNode().equals(((DOMBaseNode)o).getDOMNode()));
	} else {
	    retval = false;
	}

	return retval;
    }


    public String toString ()
    {
	return getXMLName();
    }


    public int hashCode ()
    {
	return node.hashCode();
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private Node node;
    protected DOMNewsMLSession session;

}

// end of DOMBaseNode.java
