// DOMFormalName.java - DOM implementation of AssignedFormalName, FormalName, and OfInterestTo
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMFormalName.java,v 1.9 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;

import org.w3c.dom.Node;

import org.newsml.toolkit.AssignedFormalName;
import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.Catalog;
import org.newsml.toolkit.DefaultVocabularyFor;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.OfInterestTo;
import org.newsml.toolkit.Party;
import org.newsml.toolkit.Property;
import org.newsml.toolkit.Resource;
import org.newsml.toolkit.Text;
import org.newsml.toolkit.Topic;
import org.newsml.toolkit.TopicSet;


/**
 * DOM implementation of AssignedFormalName, FormalName, and OfInterestTo
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
class DOMFormalName
    extends DOMBaseNode
    implements AssignedFormalName, FormalName, OfInterestTo, Party, Property
{

    /**
     * Constructor.
     */
    protected DOMFormalName (Node element, DOMNewsMLSession session)
    {
	super(element, session);
	this.attName = "FormalName";
    }


    /**
     * Constructor.
     */
    protected DOMFormalName (Node element, DOMNewsMLSession session,
			     String attName)
    {
	super(element, session);
	this.attName = attName;
    }


    /**
     * @see FormalName#getName
     */
    public Text getName ()
    {
	return new DOMText(getDOMAttribute(attName), session);
    }


    /**
     * @see FormalName#getVocabulary
     */
    public Text getVocabulary ()
    {
	if (attName.equals("FormalName"))
	    return getAttribute("Vocabulary");
	else
	    return null;
    }


    /**
     * @see FormalName#getDefaultVocabulary
     */
    public String getDefaultVocabulary (boolean useExternal)
	throws IOException
    {
	Node element = getDOMNode();
	Node attribute = element.getAttributes().getNamedItem(attName);
	Text vocab = getVocabulary();
	if (vocab != null)
	    return vocab.toString();

				// Allow matches on the element only if
				// the attribute is FormalName
	Decl decl =
	    getDefaultDecl((DOMBaseNode)getParent(),
			   (attName.equals("FormalName") ? element : null),
			   attribute,
			   useExternal);
	if (decl != null) {
	    if (decl.resource.getUrlCount() > 0)
		return decl.resource.getUrl(0).toString();
	    else
		return decl.resource.getUrn().toString();
	} else {
	    return null;
	}
    }


    /**
     * @see FormalName#getScheme
     */
    public Text getScheme ()
    {
	if (attName.equals("FormalName"))
	    return getAttribute("Scheme");
	else
	    return null;
    }


    /**
     * @see FormalName#getScheme
     */
    public String getDefaultScheme (boolean useExternal)
	throws IOException
    {
	Node element = getDOMNode();
	Node attribute = element.getAttributes().getNamedItem(attName);
	Text scheme = getScheme();
	if (scheme != null)
	    return scheme.toString();
				// Allow matches on the element only if
				// the attribute is FormalName
	Decl decl = 
	    getDefaultDecl((DOMBaseNode)getParent(),
			   (attName.equals("FormalName") ? element : null),
			   attribute,
			   useExternal);
	if (decl == null || decl.defaultVocabularyFor.getScheme() == null)
	    return null;
	else
	    return decl.defaultVocabularyFor.getScheme().toString();
    }


    /**
     * @see FormalName#getTopicSet
     */
    public TopicSet getTopicSet (boolean useExternal)
	throws IOException
    {
	String vocab = getDefaultVocabulary(useExternal);
	if (vocab == null)
	    return null;

	BaseNode node = getSession().findReference(vocab, useExternal);
	if (node == null || !("TopicSet".equals(node.getXMLName()))) {
	    return null;
	} else {
	    return (TopicSet)node;
	}
    }


    /**
     * @see FormalNameNode#findTopic
     */
    public Topic findTopic (boolean useExternal)
	throws IOException
    {
	TopicSet topicset = getTopicSet(useExternal);
	if (topicset != null) {
	    String scheme = getDefaultScheme(useExternal);
	    return topicset.findTopic(getName().toString(),
				      scheme, useExternal);
	} else {
	    return null;
	}
    }


    /**
     * @see FormalName#toString()
     */
    public String toString ()
    {
	Text v = getVocabulary();
	Text s = getScheme();
	Text n = getName();
	return makeString((v == null ? null : v.toString()),
			  (s == null ? null : s.toString()),
			  (n == null ? null : n.toString()));
    }


    /**
     * @see FormalName#toString(boolean)
     */
    public String toString (boolean useExternal)
	throws IOException
    {
	Text n = getName();
	return makeString(getDefaultVocabulary(useExternal),
			  getDefaultScheme(useExternal),
			  (n == null ? null : n.toString()));
    }


    /**
     * @see OfInterestTo#getRelevance()
     */
    public AssignedFormalName getRelevance ()
    {
	return (AssignedFormalName)getChild("Relevance", 0);
    }


    /**
     * @see Property#getValue()
     */
    public Text getValue ()
    {
	return getAttribute("Value");
    }


    /**
     * @see Property#getValueRef()
     */
    public Text getValueRef ()
    {
	return getAttribute("ValueRef");
    }


    /**
     * @see Property#getAllowedValue()
     */
    public Text getAllowedValues ()
    {
	return getAttribute("AllowedValues");
    }

    private String attName;


    /**
     * Construct a string version of the name parts.
     */
    private String makeString (String vocab, String scheme, String name)
    {
	StringBuffer buf = new StringBuffer();
	buf.append('{');
	buf.append(vocab == null ? "" : vocab);
	buf.append("}{");
	buf.append(scheme == null ? "" : scheme);
	buf.append('}');
	buf.append(name);
	return buf.toString();
    }



    ////////////////////////////////////////////////////////////////////
    // Internal methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Look up a default vocabulary for an element.
     *
     * This method currently supports only element names, not full
     * XPath expressions.  This method climbs recursively to the
     * root element of the document, check each element to see if it
     * has a Catalog child containing an appropriate
     * DefaultVocabularyFor declaration.
     *
     * @param context The first node to search (usually starting with
     * the parent element of the element containing the formal name.
     * @param target1 The first target (usually an element, or null).
     * @param target2 The second target (usually an attribute, or
     * null).
     * @param useExternal true if the library should try looking
     * up the default vocabulary in external catalogs; false to
     * stay in the local document.
     * @return The Resource element that provides the default
     * vocabulary, or null if none was specified.
     */
    private Decl getDefaultDecl (DOMBaseNode context,
				 Node target1,
				 Node target2,
				 boolean useExternal)
	throws IOException
    {
				// A context node must have a catalog...
	Node contextNode = context.getDOMNode();
	Catalog catalog = (Catalog)context.getChild("Catalog", 0);
	if (catalog != null) {
	    catalog = catalog.getReferencedCatalog(useExternal);
	    if (catalog != null) {

				// Look through every default vocab
				// declaration in every resource
		int nResources = catalog.getResourceCount();
		for (int i = 0; i < nResources; i++) {
		    Resource resource = catalog.getResource(i);
		    int nDecls = resource.getDefaultVocabularyForCount();
		    for (int j = 0; j < nDecls; j++) {
			DefaultVocabularyFor dv =
			    resource.getDefaultVocabularyFor(j);
			if (dv != null) {
			    Text ctxt = dv.getContext();
			    if (ctxt != null) {
				DOMXPath path =
				    new DOMXPath(ctxt.toString());
				if ((target1 != null &&
				     path.matches(contextNode, target1)) ||
				    (target2 != null &&
				     path.matches(contextNode, target2)))
				    return new Decl(resource, dv);
			    }
			}
		    }
		}
	    }
	}
				// If we arrive here, we failed, so
				// try the parent.
	BaseNode parent = context.getParent();
	return (parent == null ? null :
		getDefaultDecl((DOMBaseNode)parent, target1,
			       target2, useExternal));
    }


    /**
     * Inner class for a default declaration.
     */
    class Decl
    {
	Decl (Resource resource, DefaultVocabularyFor defaultVocabularyFor)
	{
	    this.resource = resource;
	    this.defaultVocabularyFor = defaultVocabularyFor;
	}

	Resource resource;
	DefaultVocabularyFor defaultVocabularyFor;
    }

}

// end of DOMFormalName.java
