// DOMNewsComponent.java - DOM implementation of NewsComponent.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMNewsComponent.java,v 1.9 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.util.Arrays;
import java.util.Comparator;

import org.w3c.dom.Node;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.newsml.toolkit.AdministrativeMetadata;
import org.newsml.toolkit.BasisForChoice;
import org.newsml.toolkit.DescriptiveMetadata;
import org.newsml.toolkit.EquivalentNode;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.Metadata;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsLines;
import org.newsml.toolkit.RightsMetadata;
import org.newsml.toolkit.Text;
import org.newsml.toolkit.TopicSet;


/**
 * DOM implementation of NewsComponent.
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
class DOMNewsComponent
    extends DOMBaseNode
    implements NewsComponent
{

    /**
     * Constructor.
     *
     * @param element The DOM node representing the news component.
     */
    protected DOMNewsComponent (Node element, DOMNewsMLSession session)
    {
	super(element, session);
    }


    /**
     * @see NewsComponent#getEssential()
     */
    public Text getEssential ()
    {
	return getAttribute("Essential");
    }


    /**
     * @see NewsComponent#getEquivalentsList()
     */
    public Text getEquivalentsList ()
    {
	return getAttribute("EquivalentsList");
    }


    /**
     * @see NewsComponent#getTopicSetCount()
     */
    public int getTopicSetCount ()
    {
	return getChildCount("TopicSet");
    }

    
    /**
     * @see NewsComponent#getTopicSet(int)
     */
    public TopicSet getTopicSet (int index)
    {
	return (TopicSet)getChild("TopicSet", index);
    }


    /**
     * @see NewsComponent#getTopicSet()
     */
    public TopicSet [] getTopicSet ()
    {
	int len = getTopicSetCount();
	TopicSet ret[] = new TopicSet[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getTopicSet(i);
	return ret;
    }


    /**
     * @see NewsComponent#getRole()
     */
    public FormalName getRole ()
    {
	return (FormalName)getChild("Role", 0);
    }


    /**
     * @see NewsComponent#getBasisForChoiceCount()
     */
    public int getBasisForChoiceCount ()
    {
	return getChildCount("BasisForChoice");
    }


    /**
     * @see NewsComponent#getBasisForChoice(int)
     */
    public BasisForChoice getBasisForChoice (int index)
    {
	return (BasisForChoice)getChild("BasisForChoice", index);
    }


    /**
     * @see NewsComponent#getBasisForChoice()
     */
    public BasisForChoice [] getBasisForChoice ()
    {
	int len = getBasisForChoiceCount();
	BasisForChoice ret[] = new BasisForChoice[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getBasisForChoice(i);
	return ret;
    }


    /**
     * @see NewsComponent#getRankedBasisForChoice()
     */
    public BasisForChoice [] getRankedBasisForChoice ()
    {
	BasisForChoice bases[] = getBasisForChoice();
	Arrays.sort(bases, 0, bases.length, new Comparator () {
		public int compare (Object o1, Object o2) {
		    Text rank1 = ((BasisForChoice)o1).getRank();
		    Text rank2 = ((BasisForChoice)o2).getRank();
		    if ((rank1 == null) && (rank2 == null)) {
			return 0;
		    } else if (rank1 == null) {
			return 1;
		    } else if (rank2 == null) {
			return -1;
		    } else {
			int r1 = rank1.toInt();
			int r2 = rank2.toInt();
			if (r1 < r2)
			    return -1;
			else if (r1 > r2)
			    return 1;
			else
			    return 0;
		    }
		}
	    });
	return bases;
    }


    /**
     * @see NewsComponent#getNewsLines()
     */
    public NewsLines getNewsLines ()
    {
	return (NewsLines)getChild("NewsLines", 0);
    }


    /**
     * @see NewsComponent#getAdministrativeMetadata()
     */
    public AdministrativeMetadata getAdministrativeMetadata ()
    {
	return (AdministrativeMetadata)getChild("AdministrativeMetadata", 0);
    }


    /**
     * @see NewsComponent#getRightsMetadata()
     */
    public RightsMetadata getRightsMetadata ()
    {
	return (RightsMetadata)getChild("RightsMetadata", 0);
    }


    /**
     * @see NewsComponent#getDescriptiveMetadata
     */
    public DescriptiveMetadata getDescriptiveMetadata ()
    {
	return (DescriptiveMetadata)getChild("DescriptiveMetadata", 0);
    }


    /**
     * @see NewsComponent#getMetadataCount()
     */
    public int getMetadataCount ()
    {
	return getChildCount("Metadata");
    }


    /**
     * @see NewsComponent#getMetadata(int)
     */
    public Metadata getMetadata (int index)
    {
	return (Metadata)getChild("Metadata", index);
    }


    /**
     * @see NewsComponent#getMetadata()
     */
    public Metadata [] getMetadata ()
    {
	int len = getMetadataCount();
	Metadata ret[] = new Metadata[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getMetadata(i);
	return ret;
    }


    /**
     * @see NewsComponent#getMemberCount()
     */
    public int getMemberCount ()
    {
	int n = 0;
	NodeList children = getDOMNode().getChildNodes();
	int nChildren = children.getLength();
	for (int i = 0; i < nChildren; i++) {
	    Node item = children.item(i);
	    if (item.getNodeType() != Node.ELEMENT_NODE) {
		continue;
	    } else if ("NewsItem".equals(item.getNodeName())) {
		n++;
	    } else if ("NewsComponent".equals(item.getNodeName())) {
		n++;
	    } else if ("NewsItemRef".equals(item.getNodeName())) {
		n++;
	    } else if ("ContentItem".equals(item.getNodeName())) {
		n++;
	    }
	}
	return n;
    }


    /**
     * @see NewsComponent#getMember(int)
     */
    public EquivalentNode getMember (int index)
    {
	NodeList children = getDOMNode().getChildNodes();
	int nChildren = children.getLength();
	for (int i = 0; i < nChildren; i++) {
	    Node item = children.item(i);
	    if (item.getNodeType() != Node.ELEMENT_NODE) {
		continue;
	    } else if ("NewsItem".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsItem(item, session);
		else
		    index--;
	    } else if ("NewsComponent".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsComponent(item, session);
		else
		    index--;
	    } else if ("NewsItemRef".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsItemRef(item, session);
		else
		    index--;
	    } else if ("ContentItem".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMContentItem(item, session);
		else
		    index--;
	    }
	}
	return null;
    }


    /**
     * @see NewsComponent#getMember()
     */
    public EquivalentNode [] getMember ()
    {
	int len = getMemberCount();
	EquivalentNode ret[] = new EquivalentNode[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getMember(i);
	return ret;
    }

}

// end of DOMNewsComponent.java
