// DOMNewsMLFactory.java - DOM implementation of NewsMLFactory.
// Copyright (c) 2001 by Reuters PLC.

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.File;
import java.io.IOException;
import java.io.Reader;

import java.net.MalformedURLException;
import java.net.URL;

import org.w3c.dom.Node;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.Catalog;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLFactory;
import org.newsml.toolkit.TopicSet;


/**
 * DOM implementation of NewsMLFactory.
 *
 * <p>This implementation uses yet another factory class, {@link
 * DOMFactory}, to allow the user to plug in different DOM
 * implementations.  A default {@link XercesDOMFactory} is included
 * with this distribution.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public class DOMNewsMLFactory
    implements NewsMLFactory
{

    /**
     * Default constructor (uses Xerces).
     */
    public DOMNewsMLFactory ()
    {
	this.factory = new XercesDOMFactory();
    }

    /**
     * Constructor.
     *
     * @param factory The factory object for creating new DOM documents.
     */
    public DOMNewsMLFactory (DOMFactory factory)
    {
	this.factory = factory;
    }


    /**
     * Constructor.
     *
     * @param factory The factory object for creating new DOM documents.
     * @param validation true if DTD validation is required, false if
     * it is forbidden.
     * @exception IOException if the implementation does not support
     * the requested validation state.
     */
    public DOMNewsMLFactory (DOMFactory factory, boolean validation)
	throws IOException
    {
	this.factory = factory;
	factory.setValidation(validation);
    }


    /**
     * @see NewsMLFactory#setValidation(boolean)
     */
    public void setValidation (boolean validation)
	throws IOException
    {
	factory.setValidation(validation);
    }


    /**
     * @see NewsMLFactory#getValidation()
     */
    public boolean getValidation ()
    {
	return factory.getValidation();
    }


    /**
     * @see NewsMLFactory#createNewsML(String)
     */
    public NewsML createNewsML (String url)
	throws IOException
    {
	try {
	    NewsML newsml = (NewsML)createNode(url);
	    if (newsml != null)
		return newsml;
	} catch (ClassCastException e) {
	    throw new NewsMLException("Root element of " +
				      url +
				      " is not NewsML");
	}
	throw new NewsMLException("Failed to create NewsML tree from " + url);
    }


    /**
     * @see NewsMLFactory#createNewsML(Reader,String)
     */
    public NewsML createNewsML (Reader input, String baseURL)
	throws IOException
    {
	try {
	    NewsML newsml = (NewsML)createNode(input, baseURL);
	    if (newsml != null)
		return newsml;
	} catch (ClassCastException e) {
	    throw new NewsMLException("Root element is not NewsML");
	}
	throw new NewsMLException("Failed to create NewsML tree");
    }


    /**
     * @see NewsMLFactory#createNode(String)
     */
    public BaseNode createNode (String url)
	throws IOException
    {
	url = makeAbsoluteURL(url);
	DOMNewsMLSession session = new DOMNewsMLSession(this);
	Node element = factory.createDOM(url).getDocumentElement();
	DOMBaseNode node = session.makeNewsMLNode(element);
	if (node == null)
	    return null;
	session.setRootNode(node);
	session.setBaseURL(url);
	return node;
    }


    /**
     * @see NewsMLFactory#createNode(Reader,String)
     */
    public BaseNode createNode (Reader input, String baseURL)
	throws IOException
    {
	baseURL = makeAbsoluteURL(baseURL);
	DOMNewsMLSession session = new DOMNewsMLSession(this);
	Node element = factory.createDOM(input, baseURL).getDocumentElement();
	DOMBaseNode node = session.makeNewsMLNode(element);
	session.setRootNode(node);
	session.setBaseURL(baseURL);
	return node;
    }



    ////////////////////////////////////////////////////////////////////
    // Internal methods.
    ////////////////////////////////////////////////////////////////////

    private String makeAbsoluteURL (String path)
	throws MalformedURLException
    {
	try {
	    return new URL(path).toString();
	} catch (MalformedURLException e) {
	    return new File(path).toURL().toString();
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private DOMFactory factory;

}

// end of DOMNewsMLFactory.java
