// DOMNewsMLSession.java - DOM implementation of NewsMLSession.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMNewsMLSession.java,v 1.14 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;

import org.w3c.dom.Attr;
import org.w3c.dom.Node;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLFactory;
import org.newsml.toolkit.NewsMLSession;


/**
 * DOM implementation of NewsMLSession.
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see NewsMLSession
 */
class DOMNewsMLSession
    implements NewsMLSession
{


    ////////////////////////////////////////////////////////////////////
    // Constructor.
    ////////////////////////////////////////////////////////////////////

    /**
     * Constructor.
     *
     * @param root The root node of the NewsML package.
     * @param factory The factory for creating new NewsML packages.
     */
    protected DOMNewsMLSession (DOMNewsMLFactory factory)
    {
	this.factory = factory;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of NewsMLSession.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see NewsMLSession#getFactory()
     */
    public NewsMLFactory getFactory ()
    {
	return factory;
    }


    /**
     * @see NewsMLSession#getRootNode()
     */
    public BaseNode getRootNode ()
    {
	return root;
    }


    /**
     * @see NewsMLSession#findReference(String,boolean)
     */
    public BaseNode findReference (String reference, boolean useExternal)
	throws IOException
    {
				// Internal reference
	if (reference.startsWith("#")) {
	    String duid = reference.substring(1);
	    Node rootNode = root.getDOMNode();
	    Node targetNode =
		DOMUtils.findElementByAttribute(rootNode, "Duid", duid);
	    return makeNewsMLNode(targetNode);
	}

				// External reference (allowed)
	else if (useExternal) {
	    String url;
	    String duid;
	    int hash = reference.indexOf('#');
	    if (hash == -1) {
		url = reference;
		duid = null;
	    } else {
		url = reference.substring(0, hash);
		duid = reference.substring(hash);
	    }
	    BaseNode node = getFactory().createNode(resolveURL(url));
	    if (duid == null)
		return node;
	    else
		return node.getSession().findReference(duid, false);
	}

				// External (not allowed)
	else {
				// FIXME: should we throw an exception
				// here?
	    return null;
	}
    }


    /**
     * @see NewsMLSession#getNodeByDuid(String)
     */
    public BaseNode getNodeByDuid (String duid)
    {
				// strip off any leading '#'
	if (duid.startsWith("#"))
	    duid = duid.substring(1);
	Node rootNode = root.getDOMNode();
	Node targetNode =
	    DOMUtils.findElementByAttribute(rootNode, "Duid", duid);
	return makeNewsMLNode(targetNode);
    }


    /**
     * @see NewsMLSession#getNodesByXPath
     */
    public BaseNode [] getNodesByXPath (BaseNode contextNode, String xpath)
	throws NewsMLException
    {
	DOMXPath path = new DOMXPath(xpath);
	Node result[] = path.select(((DOMBaseNode)contextNode).getDOMNode());
	BaseNode nodes[] = new BaseNode[result.length];
	int pos = 0;
	for (int i = 0; i < result.length; i++) {
	    BaseNode node = makeNewsMLNode(result[i]);
	    if (node != null)
		nodes[pos++] = node;
	}
				// Reallocate if some results weren't
				// NewsML nodes.
	if (pos != nodes.length) {
	    BaseNode tmp[] = new BaseNode[pos];
	    System.arraycopy(nodes, 0, tmp, 0, pos);
	    nodes = tmp;
	}

	return nodes;
    }


    /**
     * @see NewsMLSession#getNodeByXPath
     */
    public BaseNode getNodeByXPath (BaseNode contextNode, String xpath)
	throws NewsMLException
    {
	BaseNode results[] = getNodesByXPath(contextNode, xpath);
	if (results.length >= 1)
	    return results[0];
	else
	    return null;
    }


    /**
     * @see NewsMLSession#getBaseURL()
     */
    public String getBaseURL ()
    {
	return baseURL;
    }


    /**
     * @see NewsMLSession#resolveHref(String)
     */
    public URL resolveHref (String href)
	throws MalformedURLException
    {
	return new URL(resolveURL(href));
    }



    ////////////////////////////////////////////////////////////////////
    // Protected methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Set the root node.
     *
     * @param root The new root node.
     * @see #getRootNode
     */
    protected void setRootNode (DOMBaseNode root)
    {
	this.root = root;
    }


    /**
     * Set the base URL.
     *
     * @param baseURL The base URL for the session.
     * @see #getBaseURL
     */
    protected void setBaseURL (String baseURL)
    {
	this.baseURL = baseURL;
    }


    /**
     * Construct a NewsML node from a DOM node.
     *
     * @param element The DOM element (or null).
     * @return A NewsML node of some kind, or null if the DOM node
     * was null.
     */
    protected DOMBaseNode makeNewsMLNode (Node node)
    {
	if (node == null)
	    return null;

				// Since we intern the name, we can
				// use == as a test, which is much
				// faster.
	String name = node.getNodeName().intern();

				// This is a big, giant, ugly table
				// mapping every element in a NewsML
				// document to its corresponding
				// class.  Eventually, all new object
				// creation should probably go through
				// here.

				// It probably wouldn't be any faster
				// to use a HashMap; the == test is
				// VERY fast, since it doesn't require
				// any string operations.
	if (node.getNodeType() == Node.ATTRIBUTE_NODE) {


				// All attributes used in NewsML 1.0
	    if (name == "AllowedValues" ||
		name == "AssignedBy" ||
		name == "Context" ||
		name == "DateAndTime" ||
		name == "Details" ||
		name == "Duid" ||
		name == "DuidRef" ||
		name == "EquivalentsList" ||
		name == "Essential" ||
		name == "Euid" ||
		name == "Href" ||
		name == "NewsItem" ||
		name == "Notation" ||
		name == "PreviousRevision" ||
		name == "Rank" ||
		name == "Repeat" ||
		name == "Revision" ||
		name == "Scheme" ||
		name == "Topic" ||
		name == "TopicSet" ||
		name == "TranslationOf" ||
		name == "Update" ||
		name == "Value" ||
		name == "ValueRef" ||
		name == "Variant" ||
		name == "Vocabulary" ||
		name == "xml:lang") {
		return new DOMText(node, this);
	    } else if (name == "FormalName") {
		return new DOMFormalName(((Attr)node).getOwnerElement(),
					 this);
	    } else if (name == "Importance" ||
		       name == "Confidence" ||
		       name == "HowPresent") {
		return new DOMFormalName(((Attr)node).getOwnerElement(),
					 this, name);
	    } else {
		return null;
	    }

	} else if (node.getNodeType() == Node.ELEMENT_NODE) {

	    if (name == "AdministrativeMetadata")
		return new DOMAdministrativeMetadata(node, this);
	    else if (name == "AssociatedWith")
		return new DOMNewsItemRef(node, this);
	    else if (name == "BasisForChoice")
		return new DOMText(node, this);
	    else if (name == "ByLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Catalog")
		return new DOMCatalog(node, this);
	    else if (name == "Characteristics")
		return new DOMCharacteristics(node, this);
	    else if (name == "Comment")
		return new DOMText(node, this);
	    else if (name == "ContentItem")
		return new DOMContentItem(node, this);
	    else if (name == "Contributor")
		return new DOMPartyList(node, this);
	    else if (name == "Copyright")
		return new DOMCopyright(node, this);
	    else if (name == "CopyrightDate")
		return new DOMOriginBase(node, this);
	    else if (name == "CopyrightHolder")
		return new DOMOriginBase(node, this);
	    else if (name == "CopyrightLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Creator")
		return new DOMPartyList(node, this);
	    else if (name == "CreditLine")
		return new DOMOriginBase(node, this);
	    else if (name == "DataContent")
		return new DOMDataContent(node, this);
	    else if (name == "DateAndTime")
		return new DOMText(node, this);
	    else if (name == "DateId")
		return new DOMText(node, this);
	    else if (name == "DateLabel")
		return new DOMText(node, this);
	    else if (name == "DateLine")
		return new DOMOriginBase(node, this);
	    else if (name == "DefaultVocabularyFor")
		return new DOMDefaultVocabularyFor(node, this);
	    else if (name == "Delete")
		return new DOMDelta(node, this);
	    else if (name == "DerivedFrom")
		return new DOMNewsItemRef(node, this);
	    else if (name == "Description")
		return new DOMText(node, this);
	    else if (name == "DescriptiveMetadata")
		return new DOMDescriptiveMetadata(node, this);
	    else if (name == "Encoding")
		return new DOMEncoding(node, this);
	    else if (name == "EndDate")
		return new DOMOriginBase(node, this);
	    else if (name == "FileName")
		return new DOMText(node, this);
	    else if (name == "FirstCreated")
		return new DOMText(node, this);
	    else if (name == "FormalName")
		return new DOMFormalNameDecl(node, this);
	    else if (name == "Format")
		return new DOMFormalName(node, this);
	    else if (name == "FutureStatus")
		return new DOMFormalName(node, this);
	    else if (name == "Genre")
		return new DOMFormalName(node, this);
	    else if (name == "Geography")
		return new DOMOriginBase(node, this);
	    else if (name == "HeadLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Identification")
		return new DOMIdentification(node, this);
	    else if (name == "InsertAfter")
		return new DOMDelta(node, this);
	    else if (name == "InsertBefore")
		return new DOMDelta(node, this);
	    else if (name == "Instruction")
		return new DOMInstruction(node, this);
	    else if (name == "KeywordLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Label")
		return new DOMLabel(node, this);
	    else if (name == "LabelText")
		return new DOMText(node, this);
	    else if (name == "LabelType")
		return new DOMFormalName(node, this);
	    else if (name == "Language")
		return new DOMFormalName(node, this);
	    else if (name == "Limitations")
		return new DOMOriginBase(node, this);
	    else if (name == "MediaType")
		return new DOMFormalName(node, this);
	    else if (name == "Metadata")
		return new DOMMetadata(node, this);
	    else if (name == "MetadataType")
		return new DOMFormalName(node, this);
	    else if (name == "MimeType")
		return new DOMFormalName(node, this);
	    else if (name == "NameLabel")
		return new DOMText(node, this);
	    else if (name == "NewsComponent")
		return new DOMNewsComponent(node, this);
	    else if (name == "NewsEnvelope")
		return new DOMNewsEnvelope(node, this);
	    else if (name == "NewsIdentifier")
		return new DOMNewsIdentifier(node, this);
	    else if (name == "NewsItem")
		return new DOMNewsItem(node, this);
	    else if (name == "NewsItemId")
		return new DOMNewsIdentifierFormalName(node, this);
	    else if (name == "NewsItemRef")
		return new DOMNewsItemRef(node, this);
	    else if (name == "NewsItemType")
		return new DOMFormalName(node, this);
	    else if (name == "NewsLine")
		return new DOMNewsLine(node, this);
	    else if (name == "NewsLineText")
		return new DOMOriginBase(node, this);
	    else if (name == "NewsLineType")
		return new DOMFormalName(node, this);
	    else if (name == "NewsLines")
		return new DOMNewsLines(node, this);
	    else if (name == "NewsML")
		return new DOMNewsML(node, this);
	    else if (name == "NewsManagement")
		return new DOMNewsManagement(node, this);
	    else if (name == "NewsProduct")
		return new DOMFormalName(node, this);
	    else if (name == "NewsService")
		return new DOMFormalName(node, this);
	    else if (name == "Notation")
		return new DOMFormalName(node, this);
	    else if (name == "OfInterestTo")
		return new DOMFormalName(node, this);
	    else if (name == "Origin")
		return new DOMOriginBase(node, this);
	    else if (name == "Party")
		return new DOMFormalName(node, this);
	    else if (name == "Priority")
		return new DOMFormalName(node, this);
	    else if (name == "Property")
		return new DOMFormalName(node, this);
	    else if (name == "Provider")
		return new DOMPartyList(node, this);
	    else if (name == "ProviderId")
		return new DOMNewsIdentifierFormalName(node, this);
	    else if (name == "PublicIdentifier")
		return new DOMText(node, this);
	    else if (name == "Relevance")
		return new DOMFormalName(node, this);
	    else if (name == "Replace")
		return new DOMDelta(node, this);
	    else if (name == "Resource")
		return new DOMResource(node, this);
	    else if (name == "RevisionHistory")
		return new DOMBaseNode(node, this);
	    else if (name == "RevisionId")
		return new DOMText(node, this);
	    else if (name == "RevisionStatus")
		return new DOMRevisionStatus(node, this);
	    else if (name == "RightsHolder")
		return new DOMOriginBase(node, this);
	    else if (name == "RightsLine")
		return new DOMOriginBase(node, this);
	    else if (name == "RightsMetadata")
		return new DOMRightsMetadata(node, this);
	    else if (name == "Role")
		return new DOMFormalName(node, this);
	    else if (name == "SentFrom")
		return new DOMPartyList(node, this);
	    else if (name == "SentTo")
		return new DOMPartyList(node, this);
	    else if (name == "SeriesLine")
		return new DOMOriginBase(node, this);
	    else if (name == "SizeInBytes")
		return new DOMText(node, this);
	    else if (name == "SlugLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Source")
		return new DOMPartyList(node, this);
	    else if (name == "StartDate")
		return new DOMOriginBase(node, this);
	    else if (name == "Status")
		return new DOMFormalName(node, this);
	    else if (name == "StatusWillChange")
		return new DOMStatusWillChange(node, this);
	    else if (name == "SubHeadLine")
		return new DOMOriginBase(node, this);
	    else if (name == "Subject")
		return new DOMFormalName(node, this);
	    else if (name == "SubjectCode")
		return new DOMSubjectCode(node, this);
	    else if (name == "SubjectDetail")
		return new DOMFormalName(node, this);
	    else if (name == "SubjectMatter")
		return new DOMFormalName(node, this);
	    else if (name == "SubjectQualifier")
		return new DOMFormalName(node, this);
	    else if (name == "SystemIdentifier")
		return new DOMText(node, this);
	    else if (name == "ThisRevisionCreated")
		return new DOMText(node, this);
	    else if (name == "Topic")
		return new DOMTopic(node, this);
	    else if (name == "TopicOccurrence")
		return new DOMBaseNode(node, this);
	    else if (name == "TopicSet")
		return new DOMTopicSet(node, this);
	    else if (name == "TopicSetRef")
		return new DOMTopicSetRef(node, this);
	    else if (name == "TopicType")
		return new DOMFormalName(node, this);
	    else if (name == "TopicUse")
		return new DOMTopicUse(node, this);
	    else if (name == "TransmissionId")
		return new DOMText(node, this);
	    else if (name == "Update")
		return new DOMUpdate(node, this);
	    else if (name == "Urgency")
		return new DOMFormalName(node, this);
	    else if (name == "Url")
		return new DOMText(node, this);
	    else if (name == "Urn")
		return new DOMText(node, this);
	    else if (name == "UsageRights")
		return new DOMUsageRights(node, this);
	    else if (name == "UsageType")
		return new DOMOriginBase(node, this);
	    else
		return null;
	} else {
	    return null;
	}
    }

    /**
     * Resolve a URL against the current relative URL.
     */
    protected String resolveURL (String url)
    {
	if (url.startsWith("urn:"))
	    return url;

	try {
	    return new URL(new URL(baseURL), url).toString();
	} catch (MalformedURLException ex) {
	    return url;
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private DOMBaseNode root;
    private DOMNewsMLFactory factory;
    private String baseURL;

}

// end of DOMNewsMLSession.java
