// DOMTopicSet.java - DOM implementation of TopicSet.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMTopicSet.java,v 1.8 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;

import org.w3c.dom.Node;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.FormalNameDecl;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.Topic;
import org.newsml.toolkit.TopicSet;
import org.newsml.toolkit.TopicSetRef;


/**
 * DOM implementation of TopicSet.
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
class DOMTopicSet
    extends DOMFormalName
    implements TopicSet
{

    /**
     * Constructor.
     */
    protected DOMTopicSet (Node element, DOMNewsMLSession session)
    {
	super(element, session);
    }


    /**
     * @see TopicSet#getTopicSetRefCount()
     */
    public int getTopicSetRefCount ()
    {
	return getChildCount("TopicSetRef");
    }


    /**
     * @see TopicSet#getTopicSetRef(int)
     */
    public TopicSetRef getTopicSetRef (int index)
    {
	return (TopicSetRef)getChild("TopicSetRef", index);
    }


    /**
     * @see TopicSet#getTopicSetRef()
     */
    public TopicSetRef [] getTopicSetRef ()
    {
	int len = getTopicSetRefCount();
	TopicSetRef ret[] = new TopicSetRef[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getTopicSetRef(i);
	return ret;
    }


    /**
     * @see TopicSet#getTopicCount()
     */
    public int getTopicCount ()
    {
	return getChildCount("Topic");
    }


    /**
     * @see TopicSet#getTopic(int)
     */
    public Topic getTopic (int index)
    {
	return (Topic)getChild("Topic", index);
    }


    /**
     * @see TopicSet#getTopic()
     */
    public Topic [] getTopic ()
    {
	int len = getTopicCount();
	Topic ret[] = new Topic[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getTopic(i);
	return ret;
    }


    /**
     * @see TopicSet#findTopic
     */
    public Topic findTopic (String name, String scheme, boolean useExternal)
	throws IOException
    {
				// Start by recursing into any referenced
				// topic sets
	int nTopicSetRefs = getTopicSetRefCount();
	for (int i = 0; i < nTopicSetRefs; i++) {
	    String ref = getTopicSetRef(i).getRef().toString();
	    BaseNode target = null;
	    if (ref.startsWith("#")) {
		target = getSession().getNodeByDuid(ref);
	    } else if (ref.startsWith("http://")) {
		target = getSession().findReference(ref, useExternal);
	    } else if (ref.startsWith("urn:newsml:")) {
		continue;
	    } else {
		throw new NewsMLException("Illegal topic set reference: " +
					  ref);
	    }
	    if (target == null)
				// FIXME: provide a warning
		continue;
	    if (!"TopicSet".equals(target.getXMLName()))
		throw new NewsMLException("Topic set reference " + ref +
					  " points to " + target.getXMLName());
	    Topic topic =
		((TopicSet)target).findTopic(name, scheme, useExternal);
	    if (topic != null)
		return topic;
	}

				// Now, search this one.
	int nTopics = getTopicCount();
	for (int i = 0; i < nTopics; i++) {
	    Topic topic = getTopic(i);
	    int nDecls = topic.getFormalNameDeclCount();
	    for (int j = 0; j < nDecls; j++) {
		FormalNameDecl decl = topic.getFormalNameDecl(j);
		String name2 = decl.toString();
		String scheme2 = null;
		if (decl.getScheme() != null)
		    scheme2 = decl.getScheme().toString();
		if (name.equals(name2) &&
		    ((scheme == null && scheme2 == null) ||
		     (scheme != null && scheme.equals(scheme2))))
		    return topic;
	    }
	}

				// Give up.
	return null;
    }

}

// end of DOMTopicSet.java
