// DOMUtils.java - static utility methods for querying a DOM.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMUtils.java,v 1.12 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;
import java.io.Writer;

import org.w3c.dom.DOMException;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;


/**
 * Static utility methods for querying a DOM tree.
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
final class DOMUtils
{

    /**
     * Private constructor.
     */
    private DOMUtils ()
    {
	// This is never invoked.
    }


    /**
     * Count the child elements with the specified name.
     */
    static int count (Node node, String name)
    {
	return count(node.getChildNodes(), name);
    }


    /**
     * Count the members of a node list with the specified name.
     */
    static int count (NodeList nodes, String name)
    {
	int count = 0;
	int len = nodes.getLength();
	for (int i = 0; i < len; i++) {
	    if (name.equals(nodes.item(i).getNodeName()))
		count++;
	}
	return count;
    }


    /**
     * Find the first child element with the specified name.
     *
     * @param e The parent element, or null (always fails).
     * @param name The name of the child.
     * @return The DOM node for the child element, or null if
     * none was found or a null element was provided.
     * @see #find(org.w3c.dom.Node,java.lang.String name,int index)
     */
    static Node find (Node e, String name)
    {
	if (e == null)
	    return null;
	else
	    return find(e.getChildNodes(), name, 0);
    }


    /**
     * Find the nth child element with the specified name.
     *
     * @param e The parent element, or null (always fails).
     * @param name The name of the child.
     * @param index The index of the child.
     * @return The DOM node for the child element, or null if
     * none was found or a null element was provided.
     * @see #find(org.w3c.dom.Node,java.lang.String name)
     */
    static Node find (Node e, String name, int index)
    {
	if (e == null)
	    return null;
	else
	    return find(e.getChildNodes(), name, index);
    }


    /**
     * Find an element in a DOM node list.
     *
     * @param nodes The node list to search, or null (always fails).
     * @param name The target element name.
     * @param index The target index.
     * @return The DOM node for the matching member of the node list,
     * or null if none was found or a null nodelist was provided.
     */
    static Node find (NodeList nodes, String name, int index)
    {
	int len = nodes.getLength();

	for (int i = 0; i < len; i++) {
	    Node node = nodes.item(i);
	    if (node.getNodeName().equals(name)) {
		index--;
		if (index < 0) {
		    return node;
		}
	    }
	}
	return null;
    }



    /**
     * Find the position of a named, indexed member of a node list.
     *
     * @param nodes The node list to search.
     * @param name The node name.
     * @param index The node index (among nodes with the same name).
     * @return The member's position, or -1 if none was found with
     * the specified name and index.
     */
    static int findPos (NodeList nodes, String name, int index)
    {
	int len = nodes.getLength();
	for (int i = 0; i < len; i++) {
	    Node node = nodes.item(i);
	    if (name.equals(node.getNodeName())) {
		index--;
		if (index < 0)
		    return i;
	    }
	}
	return -1;
    }


    /**
     * Get an attribute value.
     *
     * @param e The element node, or null (always fails).
     * @param name The attribute name.
     * @return The text of the attribute value, or null if the
     * element was null or the attribute was not present.
     */
    static String getAttVal (Node e, String name)
    {
	if (e == null)
	    return null;
	NamedNodeMap atts = e.getAttributes();
	Node att = atts.getNamedItem(name);
	if (att != null)
	    return att.getNodeValue();
	else
	    return null;
    }


    /**
     * Get the text of a node's descendants.
     *
     * <p>Note: this method does not return the text of the node
     * itself.</p>
     *
     * @param node A DOM node, or null (always fails).
     * @return A string containing the text of the node's
       descendants, or null if the node is null.
     */
    static String getText (Node node)
    {
	if (node == null)
	    return null;
	else
	    return getText(node.getChildNodes());
    }


    /**
     * Get the text of a node list.
     *
     * @param nodes The node list.
     * @return All of the text descendants of the node list in a
     * single string.
     */
    static String getText (NodeList nodes)
    {
	if (nodes == null)
	    return null;
	int len = nodes.getLength();
	StringBuffer text = new StringBuffer();
	for (int i = 0; i < len; i++) {
	    Node child = nodes.item(i);
	    if (child.getNodeType() == Node.TEXT_NODE ||
		child.getNodeType() == Node.CDATA_SECTION_NODE)
		text.append(child.getNodeValue());
	    else
		text.append(getText(child));
	}
	return text.toString();
    }


    /**
     * Find a DOM node with the specified attribute and value.
     *
     * This method currently uses a left-right recursive descent
     * search.
     *
     * @param node The starting node in the DOM tree.
     * @param name The attribute name.
     * @param value The attribute value.
     * @return The node of the first element with the specified
     * attribute and value, or null if none was found.
     */
    static Node findElementByAttribute (Node node, String name, String value)
    {
				// If this node matches, return it.
	if (node.getNodeType() == Node.ELEMENT_NODE &&
	    value.equals(((Element)node).getAttribute(name))) {
	    return node;
	}

				// If there are children, walk through them
	else if (node.hasChildNodes()) {
	    NodeList children = node.getChildNodes();
	    int nChildren = children.getLength();
	    for (int i = 0; i < nChildren; i++) {
		Node retval =
		    findElementByAttribute(children.item(i), name, value);
		if (retval != null)
		    return retval;
	    }
	}

				// OK, not found.
	return null;
    }


    /**
     * Remove a node from the DOM tree.
     */
    static void removeNode (Node node)
	throws DOMException
    {
	Node parent = node.getParentNode().removeChild(node);
    }


    /**
     * Write a node and its children as XML.
     *
     * @param node The DOM node to write.
     * @param output The output character stream.
     * @param isDocument If true, include a document prolog.
     */
    static void writeNode (Node node, Writer output, boolean isDocument,
			   String encoding, String internalSubset)
	throws IOException
    {
				// FIXME: do something about attributes
	if (isDocument) {
	    if (encoding == null)
		output.write("<?xml version=\"1.0\"?>\n\n");
	    else
		output.write("<?xml version=\"1.0\" encoding=\"" +
			     encoding +
			     "\"?>\n\n");
	    DocumentType doctype = node.getOwnerDocument().getDoctype();
	    if (doctype != null) {
		String pubid = doctype.getPublicId();
		String sysid = doctype.getSystemId();
		output.write("<!DOCTYPE ");
		output.write(node.getNodeName());
		if (pubid != null) {
		    output.write(" PUBLIC \"");
		    output.write(pubid);
		    if (sysid != null) {
			output.write("\" \"");
			output.write(sysid);
		    }
		    output.write('"');
		} else if (sysid != null) {
		    output.write(" SYSTEM \"");
		    output.write(sysid);
		    output.write('"');
		}
		String subset = internalSubset;
		if (subset == null)
		    subset = doctype.getInternalSubset();
		if (subset != null) {
		    output.write(" [");
		    output.write(subset);
		    output.write(']');
		}
		output.write(">\n\n");
	    }
	}
	writeNode(node, output);

	if (isDocument)
	    output.write("\n");
    }


    /**
     * Write out a single node recursively.
     *
     * @param node The DOM node to write.
     * @param output The output character stream.
     */
    private static void writeNode (Node node, Writer output)
	throws IOException
    {
	int type = node.getNodeType();
	switch (type) {
	case Node.ATTRIBUTE_NODE:
	    output.write(' ');
	    output.write(node.getNodeName());
	    output.write("=\"");
	    writeXMLData(node.getNodeValue(), true, output);
	    output.write('"');
	    break;
	case Node.CDATA_SECTION_NODE:
	case Node.TEXT_NODE:
	    // NewsML does not preserve CDATA section boundaries
	    writeXMLData(node.getNodeValue(), false, output);
	    break;
	case Node.COMMENT_NODE:
	    // NewsML does not preserve comments
	    break;
	case Node.DOCUMENT_FRAGMENT_NODE:
	    // NewsML does not preserve DocumentFragment boundaries
	    writeNodes(node.getChildNodes(), output);
	    break;
	case Node.DOCUMENT_NODE:
	    // Nothing special about the document here.
	    writeNodes(node.getChildNodes(), output);
	    break;
	case Node.DOCUMENT_TYPE_NODE:
	    // This is handled elsewhere.
	    break;
	case Node.ELEMENT_NODE: {
	    NamedNodeMap atts = node.getAttributes();

	    output.write('<');
	    output.write(node.getNodeName());
	    if (atts != null) {
		int length = atts.getLength();
		for (int i = 0; i < length; i++)
		    writeNode(atts.item(i), output);
	    }
	    
	    if (node.hasChildNodes()) {
		output.write('>');
		writeNodes(node.getChildNodes(), output);
		output.write("</");
		output.write(node.getNodeName());
		output.write('>');
	    } else {
		output.write("/>");
	    }
	    break;
	}
	case Node.ENTITY_NODE:
	    // NewsML does not preserve entity declarations.
	    break;
	case Node.ENTITY_REFERENCE_NODE:
	    // NewsML does not preserve entity boundaries.
	    writeNodes(node.getChildNodes(), output);
	    break;
	case Node.NOTATION_NODE:
	    // NewsML does not preserve notation declarations.
	    break;
	case Node.PROCESSING_INSTRUCTION_NODE:
	    // NewsML does not preserve processing instructions.
	    break;
	default:
	    throw new Error("Unexpected DOM node type: " + type);
	}
    }


    /**
     * Write out all the nodes in a list.
     *
     * @param nodes The DOM node list.
     * @param output The output character stream.
     */
    private static void writeNodes (NodeList nodes, Writer output)
	throws IOException
    {
	int length = nodes.getLength();
	for (int i = 0; i < length; i++)
	    writeNode(nodes.item(i), output);
    }


    /**
     * Escape XML data characters.
     *
     * @param data The string to escape.
     * @param isLiteral True if the string appears in an attribute
     * value literal.
     * @param output The output character stream.
     */
    private static void writeXMLData (String data, boolean isLiteral,
				      Writer output)
	throws IOException
    {
	char ch[] = data.toCharArray();
	for (int i = 0; i < ch.length; i++) {
	    switch (ch[i]) {
	    case '&':
		output.write("&amp;");
		break;
	    case '<':
		output.write("&lt;");
		break;
	    case '>':
		output.write("&gt;");
		break;
	    case '"':
		if (isLiteral) {
		    output.write("&quot;");
		    break;
		} // else fall through...
	    default:
		if (ch[i] >= 128) {
		    output.write("&#");
		    output.write(new Integer(ch[i]).toString());
		    output.write(';');
		} else if (ch[i] < 32 && ch[i] != '\n'
			   && ch[i] != '\r' && ch[i] != '\t') {
		    throw new IOException("Illegal XML data character " +
					  new Integer(ch[i]).toString());
		} else {
		    output.write(ch[i]);
		}
		break;
	    }
	}
    }

}

// end of DOMUtils.java
