// DOMXPath.java - a compiled XPath expression for DOM node matching.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMXPath.java,v 1.9 2001/11/26 16:33:37 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.util.List;
import java.util.Vector;

import org.w3c.dom.Node;

import org.jaxen.JaxenException;
import org.jaxen.NamespaceContext;
import org.jaxen.dom.XPath;

import org.saxpath.SAXPathException;

import org.newsml.toolkit.NewsMLException;


/**
 * A compiled XPath expression for DOM node matching.
 *
 * <p>This class currently supports only trivially simple XPath
 * expressions, but it can grow over time into a full XPath library.
 * The class does not depend on anything else in this package, so some
 * day it can spin off into a separate library.</p>
 *
 * <p>The current implementation of this class requires Jaxen 1.0beta4
 * or higher, available from www.jaxen.org.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
class DOMXPath
{
  
    /**
     * Constructor.
     *
     * <p>This constructor compiles its argument (which must be an
     * XPath expression) for future matching.</p>
     *
     * @param path An XPath expression as a string.
     * @exception NewsMLException if the XPath expression is invalid.
     */
    DOMXPath (String path)
	throws NewsMLException
    {
	try {
	    xpath = new XPath(path);
	    xpath.setNamespaceContext(new NSContext());
	} catch (SAXPathException e) {
	    throw new NewsMLException(e);
	}
    }


    /**
     * Return all nodes matching a path.
     *
     * @param context The DOM context node.
     * @return An possibly-empty array of matching nodes.
     */
    public Node [] select (Node context)
	throws NewsMLException
    {
	try {
	    List result = xpath.selectNodes(context);
	    int length = result.size();
	    Node nodes[] = new Node[length];
	    for (int i = 0; i < length; i++)
		nodes[i] = (Node)result.get(i);
	    return nodes;
	} catch (JaxenException e) {
	    throw new NewsMLException(e);
	}
    }


    /**
     * Test whether a DOM node matches an XPath expression.
     *
     * <p>This method tests whether an the current XPath expression
     * (as supplied to the constructor) matches the current DOM node.
     * The current node is assumed to appear at the end of the
     * path.</p>
     *
     * <p>Currently, this method supports only trivially simple XPath
     * expressions; in particular, it matches only a series of simple
     * element names separated by slashes, and does not recognize a
     * leading slash as the document root.</p>
     *
     * @param node The DOM node to match.
     * @return true if the node matches the expression, false
     * otherwise.
     */    
    public boolean matches (Node context, Node target)
	throws NewsMLException
    {
	try {
	    List result = xpath.selectNodes(context);
	    return result.contains(target);
	} catch (JaxenException e) {
	    throw new NewsMLException(e);
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private XPath xpath;



    ////////////////////////////////////////////////////////////////////
    // Inner class for resolving namespace prefixes.
    ////////////////////////////////////////////////////////////////////

    class NSContext
	implements NamespaceContext
    {
	public String translateNamespacePrefixToUri (String prefix)
	{
	    if ("xml".equals(prefix))
		return "http://www.w3.org/XML/1998/namespace";
	    else
		return null;
	}
    }

}

// end of DOMXPath.java
