// XercesDOMFactory.java - Xerces implementation of DOMFactory.
// Copyright (c) 2001 by Reuters PLC.
// $Id: XercesDOMFactory.java,v 1.7 2001/11/26 16:33:38 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.Reader;

import org.apache.xerces.parsers.DOMParser;

import org.w3c.dom.Document;

import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.newsml.toolkit.NewsMLException;


/**
 * Xerces implementation of DOMFactory.
 *
 * @author Reuters PLC
 * @version 1.1beta
 */
public class XercesDOMFactory
    implements DOMFactory, EntityResolver, ErrorHandler
{


    ////////////////////////////////////////////////////////////////////
    // Constructor.
    ////////////////////////////////////////////////////////////////////


    /**
     * Default constructor.
     *
     * <p>Construct a new, non-validating DOM factory.</p>
     */
    public XercesDOMFactory ()
    {
	validation = false;
    }


    /**
     * Constructor.
     *
     * <p>Construct a new DOM factory with the requested validation
     * type.</p>
     *
     * @param validation true to require DTD validation, false to
     * forbid it.
     * @see DOMFactory#setValidation
     */
    public XercesDOMFactory (boolean validation)
    {
	this.validation = validation;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of org.newsml.toolkit.dom.DOMFactory.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see DOMFactory#setValidation(boolean)
     */
    public void setValidation (boolean validation)
    {
	this.validation = validation;
    }


    /**
     * @see DOMFactory#getValidation()
     */
    public boolean getValidation ()
    {
	return validation;
    }


    /**
     * @see DOMFactory#createDOM(String)
     */
    public Document createDOM (String url)
	throws IOException
    {
	DOMParser parser;

	try {
	    parser = makeParser();
	    parser.parse(url);
	} catch (SAXException ex) {
	    throw new NewsMLException(ex);
	}

	return parser.getDocument();
    }



    /**
     * @see DOMFactory#createDOM(Reader,String)
     */
    public Document createDOM (Reader reader, String baseURL)
	throws IOException
    {
	DOMParser parser;

	try {
	    InputSource source = new InputSource();
	    source.setCharacterStream(reader);
	    source.setSystemId(baseURL);
	    parser = makeParser();
	    parser.parse(source);
	} catch (SAXException ex) {
	    throw new NewsMLException(ex);
	}

	return parser.getDocument();
    }




    ////////////////////////////////////////////////////////////////////
    // Implementation of org.xml.sax.EntityResolver.
    ////////////////////////////////////////////////////////////////////

    /**
     * Entity resolver to ignore external entities (including DTD).
     *
     * This method is used by {@link #createDOM} so that the
     * Xerces XML parser will not attempt to load the external
     * DTD subset; it may not be necessary for other DOM
     * libraries.
     *
     * @param publicId The public identifier (ignored).
     * @param systemId The system identifier (ignored).
     * @return An InputSource containing an empty byte stream.
     */
    public InputSource resolveEntity (String publicId, String systemId)
	throws IOException
    {
	return new InputSource(new ByteArrayInputStream(new byte[0]));
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of org.xml.sax.ErrorHandler.
    ////////////////////////////////////////////////////////////////////

    /**
     * Catch warnings.
     *
     * @param e A SAXParseException describing the warning.
     */
    public void warning (SAXParseException e)
    {
	System.err.println("XML WARNING: " +
			   e.getMessage() + " at " +
			   e.getSystemId() + ", " +
			   e.getLineNumber() + ':' +
			   e.getColumnNumber());
    }


    /**
     * Catch validation errors.
     *
     * @param e A SAXParseException describing the validation error.
     * @exception Always throws the exception on back.
     */
    public void error (SAXParseException e)
	throws SAXParseException
    {
	throw e;
    }


    /**
     * Catch fatal errors.
     *
     * @param e A SAXParseException describing the fatal error.
     * @exception Always throws the exception on back.
     */
    public void fatalError (SAXParseException e)
	throws SAXParseException
    {
	throw e;
    }



    ////////////////////////////////////////////////////////////////////
    // Internal methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Construct a Xerces parser with the appropriate validation value.
     *
     * @return A new Xerces DOMParser implementation.
     * @exception SAXException If there is a problem setting up the
     * parser (such as a SAXNotSupportedException).
     */
    private DOMParser makeParser ()
	throws SAXException
    {
	DOMParser parser = new DOMParser();
	parser.setFeature("http://xml.org/sax/features/validation",
			  validation);

				// extra step, to keep from loading
				// any external DTD subsets
	if (!validation)
	    parser.setEntityResolver(this);

				// make sure we catch validation errors
	parser.setErrorHandler(this);

	return parser;
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    boolean validation;

}

// end of XercesDOMFactory.java
