// BaseNodeTest.java - unit tests for BaseNode

package org.newsml.toolkit.dom.unittests;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import junit.framework.TestCase;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsItem;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLSession;


public class BaseNodeTest extends TestCase
{

    public BaseNodeTest (String name)
    {
	super(name);
    }

    protected void setUp ()
	throws Exception
    {
	super.setUp();
	sample = SampleFactory.createNewsML().getNewsItem(0)
	    .getRootNewsComponent();
	sample2 = ((NewsComponent)sample).getBasisForChoice(1).getDuid();
    }

    public void testXMLName ()
    {
	assertEquals(sample.getXMLName(), "NewsComponent");
    }

    public void testXPath ()
    {
	assertEquals(sample.getXPath(),
		     "/NewsML/NewsItem[0]/NewsComponent[0]");
	assertEquals(sample2.getXPath(),
		     "/NewsML/NewsItem[0]/NewsComponent[0]/BasisForChoice[1]/@Duid");
    }

    public void testSession ()
    {
	NewsMLSession session = sample.getSession();
	assertNotNull(session);
    }

    public void testParent ()
    {
	assertNotNull(sample.getParent());
	assertEquals(sample.getParent().getXMLName(), "NewsItem");
	assertNotNull(sample2.getParent());
	assertEquals(sample2.getParent().getXMLName(), "BasisForChoice");
    }

    public void testChildCount ()
    {
	assertEquals(sample.getChildCount(), 16);
	assertEquals(sample.getChildCount("TopicSet"), 2);
	assertEquals(sample.getChildCount("BasisForChoice"), 3);
    }

    public void testChild ()
    {
	assertEquals(sample.getChild(0).getXMLName(), "Comment");
	assertEquals(sample.getChild(5).getXMLName(), "Role");
	assertNull(sample.getChild(16));
	assertEquals(sample.getChild("Comment", 1).getXMLName(), "Comment");
	assertEquals(sample.getChild("NewsComponent", 0).getXMLName(),
		     "NewsComponent");
	assertNull(sample.getChild("NewsComponent", 2));
    }

    public void testRemoveSelf ()
	throws IOException
    {
	NewsComponent sample2 = SampleFactory.createNewsML(true).getNewsItem(0)
	    .getRootNewsComponent();
	BaseNode parent = sample2.getParent();
	int nchildren1 = parent.getChildCount();
	sample2.removeSelf();
	int nchildren2 = parent.getChildCount();
	assert("Parent has fewer children", nchildren2 == nchildren1 - 1);
	assertNull(((NewsItem)parent).getRootNewsComponent());

				// Now, try removing the root element
				// (should fail)
	boolean sawException = false;
	NewsML newsml = SampleFactory.createNewsML(true);
	try {
				// make sure we have the right one.
	    newsml.removeSelf();
	} catch (NewsMLException e) {
	    sawException = true;
	}
	assert("Exception removing root element", sawException);
    }

    public void testPath ()
    {
	BaseNode path[] = sample.getPath();
	assertEquals(path.length, 3);
	assertEquals(path[0].getXMLName(), "NewsML");
	assertEquals(path[1].getXMLName(), "NewsItem");
	assertEquals(path[2].getXMLName(), "NewsComponent");
    }

    public void testXML ()
    {
	String xml = sample.toXML(false);
	assert("Starts with open tag",
	       xml.startsWith("<NewsComponent"));
	assert("Ends with close tag",
	       xml.endsWith("</NewsComponent>"));
    }

    public void testWriteXMLFragment ()
	throws IOException
    {
	Writer output = new StringWriter();

	sample.writeXML(output, false);
	assertTrue("Starts with open tag",
		   output.toString().startsWith("<NewsComponent"));
	assertTrue("Ends with close tag",
		   output.toString().endsWith("</NewsComponent>"));
    }

    public void testWriteXMLDocument ()
	throws IOException
    {
	Writer output = new StringWriter();
	sample.writeXML(output, true);
	assertTrue("Has DOCTYPE NewsComponent",
		   output.toString().indexOf("<!DOCTYPE NewsComponent") != -1);
	assertTrue("Starts with XML declaration",
		   output.toString().startsWith("<?xml"));
	assertTrue("Contains start tag",
		   output.toString().indexOf("<NewsComponent") != -1);
	assertTrue("Ends with close tag and newline",
		   output.toString().endsWith("</NewsComponent>\n"));

	output = new StringWriter();
	sample.writeXML(output, "UTF-8", "<!ENTITY foo 'bar'>");
	assertTrue("Has UTF-8 encoding",
		   output.toString()
		   .startsWith("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
	assertTrue("Has internal subset",
		   output.toString().indexOf("<!ENTITY foo 'bar'>") > 0);
    }

    private BaseNode sample;
    private BaseNode sample2;

}

// end of BaseNodeTest.java
