// ExplorerFrame.java - frame for viewing a NewsML package.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: ExplorerFrame.java,v 1.7 2001/11/26 16:33:38 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.explorer;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;

import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import org.w3c.dom.Document;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.ContentItem;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsItem;
import org.newsml.toolkit.NewsMLFactory;
import org.newsml.toolkit.NewsML;


/**
 * Application for viewing a NewsML package interactively.
 *
 * <p>This class sets up a frame containing a menu bar with commands
 * linked to an {@link ExplorerPanel}.  It is a nearly complete
 * application, but requires that a {@link
 * org.newsml.toolkit.dom.DOMFactory} be provided, usually in the
 * application's static <var>main</var> method, so that it can create
 * DOM trees (and thus, NewsML packages) from URL strings.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see ExplorerPanel
 */
public class ExplorerFrame extends JFrame
    implements ActionListener
{


    ////////////////////////////////////////////////////////////////////
    // Constructors.
    ////////////////////////////////////////////////////////////////////


    /**
     * Create a new, empty frame with the default title.
     *
     * @param DOMFactory A factory for create a DOM tree from a URL.
     */
    public ExplorerFrame (NewsMLFactory factory)
    {
	init("NewsML Explorer", null, factory);
    }


    /**
     * Create a new, empty frame with the specified title.
     *
     * @param title The title for the frame.
     * @param DOMFactory A factory for create a DOM tree from a URL.
     */
    public ExplorerFrame (String title, NewsMLFactory factory)
    {
	init(title, null, factory);
    }


    /**
     * Create a new frame exploring the NewsML package provided.
     *
     * @param newsml The NewsML package to explore.
     * @param DOMFactory A factory for create a DOM tree from a URL.
     */
    public ExplorerFrame (NewsML newsml, NewsMLFactory factory)
    {
	init("NewsML Explorer", newsml, factory);
    }


    /**
     * Create a new frame with an explicit title and NewsML package.
     *
     * @param title The title for the frame.
     * @param newsml The NewsML package to explore.
     * @param DOMFactory A factory for create a DOM tree from a URL.
     */
    public ExplorerFrame (String title, NewsML newsml, NewsMLFactory factory)
    {
	init(title, newsml, factory);
    }


    /**
     * Internal initialization method.
     *
     * @param title The frame title.
     * @param newsml The NewsML package to explore.
     * @param DOMFactory A factory for create a DOM tree from a URL.
     */
    private void init (String title, NewsML newsml, NewsMLFactory factory)
    {
	preferredSize = new Dimension(800, 600);
	newsmlFactory = factory;
	setTitle(title);
	setJMenuBar(makeMenuBar());
	if (newsml != null)
	    setExplorerPanel(new ExplorerPanel(newsml));
	fileChooserPath = System.getProperty("user.dir");
    }



    ////////////////////////////////////////////////////////////////////
    // Accessors and setters.
    ////////////////////////////////////////////////////////////////////


    /**
     * Get the current NewsMLFactory.
     *
     * @return The current NewsMLFactory.
     */
    public NewsMLFactory getNewsMLFactory ()
    {
	return newsmlFactory;
    }


    /**
     * Set a new NewsMLFactory.
     *
     * @param domFactory The new NewsML factory.
     */
    public void setNewsMLFactory (NewsMLFactory factory)
    {
    }


    /**
     * Get the preferred size for this window.
     *
     * @return The current preferred size for the window (800x600
     * by default).
     */
    public Dimension getPreferredSize ()
    {
	return preferredSize;
    }


    /**
     * Set the preferred size for this window.
     *
     * @param preferredSize The new preferred size for the window.
     */
    public void setPreferredSize (Dimension preferredSize)
    {
	this.preferredSize = preferredSize;
    }


    /**
     * Get the current explorer panel for this application.
     *
     * @return The current explorer panel.
     */
    public ExplorerPanel getExplorerPanel ()
    {
	return explorerPanel;
    }


    /**
     * Change the explorer panel.
     *
     * @param explorerPanel The new explorer panel.
     */
    public void setExplorerPanel (ExplorerPanel explorerPanel)
    {
	if (this.explorerPanel != null)
	    getContentPane().remove(this.explorerPanel);
	this.explorerPanel = explorerPanel;
	getContentPane().add(this.explorerPanel);
	validate();
    }



    ////////////////////////////////////////////////////////////////////
    // Actions.
    ////////////////////////////////////////////////////////////////////


    /**
     * Load a NewsML package.
     *
     * @param file The new file to load.
     */
    public void readNewsML (File file)
    {
	readNewsML("file:" + file.getAbsolutePath());
    }


    /**
     * Load a NewsML package.
     *
     * @param url The new URL to load.
     */
    public void readNewsML (String url)
    {
	NewsML newsml = null;
	try {
	    newsml = newsmlFactory.createNewsML(url);
	} catch (IOException e1) {
	    alert("I/O Error", url + ":\n" + e1.getMessage());
	}

	if (newsml != null) {
	    setTitle(url);
	    setExplorerPanel(new ExplorerPanel(newsml));
	    explorerPanel.setDividerLocation(0.67); // kludge
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of java.awt.event.ActionListener.
    ////////////////////////////////////////////////////////////////////

    /**
     * Handle an action for the file menu.
     *
     * This method allows the class to respond to menu events;
     * it is not meant for other uses.
     *
     * @param command The action command for the menu item.
     */
    public void actionPerformed (ActionEvent e)
    {
	String command = e.getActionCommand();

	if ("File/Open File".equals(command)) {
	    JFileChooser chooser = new JFileChooser(fileChooserPath);
	    chooser.setDialogTitle("Open NewsML File");
	    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
		File file = chooser.getSelectedFile();
		readNewsML(file);
	    }
	    fileChooserPath = chooser.getCurrentDirectory().getAbsolutePath();
	}

	else if ("File/Open URL".equals(command)) {
				// pad with spaces to get a wider dialog
	    String url = JOptionPane
		.showInputDialog(this,
				 "Enter new URL:                                                                  ");
	    if (url != null)
		readNewsML(url);
	}

	else if ("File/About".equals(command)) {
	    alert("About", "NewsML Explorer 1.1beta\n" +
		  "Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.");
	}

	else if ("File/Quit".equals(command)) {
	    if (confirm("Really Quit?"))
		System.exit(0);
	} 

	else if ("Edit/Show All".equals(command)) {
	    explorerPanel.showAll();
	}

	else if ("Edit/Hide All".equals(command)) {
	    explorerPanel.hideAll();
	}

	else if ("Edit/Highlight/Language".equals(command)) {
	    String language = JOptionPane.showInputDialog(this, "Language:");
	    if (language != null) {
		LanguageSearch predicate = new LanguageSearch(language);
		explorerPanel.highlight(predicate);
	    }
	}

	else if ("Edit/Highlight/Role".equals(command)) {
	    String role = JOptionPane.showInputDialog(this, "Role:");
	    if (role != null) {
		RoleSearch predicate = new RoleSearch(role);
		explorerPanel.highlight(predicate);
	    }
	}

	else if ("Edit/Highlight/MediaType".equals(command)) {
	    String mediaType = JOptionPane.showInputDialog(this, "MediaType:");
	    if (mediaType != null) {
		MediaTypeSearch predicate = new MediaTypeSearch(mediaType);
		explorerPanel.highlight(predicate);
	    }
	}

	else if ("Edit/Highlight/Format".equals(command)) {
	    String format = JOptionPane.showInputDialog(this, "Format:");
	    if (format != null) {
		FormatSearch predicate = new FormatSearch(format);
		explorerPanel.highlight(predicate);
	    }
	}

	else {
	    notImplemented(command);
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal GUI methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Construct the menu bar for the application.
     *
     * @return A new JMenuBar, with listeners attached.
     */
    private JMenuBar makeMenuBar ()
    {
	JMenuBar menuBar = new JMenuBar();
	JMenu menu;
	JMenuItem menuItem;
	menu = new JMenu("File");

	menuItem = new JMenuItem("Open File");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("File/Open File");
	menu.add(menuItem);

	menuItem = new JMenuItem("Open URL");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("File/Open URL");
	menu.add(menuItem);

	menuItem = new JMenuItem("About");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("File/About");
	menu.add(menuItem);

	menu.add(new JSeparator());

	menuItem = new JMenuItem("Quit");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("File/Quit");
	menu.add(menuItem);

	menuBar.add(menu);

				// Edit menu.
	menu = new JMenu("Edit");

	menuItem = new JMenuItem("Show All");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Show All");
	menu.add(menuItem);

	menuItem = new JMenuItem("Hide All");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Hide All");
	menu.add(menuItem);

	menu.add(new JSeparator());

	JMenu menu2 = new JMenu("Highlight");
	menuItem = new JMenuItem("Language");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Highlight/Language");
	menu2.add(menuItem);
	menuItem = new JMenuItem("Role");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Highlight/Role");
	menu2.add(menuItem);
	menuItem = new JMenuItem("MediaType");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Highlight/MediaType");
	menu2.add(menuItem);
	menuItem = new JMenuItem("Format");
	menuItem.addActionListener(this);
	menuItem.setActionCommand("Edit/Highlight/Format");
	menu2.add(menuItem);
	
	menu.add(menu2);

	menuBar.add(menu);

				// Return the menu bar.
	return menuBar;
    }


    /**
     * Pop up a yes/no confirmation dialog.
     *
     * @param message The confirmation message.
     */
    private boolean confirm (String message)
    {
	int result =
	    JOptionPane.showConfirmDialog(this,
					  message,
					  "Confirm",
					  JOptionPane.YES_NO_OPTION);
	return (result == JOptionPane.YES_OPTION);
    }


    /**
     * Pop up a not-implemented dialog box.
     *
     * @param command The unimplemented command.
     */
    private void notImplemented (String command)
    {
	String message = "Command \"" + command + "\" not yet implemented.";
	JOptionPane.showMessageDialog(this, message);
    }


    /**
     * Pop up an alert message.
     *
     * @param message The alert message.
     */
    private void alert (String title, String message)
    {
	JOptionPane.showMessageDialog(this, message, title,
				      JOptionPane.ERROR_MESSAGE);
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private Dimension preferredSize;
    private NewsMLFactory newsmlFactory;
    private ExplorerPanel explorerPanel;
    private String fileChooserPath;



    ////////////////////////////////////////////////////////////////////
    // Inner classes.
    ////////////////////////////////////////////////////////////////////


    /**
     * Predicate for selecting ContentItems by media type.
     */
    class MediaTypeSearch implements BaseNodePredicate
    {
	MediaTypeSearch (String value)
	{
	    this.value = value;
	}

	public boolean accept (BaseNode node)
	{
	    return
		(node.getXMLName().equals("ContentItem") &&
		 ((ContentItem)node).getMediaType() != null &&
		 value.equals(((ContentItem)node).getMediaType().getName()));
	}

	private String value;
    }


    /**
     * Predicate for selecting ContentItems by format.
     */
    class FormatSearch implements BaseNodePredicate
    {
	FormatSearch (String value)
	{
	    this.value = value;
	}

	public boolean accept (BaseNode node)
	{
	    return
		(node.getXMLName().equals("ContentItem") &&
		 ((ContentItem)node).getFormat() != null &&
		 value.equals(((ContentItem)node).getFormat().getName()));
	}

	private String value;
    }


    /**
     * Predicate for selecting NewsComponents by role.
     */
    class RoleSearch implements BaseNodePredicate
    {
	RoleSearch (String value)
	{
	    this.value = value;
	}

	public boolean accept (BaseNode node)
	{
	    return
		(node.getXMLName().equals("NewsComponent") &&
		 ((NewsComponent)node).getRole() != null &&
		 value.equals(((NewsComponent)node).getRole().getName()));
	}

	private String value;
    }


    /**
     * Predicate for selecting NewsItems or NewsComponents by language.
     */
    class LanguageSearch implements BaseNodePredicate
    {
	LanguageSearch (String value)
	{
	    this.value = value;
	}

	public boolean accept (BaseNode node)
	{
	    String type = node.getXMLName();
	    if (type.equals("NewsItem"))
		return value.equals(((NewsItem)node).getLang());
	    else if (type.equals("NewsComponent"))
		return value.equals(((NewsComponent)node).getLang());
	    else
		return false;
	}

	private String value;
    }


}

// end of ExplorerFrame.java
