// ExplorerPanel.java - split pane for viewing a NewsML document.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: ExplorerPanel.java,v 1.8 2001/11/26 16:33:38 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.explorer;

import java.awt.Dimension;

import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import javax.swing.tree.TreePath;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsML;

import org.newsml.toolkit.BasisForChoice;
import org.newsml.toolkit.ContentItem;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.HeadLineGroup;
import org.newsml.toolkit.IdNode;
import org.newsml.toolkit.NewsEnvelope;
import org.newsml.toolkit.NewsItem;
import org.newsml.toolkit.NewsItemRef;
import org.newsml.toolkit.NewsLines;
import org.newsml.toolkit.Text;
import org.newsml.toolkit.TransmissionId;


/**
 * Split panel for viewing a NewsML package.
 *
 * <p>This class extends the Java Swing {@link JSplitPane} class to
 * provide a view of a NewsML package: the left side of the panel
 * contains a tree view, and the right side contains a tabular summary
 * of the tree node currently selected.
 *
 * <p>The {@link ExplorerFrame} class embeds an instance of this
 * class, and links the menu commands to public methods.  Application
 * developers can embed this ExplorerPanel objects directly in other
 * applications or applets (Java2 required) if they wish.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see ExplorerFrame
 */
public class ExplorerPanel extends JSplitPane
    implements TreeSelectionListener
{


    ////////////////////////////////////////////////////////////////////
    // Constructors.
    ////////////////////////////////////////////////////////////////////


    /**
     * Public constructor.
     *
     * @param newsml The NewsML package to explorer.
     */
    public ExplorerPanel (NewsML newsml)
    {
	init(newsml);
    }


    /**
     * Internal initialisation method.
     *
     * @param newsml The NewsML package to explore.
     */
    private void init (NewsML newsml)
    {
	this.newsml = newsml;
	setPreferredSize(new Dimension(800, 600));
	setOneTouchExpandable(true);


	//
	// Set up the tree.
	//
	tree = new JTree() {
		public String convertValueToText (Object value,
						  boolean selected,
						  boolean expanded,
						  boolean leaf,
						  int row,
						  boolean hasFocus)
		{
		    return value.toString();
		}
	    };
	tree.setModel(new NewsMLTreeModel(newsml));
	tree.addTreeSelectionListener(this);


	//
	// Set up the table.
	//

	tableModel = new NewsMLTableModel(newsml);
	table = new JTable(tableModel);


	//
	// Add the table and the tree.
	//

 	setLeftComponent(new JScrollPane(tree));
	setRightComponent(new JScrollPane(table));
    }



    ////////////////////////////////////////////////////////////////////
    // Accessors and setters.
    ////////////////////////////////////////////////////////////////////


    /**
     * Get the NewsML object being displayed.
     *
     * @return The NewsML package currently being explored.
     */
    public NewsML getNewsML ()
    {
	return newsml;
    }


    /**
     * Set the NewsML object to display.
     *
     * @param newsml A new NewsML package to explore.
     */
    public void setNewsML (NewsML newsml)
    {
	tree.setModel(new NewsMLTreeModel(newsml));
	tree.revalidate();
	table.setModel(new NewsMLTableModel(newsml));
	table.revalidate();
    }



    ////////////////////////////////////////////////////////////////////
    // Action methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Highlight nodes that match a predicate.
     *
     * @param predicate A predicate that will be tested against
     * every main node in the tree.
     */
    public void highlight (BaseNodePredicate predicate)
    {
	showAll();
	tree.clearSelection();
	for (int i = 0; i < tree.getRowCount(); i++) {
	    TreePath path = tree.getPathForRow(i);
	    BaseNode node = (BaseNode)(path.getLastPathComponent());
	    if (predicate.accept(node))
		tree.addSelectionRow(i);
	}
    }


    /**
     * Expand all of the nodes in the tree.
     *
     * @see #hideAll
     */
    public void showAll ()
    {
	for (int row = 0; row < tree.getRowCount(); row++)
	    tree.expandRow(row);
    }


    /**
     * Collapse all of the nodes in the tree.
     *
     * @see #showAll
     */
    public void hideAll ()
    {
	for (int row = 0; row < tree.getRowCount(); row++)
	    tree.collapseRow(row);
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of javax.swing.event.TreeSelectionListener.
    ////////////////////////////////////////////////////////////////////

    
    /**
     * Respond to user selections in the tree.
     *
     * This method should be invoked only by the Java library.
     *
     * @param e The event with the change information.
     */
    public void valueChanged (TreeSelectionEvent e)
    {
				// FIXME: deal with multiple
				// selections and deselection somehow
	Object node = e.getPath().getLastPathComponent();
	if (node instanceof BaseNode)
	    tableModel.setNode((BaseNode)node);
    }

    

    ////////////////////////////////////////////////////////////////////
    // Internal methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Make a label for a NewsML node in a tree.
     *
     * @param node The node to label.
     * @return A string label for the tree node.
     */
    private String makeLabel (BaseNode node)
    {
	String duid = "";
	if (node instanceof IdNode)
	    duid = ((IdNode)node).getDuid().toString();
	String type = node.getXMLName();
	if (type.equals("NewsML")) {
	    NewsEnvelope envelope = ((NewsML)node).getNewsEnvelope();
	    if (envelope != null) {
		TransmissionId id = envelope.getTransmissionId();
		if (id != null)
		    return "NewsML: " + id.toString();
	    }
	    return "NewsML";
	} else if (type.equals("NewsItem")) {
			    // FIXME: may crash on bad input
	    return "NewsItem: " +
		((NewsItem)node).getIdentification().getNewsIdentifier()
		.getPublicIdentifier();
	} else if (type.equals("NewsComponent")) {
	    NewsComponent component = (NewsComponent)node;
	    NewsLines lines = component.getNewsLines();
	    FormalName role = component.getRole();
	    BasisForChoice basisForChoice = component.getBasisForChoice(0);
	    StringBuffer text = new StringBuffer("NewsComponent:");
	    if (role != null) {
		text.append(" [");
		text.append(role.getName());
		text.append(']');
	    }
	    if (lines != null) {
		HeadLineGroup headline_group = lines.getHeadLineGroup(0);
		if (headline_group != null) {
		    text.append(' ');
		    text.append(headline_group.getHeadLine().toString());
		}
	    }
	    if (basisForChoice != null) {
		text.append(" {BasisForChoice=");
		text.append(basisForChoice.toString());
		text.append('}');
	    }
	    return text.toString();
	} else if (type.equals("NewsItemRef")) {
	    return "NewsItemRef: " +
		((NewsItemRef)node).getRef();
	} else if (type.equals("ContentItem")) {
	    StringBuffer label = new StringBuffer("ContentItem:");
	    if (((ContentItem)node).getMediaType() != null) {
		label.append(' ');
		label.append(((ContentItem)node).getMediaType().getName());
	    }
	    if (((ContentItem)node).getFormat() != null) {
		label.append(' ');
		label.append(((ContentItem)node).getFormat().getName());
	    }
	    return label.toString();
	} else {
	    return "Unknown!!";
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private NewsML newsml;
    private JTree tree;
    private JTable table;
    private NewsMLTableModel tableModel;

}

// end of ExplorerPanel.java
