// NewsMLTableModel.java - Swing table model for NewsML nodes.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsMLTableModel.java,v 1.9 2001/11/26 16:33:38 dmegginson Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.explorer;

import java.io.IOException;

import javax.swing.table.AbstractTableModel;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsItem;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsItemRef;
import org.newsml.toolkit.ContentItem;

import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.HeadLineGroup;
import org.newsml.toolkit.NewsEnvelope;
import org.newsml.toolkit.NewsLines;
import org.newsml.toolkit.NewsManagement;
import org.newsml.toolkit.PartyList;
import org.newsml.toolkit.StatusWillChange;
import org.newsml.toolkit.Text;


/**
 * NewsMLTableModel.java - Swing table model for NewsML nodes.
 *
 * <p>Simple class for populating a table summary for a NewsML node.</p>
 *
 * @author Reuters PLC
 * @version 1.1beta
 * @see ExplorerPanel
 */
class NewsMLTableModel extends AbstractTableModel
{

    /**
     * Constructor.
     *
     * @param node The NewsML node to display in the table.
     */
    NewsMLTableModel (BaseNode node)
    {
	this.node = node;
    }


    /**
     * Get the display node.
     *
     * @return The display node (not null).
     */
    BaseNode getNode ()
    {
	return node;
    }


    /**
     * Set a new display node.
     *
     * This method will also inform any listeners that the table
     * structure has changed.
     */
    void setNode (BaseNode node)
    {
	this.node = node;
	fireTableStructureChanged();
    }


    /**
     * Get the number of table rows.
     *
     * The number of rows will vary depending on the node type.  This
     * method dispatches the requested to the appropriate internal
     * method.
     *
     * @return The number of table rows for this node's summary.
     */
    public int getRowCount ()
    {
	String type = node.getXMLName();
	if (type.equals("NewsML")) {
	    return getNewsMLRowCount();
	} else if (type.equals("NewsItem")) {
	    return getNewsItemRowCount();
	} else if (type.equals("NewsComponent")) {
	    return getNewsComponentRowCount();
	} else if (type.equals("NewsItemRef")) {
	    return getNewsItemRefRowCount();
	} else if (type.equals("ContentItem")) {
	    return getContentItemRowCount();
	} else {
	    return 0;
	}
    }


    /**
     * Get the number of table columns.
     *
     * @return Always 2 (one column for property names, and one column
     * for property values).
     */
    public int getColumnCount ()
    {
	return 2;
    }


    /**
     * Get the name of a column.
     *
     * @param column The number of the column (should be 0 or 1).
     * @return The display name for the column.
     */
    public String getColumnName (int column)
    {
	switch (column) {
	case 0:
	    return "Property";
	case 1:
	    return "Value";
	}
	return "Unknown";
    }


    /**
     * Get a value for a table cell.
     *
     * The values vary depending on the NewsML node type.  This method
     * dispatches the request to the appropriate internal method.
     *
     * @param row The row number.
     * @param column The column number (should be 1 or 2).
     * @return A string representing the value (not null).
     */
    public Object getValueAt (int row, int column)
    {
	String type = node.getXMLName();
	if (type.equals("NewsML")) {
	    return getNewsMLValueAt(row, column);
	} else if (type.equals("NewsItem")) {
	    return getNewsItemValueAt(row, column);
	} else if (type.equals("NewsComponent")) {
	    return getNewsComponentValueAt(row, column);
	} else if (type.equals("NewsItemRef")) {
	    return getNewsItemRefValueAt(row, column);
	} else if (type.equals("ContentItem")) {
	    return getContentItemValueAt(row, column);
	} else {
	    return "Unknown!!";
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Accessor methods for specific types.
    ////////////////////////////////////////////////////////////////////


    private int getNewsMLRowCount ()
    {
	return 7;
    }

    private String getNewsMLValueAt (int row, int column)
    {
	if (column == 0) {
	    switch (row) {
	    case 0:
		return "Transmission Id";
	    case 1:
		return "Senders";
	    case 2:
		return "Recipients";
	    case 3:
		return "Date";
	    case 4:
		return "News Services";
	    case 5:
		return "News Products";
	    case 6:
		return "Priority";
	    }
	} else {
	    NewsEnvelope envelope = ((NewsML)node).getNewsEnvelope();
	    if (envelope == null)
		return "[none]";
	    switch (row) {
	    case 0:
		if (envelope.getTransmissionId() == null)
		    return "[none]";
		else
		    return envelope.getTransmissionId().toString();
	    case 1: {
		StringBuffer senders = new StringBuffer();
		PartyList sentFrom = envelope.getSentFrom();
		if (sentFrom != null) {
		    senders.append(makeString(sentFrom.getParty(0)));
		    for (int i = 1; sentFrom.getParty(i) != null; i++) {
			senders.append(' ');
			senders
			    .append(makeString(sentFrom.getParty(i)));
		    }
		}
		return senders.toString();
	    }
	    case 2: {
		StringBuffer recipients = new StringBuffer();
		PartyList sentTo = envelope.getSentTo();
		if (sentTo != null) {
		    recipients.append(makeString(sentTo.getParty(0)));
		    for (int i = 1; sentTo.getParty(i) != null; i++) {
			recipients.append(' ');
			recipients
			    .append(makeString(sentTo.getParty(i)));
		    }
		}
		return recipients.toString();
	    }
	    case 3:
		return makeString(envelope.getDateAndTime());
	    case 4: {
		StringBuffer services = new StringBuffer();
		services.append(makeString(envelope.getNewsService(0)));
		for (int i = 1; envelope.getNewsService(i) != null; i++) {
		    services.append(' ');
		    services.append(makeString(envelope.getNewsService(i)));
		}
		return services.toString();
	    }
	    case 5: {
		StringBuffer products = new StringBuffer();
		products.append(makeString(envelope.getNewsProduct(0)));
		for (int i = 1; envelope.getNewsProduct(i) != null; i++) {
		    products.append(' ');
		    products.append(makeString(envelope.getNewsProduct(i)));
		}
		return products.toString();
	    }
	    case 6:
		return makeString(envelope.getPriority());
	    }
	}
	return "Unknown";
    }

    private int getNewsItemRowCount ()
    {
	return 9;
    }

    private String getNewsItemValueAt (int row, int column)
    {
	if (column == 0) {
	    switch (row) {
	    case 0:
		return "News Identifier";
	    case 1:
		return "News Item Type";
	    case 2:
		return "First Created";
	    case 3:
		return "This Revision Created";
	    case 4:
		return "Status";
	    case 5:
		return "Status Will Change";
	    case 6:
		return "Urgency";
	    case 7:
		return "Revision History";
	    case 8:
		return "Language";
	    }
	} else {
	    NewsManagement management = ((NewsItem)node).getNewsManagement();
	    switch (row) {
	    case 0: {
		if (((NewsItem)node).getIdentification()
		    .getNewsIdentifier() == null)
		    return "[none]";
		else
		    return makeString(((NewsItem)node).getIdentification()
				      .getNewsIdentifier()
				      .getPublicIdentifier());
	    }
	    case 1:
		return makeString(management.getNewsItemType());
	    case 2:
		return makeString(management.getFirstCreated());
	    case 3:
		return makeString(management.getThisRevisionCreated());
	    case 4:
		return makeString(management.getStatus());
	    case 5: {
		StatusWillChange change = management.getStatusWillChange();
		if (change == null) {
		    return "[none]";
		} else {
		    return makeString(change.getDateAndTime()) + ": " +
			makeString(change.getFutureStatus());
		}
	    }
	    case 6:
		return makeString(management.getUrgency());
	    case 7:
		if (management.getRevisionHistory() != null)
		    return makeString(management.getRevisionHistory().getHref());
		else
		    return "[none]";
	    case 8:
		if (((NewsItem)node).getLang() == null)
		    return "[none]";
		else
		    return ((NewsItem)node).getLang().toString();
	    }
	}
	return "Unknown";
    }

    private int getNewsComponentRowCount ()
    {
	return 8;
    }

    private String getNewsComponentValueAt (int row, int column)
    {
	if (column == 0) {
	    switch (row) {
	    case 0:
		return "Essential?";
	    case 1:
		return "Equivalents List?";
	    case 2:
		return "Role";
	    case 3:
		return "First Basis for Choice";
	    case 4:
		return "First Headline";
	    case 5:
		return "First Byline";
	    case 6:
		return "First Dateline";
	    case 7:
		return "Language";
	    }
	} else {
	    NewsComponent component = (NewsComponent)node;
	    NewsLines lines = component.getNewsLines();
	    switch (row) {
	    case 0:
		return makeString(component.getEssential());
	    case 1:
		return makeString(component.getEquivalentsList());
	    case 2:
		return makeString(component.getRole());
	    case 3: {
		if (component.getBasisForChoice(0) == null)
		    return "[none]";
		else
		    return
			makeString(component.getBasisForChoice(0).toString());
	    }
	    case 4: {
		if (lines == null) {
		    return "[none]";
		} else {
		    HeadLineGroup group = lines.getHeadLineGroup(0);
		    if (group == null)
			return "[none]";
		    else
			return makeString(group.getHeadLine().toString());
		}
	    }
	    case 5:
		if (lines == null)
		    return "[none]";
		else
		    return makeString(lines.getByLine(0));
	    case 6:
		if (lines == null)
		    return "[none]";
		else
		    return makeString(lines.getDateLine(0));
	    case 7:
		if (component.getLang() == null)
		    return "[none]";
		else
		    return component.getLang().toString();
	    }
	}
	return "Unknown";
    }

    private int getNewsItemRefRowCount ()
    {
	return getNewsItemRowCount() + 1;
    }

    private String getNewsItemRefValueAt (int row, int column)
    {
	if (column == 0) {
	    switch (row) {
	    case 0:
		return "Target NewsItem";
	    case 1:
		return "News Identifier";
	    case 2:
		return "News Item Type";
	    case 3:
		return "First Created";
	    case 4:
		return "This Revision Created";
	    case 5:
		return "Status";
	    case 6:
		return "Status Will Change";
	    case 7:
		return "Urgency";
	    case 8:
		return "Revision History";
	    case 9:
		return "Language";
	    }
	} else {
				// Set up initial values
	    NewsItem item;
	    NewsManagement management;
	    try {
		item = ((NewsItemRef)node).getReferencedNewsItem(true);
		management = item.getNewsManagement();
	    } catch (IOException ex) {
		item = null;
		management = null;
	    }

	    switch (row) {
	    case 0:
		return makeString(((NewsItemRef)node).getRef());
	    case 1:
		if (item == null || item.getIdentification()
		    .getNewsIdentifier() == null)
		    return "[none]";
		return makeString(item.getIdentification()
				  .getNewsIdentifier()
				  .getPublicIdentifier());
	    case 2:
		if (management == null)
		    return "[none]";
		return makeString(management.getNewsItemType());
	    case 3:
		if (management != null)
		    return makeString(management.getFirstCreated());
		else
		    return "[none]";
	    case 4:
		if (management == null)
		    return "[none]";
		return makeString(management.getThisRevisionCreated());
	    case 5:
		if (management == null)
		    return "[none]";
		return makeString(management.getStatus());
	    case 6: {
		if (management == null)
		    return "[none]";
		StatusWillChange change = management.getStatusWillChange();
		if (change == null)
		    return "[none]";
		return makeString(change.getDateAndTime()) + ": " +
		    makeString(change.getFutureStatus());
	    }
	    case 7:
		if (management == null)
		    return "[none]";
		return makeString(management.getUrgency());
	    case 8:
		if (management == null)
		    return "[none]";
		return makeString(management.getRevisionHistory().getHref());
	    case 9:
		if (item == null)
		    return "[none]";
		return makeString(item.getLang());
	    }
	}
	return "Unknown";
    }

    private int getContentItemRowCount ()
    {
	return 6;
    }

    private String getContentItemValueAt (int row, int column)
    {
	if (column == 0) {
	    switch (row) {
	    case 0:
		return "External Location";
	    case 1:
		return "Media Type";
	    case 2:
		return "Format";
	    case 3:
		return "MIME Type";
	    case 4:
		return "Notation";
	    case 5:
		return "Size In Bytes";
	    }
	} else {
	    ContentItem item = (ContentItem)node;
	    switch (row) {
	    case 0:
		return makeString(item.getHref());
	    case 1:
		return makeString(item.getMediaType());
	    case 2:
		return makeString(item.getFormat());
	    case 3:
		return makeString(item.getMimeType());
	    case 4:
		return makeString(item.getNotation());
	    case 5:
		if (item.getCharacteristics() != null) {
		    return
			makeString(item.getCharacteristics().getSizeInBytes());
		} else {
		    return "[unspecified]";
		}
	    }
	}
	return "Unknown";
    }



    ////////////////////////////////////////////////////////////////////
    // Utility methods.
    ////////////////////////////////////////////////////////////////////

    private String makeString (String string)
    {
	return (string == null ? "[none]" : string);
    }

    private String makeString (FormalName name)
    {
	if (name == null)
	    return "[none]";

	StringBuffer buf = new StringBuffer();
	String vocab;
	try {
	    vocab = name.getDefaultVocabulary(true);
	} catch (IOException ex) {
	    vocab = "{ERROR!!}";
	}
	if (vocab != null) {
	    buf.append('{');
	    buf.append(vocab);
	    buf.append('}');
	}
	buf.append(name.getName());
	return buf.toString();
    }

    private String makeString (boolean value)
    {
	return (value ? "yes" : "no");
    }

    private String makeString (int value)
    {
	return new Integer(value).toString();
    }

    private String makeString (Text text)
    {
	return text.toString();
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private BaseNode node;

}

// end of NewsMLTableModel.java
